/** 
* jQuery HzUI V0.0.2 
*/

+function ($) {
  'use strict';

  /**
   *
   * @param element，容器
   * @param options，配置表，包含以下：
   * * @param options.list，checkbox列表，{key: 1, value: '键1', checked: false, disabled: false}，必需
   * * @param options.checkedHandle，选中当前框的触发
   */
  var CheckBox = function (element, options) {
    this.options = options
    this.$element = $(element)

    init(this.$element, this.options)
  }

  // 获取已选中的数据
  CheckBox.prototype.getChecked = function () {
    var res = []
    var _this = this
    _this.$element.find('.hzui-checkbox.active').each(function(){
      var index = $(this).data('index')

      res.push(_this.options.list[index])
    })

    return res
  }

  /**
   * @description 手动操作checkBox的选中状态
   * @param keyArray，需要修改的键与状态：[{key: 1, checked: true}]
   */
  CheckBox.prototype.active = function (keyArray) {
    var _this = this
    keyArray.forEach(function (item) {
      var ele = _this.$element.find('.hzui-checkbox[data-key="' + item.key + '"]')

      if (item.checked) {
        ele.addClass('active')
        ele.find('.hzui-checkbox-icon').removeClass('icon-non-check')
        ele.find('.hzui-checkbox-icon').addClass('icon-check_focus')
      } else {
        ele.removeClass('active')
        ele.find('.hzui-checkbox-icon').addClass('icon-non-check')
        ele.find('.hzui-checkbox-icon').removeClass('icon-check_focus')
      }

      var index = ele.data('index')
      var option = _this.options.list[index]
      option.isChecked = item.checked

      if (option.checkedHandle && typeof option.checkedHandle == 'function') {
        option.checkedHandle(option, ele)
      }
    })
  }

  /**
   * @description 手动操作checkBox的禁用状态
   * @param keyArray，需要修改的键与状态：[{key: 1, disabled: true}]
   */
  CheckBox.prototype.disabled = function (keyArray) {
    var _this = this
    keyArray.forEach(function (item) {
      var ele = _this.$element.find('.hzui-checkbox[data-key="' + item.key + '"]')
      
      if (item.disabled) {
        ele.addClass('disabled')
      } else {
        ele.removeClass('disabled')
      }

      var index = ele.data('index')
      var option = _this.options.list[index]
      option.disabled = item.disabled
    })
  }

  function init ($element, options) {
    var html = ''
    for (var i = 0; i < options.list.length; i++) {
      var item = options.list[i]
      html += '<div class="hzui-checkbox'

      if (item.checked) {
        html += ' active'
      }

      if (item.disabled) {
        html += ' disabled'
      }

      html += '" data-index="' + i + '"' + ' data-key="' + item.key + '"' + '><i class="hzui-checkbox-icon icon iconfont'
      if (item.checked) {
        html += ' icon-check_focus'
      } else {
        html += ' icon-non-check'
      }

      html += '"></i>'
      html += '<span class="hzui-checkbox-label">' + item.value + '</span>'
      html += '</div>'
    }

    $element.append(html)

    checkBoxSelectHandle($element, options)
  }

  function checkBoxSelectHandle ($element, options) {
    $element.on('click', '.hzui-checkbox', function(){
      var index = $(this).index()
      var item = options.list[index]
      if (item.disabled) {
        return
      }

      if (!$(this).hasClass('active')) {
        $(this).addClass('active')
        $(this).find('.icon').removeClass('icon-non-check')
        $(this).find('.icon').addClass('icon-check_focus')
      } else {
        $(this).removeClass('active')
        $(this).find('.icon').addClass('icon-non-check')
        $(this).find('.icon').removeClass('icon-check_focus')
      }

      if (item.checkedHandle && typeof item.checkedHandle == 'function') {
        item.checkedHandle(item, $(this))
      }
    })
  }

  $.fn._checkBox = function(arg1, arg2, arg3) {
    var results = []

    this.each(function () {
      var checkBox = $(this).data('_checkBox')
      //初始化新的checkBox
      if (!checkBox) {
        checkBox = new CheckBox(this, arg1)
        $(this).data('_checkBox', checkBox)
        results.push(checkBox)
        $(this).val($(this).val())
      } else if (!arg1 && !arg2) {
        results.push(checkBox)
      } else if (checkBox[arg1] !== undefined) {
        var retVal = undefined
        if (checkBox[arg1].length === 3 && arg3 !== undefined) {
          retVal = checkBox[arg1](arg2, null, arg3)
        } else {
          retVal = checkBox[arg1](arg2)
        }
        if (retVal !== undefined)
          results.push(retVal)
      }
    });

    if (typeof arg1 == 'string') {
      return results.length > 1 ? results : results[0]
    } else {
      return results
    }
  }

  $.fn._checkBox.Constructor = CheckBox
}(jQuery)

+function ($) {
    'use strict';
  
    // input ie9 兼容
    // ======================
  
    $("[placeholder]")
    .focus(function() {
      var input = $(this);
      if (input.val() == input.attr("placeholder")) {
        input.val("");
        input.removeClass("placeholder");
      }
    })
    .blur(function() {
      var input = $(this);
      if (input.val() == "" || input.val() == input.attr("placeholder")) {
        input.addClass("placeholder");
        input.val(input.attr("placeholder"));
      }
    })
    .blur()
    .parents("form")
    .submit(function() {
      $(this).find("[placeholder]").each(function() {
        var input = $(this);
        if (input.val() == input.attr("placeholder")) {
          input.val("");
        }
      });
    });
  
  }(jQuery);
  
/* ========================================================================
 * Bootstrap: modal.js v3.3.7
 * http://getbootstrap.com/javascript/#modals
 * ========================================================================
 * Copyright 2011-2016 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // MODAL CLASS DEFINITION
  // ======================

  var Modal = function (element, options) {
    this.options             = options
    this.$body               = $(document.body)
    this.$element            = $(element)
    this.$dialog             = this.$element.find('.modal-dialog')
    this.$backdrop           = null
    this.isShown             = null
    this.originalBodyPad     = null
    this.scrollbarWidth      = 0
    this.ignoreBackdropClick = false

    if (this.options.remote) {
      this.$element
        .find('.modal-content')
        .load(this.options.remote, $.proxy(function () {
          this.$element.trigger('loaded.bs.modal')
        }, this))
    }
  }

  Modal.VERSION  = '3.3.7'

  Modal.TRANSITION_DURATION = 300
  Modal.BACKDROP_TRANSITION_DURATION = 150

  Modal.DEFAULTS = {
    backdrop: true,
    keyboard: true,
    show: true
  }

  Modal.prototype.toggle = function (_relatedTarget) {
    return this.isShown ? this.hide() : this.show(_relatedTarget)
  }

  Modal.prototype.show = function (_relatedTarget) {
    var that = this
    var e    = $.Event('show.bs.modal', { relatedTarget: _relatedTarget })

    this.$element.trigger(e)

    if (this.isShown || e.isDefaultPrevented()) return

    this.isShown = true

    this.checkScrollbar()
    this.setScrollbar()
    this.$body.addClass('modal-open')

    this.escape()
    this.resize()

    this.$element.on('click.dismiss.bs.modal', '[data-dismiss="modal"]', $.proxy(this.hide, this))

    this.$dialog.on('mousedown.dismiss.bs.modal', function () {
      that.$element.one('mouseup.dismiss.bs.modal', function (e) {
        if ($(e.target).is(that.$element)) that.ignoreBackdropClick = true
      })
    })

    this.backdrop(function () {
      var transition = $.support.transition && that.$element.hasClass('fade')

      if (!that.$element.parent().length) {
        that.$element.appendTo(that.$body) // don't move modals dom position
      }

      that.$element
        .show()
        .scrollTop(0)

      that.adjustDialog()

      if (transition) {
        that.$element[0].offsetWidth // force reflow
      }

      that.$element.addClass('in')

      that.enforceFocus()

      var e = $.Event('shown.bs.modal', { relatedTarget: _relatedTarget })

      transition ?
        that.$dialog // wait for modal to slide in
          .one('bsTransitionEnd', function () {
            that.$element.trigger('focus').trigger(e)
          })
          .emulateTransitionEnd(Modal.TRANSITION_DURATION) :
        that.$element.trigger('focus').trigger(e)
    })
  }

  Modal.prototype.hide = function (e) {
    if (e) e.preventDefault()

    e = $.Event('hide.bs.modal')

    this.$element.trigger(e)

    if (!this.isShown || e.isDefaultPrevented()) return

    this.isShown = false

    this.escape()
    this.resize()

    $(document).off('focusin.bs.modal')

    this.$element
      .removeClass('in')
      .off('click.dismiss.bs.modal')
      .off('mouseup.dismiss.bs.modal')

    this.$dialog.off('mousedown.dismiss.bs.modal')

    $.support.transition && this.$element.hasClass('fade') ?
      this.$element
        .one('bsTransitionEnd', $.proxy(this.hideModal, this))
        .emulateTransitionEnd(Modal.TRANSITION_DURATION) :
      this.hideModal()
  }

  Modal.prototype.enforceFocus = function () {
    $(document)
      .off('focusin.bs.modal') // guard against infinite focus loop
      .on('focusin.bs.modal', $.proxy(function (e) {
        if (document !== e.target &&
            this.$element[0] !== e.target &&
            !this.$element.has(e.target).length) {
          this.$element.trigger('focus')
        }
      }, this))
  }

  Modal.prototype.escape = function () {
    if (this.isShown && this.options.keyboard) {
      this.$element.on('keydown.dismiss.bs.modal', $.proxy(function (e) {
        e.which == 27 && this.hide()
      }, this))
    } else if (!this.isShown) {
      this.$element.off('keydown.dismiss.bs.modal')
    }
  }

  Modal.prototype.resize = function () {
    if (this.isShown) {
      $(window).on('resize.bs.modal', $.proxy(this.handleUpdate, this))
    } else {
      $(window).off('resize.bs.modal')
    }
  }

  Modal.prototype.hideModal = function () {
    var that = this
    this.$element.hide()
    this.backdrop(function () {
      that.$body.removeClass('modal-open')
      that.resetAdjustments()
      that.resetScrollbar()
      that.$element.trigger('hidden.bs.modal')
    })
  }

  Modal.prototype.removeBackdrop = function () {
    this.$backdrop && this.$backdrop.remove()
    this.$backdrop = null
  }

  Modal.prototype.backdrop = function (callback) {
    var that = this
    var animate = this.$element.hasClass('fade') ? 'fade' : ''

    if (this.isShown && this.options.backdrop) {
      var doAnimate = $.support.transition && animate

      this.$backdrop = $(document.createElement('div'))
        .addClass('modal-backdrop ' + animate)
        .appendTo(this.$body)

      this.$element.on('click.dismiss.bs.modal', $.proxy(function (e) {
        if (this.ignoreBackdropClick) {
          this.ignoreBackdropClick = false
          return
        }
        if (e.target !== e.currentTarget) return
        this.options.backdrop == 'static'
          ? this.$element[0].focus()
          : this.hide()
      }, this))

      if (doAnimate) this.$backdrop[0].offsetWidth // force reflow

      this.$backdrop.addClass('in')

      if (!callback) return

      doAnimate ?
        this.$backdrop
          .one('bsTransitionEnd', callback)
          .emulateTransitionEnd(Modal.BACKDROP_TRANSITION_DURATION) :
        callback()

    } else if (!this.isShown && this.$backdrop) {
      this.$backdrop.removeClass('in')

      var callbackRemove = function () {
        that.removeBackdrop()
        callback && callback()
      }
      $.support.transition && this.$element.hasClass('fade') ?
        this.$backdrop
          .one('bsTransitionEnd', callbackRemove)
          .emulateTransitionEnd(Modal.BACKDROP_TRANSITION_DURATION) :
        callbackRemove()

    } else if (callback) {
      callback()
    }
  }

  // these following methods are used to handle overflowing modals

  Modal.prototype.handleUpdate = function () {
    this.adjustDialog()
  }

  Modal.prototype.adjustDialog = function () {
    var modalIsOverflowing = this.$element[0].scrollHeight > document.documentElement.clientHeight

    this.$element.css({
      paddingLeft:  !this.bodyIsOverflowing && modalIsOverflowing ? this.scrollbarWidth : '',
      paddingRight: this.bodyIsOverflowing && !modalIsOverflowing ? this.scrollbarWidth : ''
    })
  }

  Modal.prototype.resetAdjustments = function () {
    this.$element.css({
      paddingLeft: '',
      paddingRight: ''
    })
  }

  Modal.prototype.checkScrollbar = function () {
    var fullWindowWidth = window.innerWidth
    if (!fullWindowWidth) { // workaround for missing window.innerWidth in IE8
      var documentElementRect = document.documentElement.getBoundingClientRect()
      fullWindowWidth = documentElementRect.right - Math.abs(documentElementRect.left)
    }
    this.bodyIsOverflowing = document.body.clientWidth < fullWindowWidth
    this.scrollbarWidth = this.measureScrollbar()
  }

  Modal.prototype.setScrollbar = function () {
    var bodyPad = parseInt((this.$body.css('padding-right') || 0), 10)
    this.originalBodyPad = document.body.style.paddingRight || ''
    if (this.bodyIsOverflowing) this.$body.css('padding-right', bodyPad + this.scrollbarWidth)
  }

  Modal.prototype.resetScrollbar = function () {
    this.$body.css('padding-right', this.originalBodyPad)
  }

  Modal.prototype.measureScrollbar = function () { // thx walsh
    var scrollDiv = document.createElement('div')
    scrollDiv.className = 'modal-scrollbar-measure'
    this.$body.append(scrollDiv)
    var scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth
    this.$body[0].removeChild(scrollDiv)
    return scrollbarWidth
  }


  // MODAL PLUGIN DEFINITION
  // =======================

  function Plugin(option, _relatedTarget) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.modal')
      var options = $.extend({}, Modal.DEFAULTS, $this.data(), typeof option == 'object' && option)

      if (!data) $this.data('bs.modal', (data = new Modal(this, options)))
      if (typeof option == 'string') data[option](_relatedTarget)
      else if (options.show) data.show(_relatedTarget)
    })
  }

  var old = $.fn.modal

  $.fn.modal             = Plugin
  $.fn.modal.Constructor = Modal


  // MODAL NO CONFLICT
  // =================

  $.fn.modal.noConflict = function () {
    $.fn.modal = old
    return this
  }


  // MODAL DATA-API
  // ==============

  $(document).on('click.bs.modal.data-api', '[data-toggle="modal"]', function (e) {
    var $this   = $(this)
    var href    = $this.attr('href')
    var $target = $($this.attr('data-target') || (href && href.replace(/.*(?=#[^\s]+$)/, ''))) // strip for ie7
    var option  = $target.data('bs.modal') ? 'toggle' : $.extend({ remote: !/#/.test(href) && href }, $target.data(), $this.data())

    if ($this.is('a')) e.preventDefault()

    $target.one('show.bs.modal', function (showEvent) {
      if (showEvent.isDefaultPrevented()) return // only register focus restorer if modal will actually get shown
      $target.one('hidden.bs.modal', function () {
        $this.is(':visible') && $this.trigger('focus')
      })
    })
    Plugin.call($target, option, this)
  })

}(jQuery);

/*!
 * Mricode Pagination Plugin
 * Github: https://github.com/momopig/Mricode.Pagination
 * Version: 1.4.4
 *
 * Required jQuery
 *
 * Copyright 2016 Mricle
 * Released under the MIT license
 * Modified By momopig
 */

(function (factory) {
  if (typeof define === 'function' && define.amd) {
      // AMD
      define(['jquery'], factory);
  } else if (typeof exports === 'object') {
      // Node / CommonJS
      factory(require('jquery'));
  } else {
      // Globals
      factory(jQuery);
  }
})(function ($) {
  "use strict";

  var Page = function (element, options) {
      var defaultOption = {
          pageIndex: 0,
          pageSize: 10,
          total: 0,
          pageBtnCount: 11,
          showFirstLastBtn: true,
          firstBtnText: null,
          lastBtnText: null,
          prevBtnText: "&laquo;",
          nextBtnText: "&raquo;",
          loadFirstPage: true,
          initSuccess: function () {
              if (this.debug && console) {
                  console.info('init success! just run once!');
              }
          },
          remote: {
              url: null,
              params: null,
              pageParams: null,
              success: null,
              beforeSend: null,
              complete: null,
              pageIndexName: 'pageIndex',
              pageSizeName: 'pageSize',
              totalName: 'total',
              traditional: false
          },
          pageElementSort: ['$page', '$size', '$jump', '$info'],
          showInfo: false,
          infoFormat: '{start} ~ {end} of {total} entires',
          noInfoText: '0 entires',
          showJump: false,
          jumpBtnText: 'Go',
          showPageSizes: false,
          pageSizeItems: [5, 10, 15, 20],
          debug: false
      }
      this.$element = $(element);
      this.$page = $('<ul class="m-pagination-page"></ul>').hide();
      this.$size = $('<div class="m-pagination-size"></div>').hide();
      this.$jump = $('<div class="m-pagination-jump"></div>').hide();
      this.$info = $('<div class="m-pagination-info"></div>').hide();
      this.options = $.extend(true, {}, defaultOption, $.fn.pagination.defaults, options);

      if (options.pageElementSort) {
          this.options.pageElementSort = options.pageElementSort
      } else {
          if ($.fn.pagination.defaults && $.fn.pagination.defaults.pageElementSort) {
              this.options.pageElementSort = $.fn.pagination.defaults.pageElementSort;
          } else {
              this.options.pageElementSort = defaultOption.pageElementSort;
          }
      }
      this.options.pageSizeItems = options.pageSizeItems || (($.fn.pagination.defaults && $.fn.pagination.defaults.pageSizeItems) ? $.fn.pagination.defaults.pageSizeItems : defaultOption.pageSizeItems);
      this.total = this.options.total;
      this.currentUrl = this.options.remote.url;
      this.currentPageIndex = utility.convertInt(this.options.pageIndex);
      this.currentPageSize = utility.convertInt(this.options.pageSize);
      this.currentParams = utility.deserializeParams(this.options.remote.params) || {};
      this.getLastPageNum = function () {
          return pagination.core.calLastPageNum(this.total, this.currentPageSize);
      }
      if (this.options.remote.success === null) {
          this.options.remote.success = this.options.remote.callback;
      }
      this.init();
  }

  Page.prototype = {
      getPageIndex: function () {
          return this.currentPageIndex;
      },
      getPageSize: function () {
          return this.currentPageSize;
      },
      getParams: function () {
          return this.currentParams;
      },
      setPageIndex: function (pageIndex) {
          if (pageIndex !== undefined && pageIndex !== null) {
              this.currentPageIndex = utility.convertInt(pageIndex);
          }
          this.renderPagination();
      },
      setPageSize: function (pageSize) {
          if (pageSize !== undefined && pageSize !== null) {
              this.currentPageSize = utility.convertInt(pageSize);
          }
          this.renderPagination();
      },
      setRemoteUrl: function (url) {
          if (url) {
              this.currentUrl = url;
          }
      },
      setTotal: function (total) {
          if (total !== undefined && total !== null) {
              this.total = utility.convertInt(total);
          }
          this.renderPagination();
      },
      setParams: function (params) {
          if (params) {
              this.currentParams = utility.deserializeParams(params);
          }
          this.renderPagination();
      },
      render: function (total) {
          if (total !== undefined && total !== null) {
              this.total = utility.convertInt(total);
          }
          this.renderPagination();
          this.debug('pagination rendered');
      },
      init: function () {
          this.initHtml();
          this.initEvent();
          if (this.currentUrl && this.options.loadFirstPage) {
              this.remote();
          } else {
              this.renderPagination();
          }
          this.debug('pagination inited');
          this.options.initSuccess();
      },
      initHtml: function () {
          //init size module
          var sizeHtml = $('<select data-page-btn="size"></select>');
          var list = '';
          for (var i = 0; i < this.options.pageSizeItems.length; i++) {
              sizeHtml.append('<option value="' + this.options.pageSizeItems[i] + '" ' + (this.currentPageSize == this.options.pageSizeItems[i] ? "selected" : "") + ' >' + this.options.pageSizeItems[i] + '</option>')
              //自定义样式切换pagesize下拉列表
              list += '<li><a href="javascript:;" key=' + this.options.pageSizeItems[i] + '>' + this.options.pageSizeItems[i] + '条/页' + '</a></li>'
          }
          sizeHtml.val(this.currentPageSize);
          this.$size.append(sizeHtml);

          this.$size.append('<div class="list-container"><ul>' + list + '</ul></div>');
          //自定义样式的pagesize框
          this.$size.append('<div class="size-container"><span class="label">' + this.currentPageSize + '条/页' + '</span><span class="iconfont icon-dropdown-copy"></span><span class="iconfont icon-pickup2"></span></div>');


          //init jump module
          //添加自定义样式label字段
          var prefix = '跳转 ';
          var endfix = ' 页';
          var jumpHtml = '<span class="jump-prefix">' + prefix + '</span><div class="m-pagination-group"><input data-page-btn="jump" type="text"><button data-page-btn="jump" type="button">' + this.options.jumpBtnText + '</button></div><span class="jump-endfix">' + endfix + '</span>';
          this.$jump.append(jumpHtml);
          this.$jump.find('input').val(1);
          for (var i = 0; i < this.options.pageElementSort.length; i++) {
              this.$element.append(this[this.options.pageElementSort[i]]);
          }
      },
      initEvent: function () {
          this.$element
              .on('click', {
                  page: this
              }, function (event) {
                  event.stopPropagation();
                  if ($(event.target).is('button')) {
                      if ($(event.target).data('pageBtn') == 'jump') {
                          var $input = event.data.page.$element.find('input[data-page-btn=jump]');
                          event.data.page.jumpEventHandler($input.val(), event);
                      }
                  } else {
                      if ($(event.target).data('pageIndex') !== undefined)
                          eventHandler.call(event.data.page, event);

                      //自定义样式pagesize下拉选择事件绑定
                      if ($(event.target).parent().attr('class') == 'size-container') {
                          var sizeContainer = $(event.target).parent();
                          var paginationSize = sizeContainer.parent();
                          var ul = paginationSize.find('ul');
                          if (!ul.is(':visible')) {
                              ul.slideDown("fast");
                              sizeContainer.find('.icon-dropdown-copy').hide();
                              sizeContainer.find('.icon-pickup2').css('display', 'inline-block');
                          } else {
                              ul.slideUp("fast");
                              sizeContainer.find('.icon-pickup2').hide();
                              sizeContainer.find('.icon-dropdown-copy').css('display', 'inline-block');
                          }
                      }
                      if ($(event.target).closest('div').attr('class') == 'list-container') {
                          var $this = $(event.target);
                          var $paginationSize=$this.closest('.m-pagination-size');
                          var txt = $this.text();
                          var key = $this.attr("key");
                          $paginationSize.find('.size-container .label').html(txt).attr('key', key);
                          $this.closest('ul').hide();
                          $paginationSize.find('.icon-pickup2').hide();
                          $paginationSize.find('.icon-dropdown-copy').css('display', 'inline-block');
                          var originSelect = $paginationSize.find('[data-page-btn="size"]');
                          originSelect.val(key);
                          originSelect.trigger('change');

                      }

                  }
              }).on('change', {
                  page: this
              }, function (event) {
                  var $this = $(event.target);
                  if ($this.data('pageBtn') == 'jump') {
                      event.data.page.jumpEventHandler($this.val(), event);
                  }
                  if ($this.data('pageBtn') == 'size') {
                      eventHandler.call(event.data.page, event);
                  }
              }).on('keypress', {
                  page: this
              }, function (event) {
                  if (event.keyCode == "13") {
                      var $input = event.data.page.$element.find('input[data-page-btn=jump]')
                      var pageIndex = $input.val();
                      event.data.page.jumpEventHandler($input.val(), event);
                      $input.val(pageIndex)
                  }
              });
          //点击空白处 隐藏pagesize下拉列表
          $(document).on('click', function (e) {
              if ($('.m-pagination .list-container ul').is(':visible')) {
                  var ul = $('.m-pagination .list-container ul');
                  var paginationSize = ul.closest('.m-pagination-size');
                  ul.hide();
                  paginationSize.find('.size-container .icon-pickup2').hide();
                  paginationSize.find('.size-container .icon-dropdown-copy').css('display', 'inline-block');
              }
          })
      },
      jumpEventHandler: function (inputValue, event) {
          if (!inputValue) {
              this.$jump.removeClass('error');
          } else if (!pagination.check.checkJumpPage(inputValue)) {
              this.$jump.addClass('error');
          } else if (utility.convertInt(inputValue) > this.getLastPageNum()) {
              this.$jump.addClass('error');
          } else {
              this.$jump.removeClass('error');
              eventHandler.call(this, event);
          }
      },

      doPagination: function () {
          if (this.currentUrl) {
              this.remote();
          } else {
              this.renderPagination();
          }
      },
      remote: function () {
          if (typeof this.options.remote.pageParams === 'function') {
              var pageParams = this.options.remote.pageParams.call(this, {
                  pageIndex: this.currentPageIndex,
                  pageSize: this.currentPageSize
              });
              this.currentParams = $.extend({}, this.currentParams, pageParams);
          } else {
              this.currentParams[this.options.remote.pageIndexName] = this.currentPageIndex;
              this.currentParams[this.options.remote.pageSizeName] = this.currentPageSize;
          }
          pagination.remote.getAjax(this, this.currentUrl, this.currentParams, this.ajaxCallBack, this.options.remote.beforeSend, this.options.remote.complete, this.options.remote.traditional);
      },

      ajaxCallBack: function (result) {
          var total = utility.mapObjectNameRecursion(result, this.options.remote.totalName);
          if (total == null || total == undefined){
              // throw new Error("the response of totalName :  '" + this.options.remote.totalName + "'  not found.");
              total = 0;
          }
          total = utility.convertInt(total);
          this.total = total;
          var lastPageNum = this.getLastPageNum();
          if (this.currentPageIndex > 0 && lastPageNum - 1 < this.currentPageIndex) {
              this.setPageIndex(lastPageNum - 1);
              this.remote();
          } else {
              if (typeof this.options.remote.success === 'function') this.options.remote.success(result);
              this.renderPagination();
          }
      },

      onEvent: function (eventName, pageIndex, pageSize) {
          if (pageIndex != null) this.currentPageIndex = utility.convertInt(pageIndex);
          if (pageSize != null) this.currentPageSize = utility.convertInt(pageSize);
          this.doPagination();
          this.$element.trigger(eventName, {
              pageIndex: this.currentPageIndex,
              pageSize: this.currentPageSize
          });
          this.debug('pagination ' + eventName);
      },
      //生成分页
      renderPagination: function () {
          var option = {
              showFirstLastBtn: this.options.showFirstLastBtn,
              firstBtnText: this.options.firstBtnText,
              prevBtnText: this.options.prevBtnText,
              nextBtnText: this.options.nextBtnText,
              lastBtnText: this.options.lastBtnText
          }
          var lastPageNum = this.getLastPageNum();
          this.currentPageIndex = lastPageNum > 0 && this.currentPageIndex > lastPageNum - 1 ? lastPageNum - 1 : this.currentPageIndex;
          this.$page.empty().append(pagination.core.renderPages(this.currentPageIndex, this.currentPageSize, this.total, this.options.pageBtnCount, option)).show();
          if (this.options.showPageSizes && lastPageNum !== 0) this.$size.show();
          else this.$size.hide();
          if (this.options.showJump && lastPageNum !== 0) this.$jump.show();
          else this.$jump.hide();
          this.$info.text(pagination.core.renderInfo(this.currentPageIndex, this.currentPageSize, this.total, this.options.infoFormat, this.options.noInfoText));
          if (this.options.showInfo) this.$info.show();
          else this.$info.hide();
      },
      //销毁分页
      destroy: function () {
          this.$element.unbind().data("pagination", null).empty();
          this.debug('pagination destroyed');
      },
      debug: function (message, data) {
          if (this.options.debug && console) {
              message && console.info(message + ' : pageIndex = ' + this.currentPageIndex + ' , pageSize = ' + this.currentPageSize + ' , total = ' + this.total);
              data && console.info(data);
          }
      }
  }

  var eventHandler = function (event) {
      var that = event.data.page;
      var $target = $(event.target);

      if (event.type === 'click' && $target.data('pageIndex') !== undefined && !$target.parent().hasClass('active')) {
          that.onEvent(pagination.event.pageClicked, $target.data("pageIndex"), null);
      } else if ((event.type === 'click' || event.type === 'keypress') && $target.data('pageBtn') === 'jump') {
          var pageIndexStr = that.$jump.find('input').val();
          if (utility.convertInt(pageIndexStr) <= that.getLastPageNum()) {
              that.onEvent(pagination.event.jumpClicked, pageIndexStr - 1, null);
              that.$jump.find('input').val(null);
          }
      } else if (event.type === 'change' && $target.data('pageBtn') === 'size') {
          var newPageSize = that.$size.find('select').val();
          var lastPageNum = pagination.core.calLastPageNum(that.total, newPageSize);
          if (lastPageNum > 0 && that.currentPageIndex > lastPageNum - 1) {
              that.currentPageIndex = lastPageNum - 1;
          }
          that.onEvent(pagination.event.pageSizeChanged, that.currentPageIndex, newPageSize);
      }
  };

  var pagination = {};
  pagination.event = {
      pageClicked: 'pageClicked',
      jumpClicked: 'jumpClicked',
      pageSizeChanged: 'pageSizeChanged'
  };
  pagination.remote = {
      getAjax: function (pagination, url, data, success, beforeSend, complate, traditional) {
          $.ajax({
              url: url,
              dataType: 'json',
              data: data,
              cache: false,
              traditional: traditional,
              contentType: 'application/Json',
              beforeSend: function (XMLHttpRequest) {
                  if (typeof beforeSend === 'function') beforeSend.call(this, XMLHttpRequest);
              },
              complete: function (XMLHttpRequest, textStatue) {
                  if (typeof complate === 'function') complate.call(this, XMLHttpRequest, textStatue);
              },
              success: function (result) {
                  success.call(pagination, result);
              }
          })
      }
  };
  pagination.core = {
      /*
      options : {
          showFirstLastBtn
          firstBtnText:
      }
      */
      renderPages: function (pageIndex, pageSize, total, pageBtnCount, options) {
          options = options || {};
          var pageNumber = pageIndex + 1;
          var lastPageNumber = this.calLastPageNum(total, pageSize);
          var html = [];

          if (lastPageNumber <= pageBtnCount) {
              html = this.renderGroupPages(1, lastPageNumber, pageNumber);
          } else {
              var firstPage = this.renderPerPage(options.firstBtnText || 1, 0);
              var lastPage = this.renderPerPage(options.lastBtnText || lastPageNumber, lastPageNumber - 1);


              //button count of  both sides
              var symmetryBtnCount = (pageBtnCount - 1 - 4) / 2;
              if (!options.showFirstLastBtn)
                  symmetryBtnCount = symmetryBtnCount + 1;
              var frontBtnNum = (pageBtnCount + 1) / 2;
              var behindBtnNum = lastPageNumber - ((pageBtnCount + 1) / 2);

              var prevPage = this.renderPerPage(options.prevBtnText, pageIndex - 1);
              var nextPage = this.renderPerPage(options.nextBtnText, pageIndex + 1);

              symmetryBtnCount = symmetryBtnCount.toString().indexOf('.') == -1 ? symmetryBtnCount : symmetryBtnCount + 0.5;
              frontBtnNum = frontBtnNum.toString().indexOf('.') == -1 ? frontBtnNum : frontBtnNum + 0.5;
              behindBtnNum = behindBtnNum.toString().indexOf('.') == -1 ? behindBtnNum : behindBtnNum + 0.5;
              if (pageNumber < frontBtnNum) {
                  if (options.showFirstLastBtn) {
                      html = this.renderGroupPages(1, pageBtnCount - 2, pageNumber);
                      html.push(nextPage);
                      html.push(lastPage);
                  } else {
                      html = this.renderGroupPages(1, pageBtnCount - 1, pageNumber);
                      html.push(nextPage);
                  }
              } else if (pageNumber > behindBtnNum) {
                  if (options.showFirstLastBtn) {
                      html = this.renderGroupPages(lastPageNumber - pageBtnCount + 3, pageBtnCount - 2, pageNumber);
                      html.unshift(prevPage);
                      html.unshift(firstPage);
                  } else {
                      html = this.renderGroupPages(lastPageNumber - pageBtnCount + 2, pageBtnCount - 1, pageNumber);
                      html.unshift(prevPage);
                  }
              } else {
                  if (options.showFirstLastBtn) {
                      html = this.renderGroupPages(pageNumber - symmetryBtnCount, pageBtnCount - 4, pageNumber);
                      html.unshift(prevPage);
                      html.push(nextPage);
                      html.unshift(firstPage);
                      html.push(lastPage);
                  } else {
                      html = this.renderGroupPages(pageNumber - symmetryBtnCount, pageBtnCount - 2, pageNumber);
                      html.unshift(prevPage);
                      html.push(nextPage);
                  }
              }
          }
          return html;
      },
      renderGroupPages: function (beginPageNum, count, currentPage) {
          var html = [];
          for (var i = 0; i < count; i++) {
              var page = this.renderPerPage(beginPageNum, beginPageNum - 1);
              if (beginPageNum === currentPage)
                  page.addClass("active");
              html.push(page);
              beginPageNum++;
          }
          return html;
      },
      renderPerPage: function (text, value) {
          return $("<li><a data-page-index='" + value + "'>" + text + "</a></li>");
      },
      renderInfo: function (currentPageIndex, currentPageSize, total, infoFormat, noInfoText) {
          if (total <= 0) {
              return noInfoText;
          } else {
              var startNum = (currentPageIndex * currentPageSize) + 1;
              var endNum = (currentPageIndex + 1) * currentPageSize;
              endNum = endNum >= total ? total : endNum;
              return infoFormat.replace('{start}', startNum).replace('{end}', endNum).replace('{total}', total);
          }
      },
      //计算最大分页数
      calLastPageNum: function (total, pageSize) {
          total = utility.convertInt(total);
          pageSize = utility.convertInt(pageSize);
          var i = total / pageSize;
          return utility.isDecimal(i) ? parseInt(i) + 1 : i;
      }
  };
  pagination.check = {
      //校验跳转页数有效性
      checkJumpPage: function (pageIndex) {
          var reg = /^\+?[1-9][0-9]*$/;
          return reg.test(pageIndex);
      }
  };

  var utility = {
      //转换为int
      convertInt: function (i) {
          if (typeof i === 'number') {
              return i;
          } else {
              var j = parseInt(i);
              if (!isNaN(j)) {
                  return j;
              } else {
                  throw new Error("convertInt Type Error.  Type is " + typeof i + ', value = ' + i);
              }
          }
      },
      //返回是否小数
      isDecimal: function (i) {
          return parseInt(i) !== i;
      },
      //匹配对象名称（递归）
      mapObjectNameRecursion: function (object, name) {
          var obj = object;
          var arr = name.split('.');
          for (var i = 0; i < arr.length; i++) {
              obj = utility.mapObjectName(obj, arr[i]);
          }
          return obj;
      },
      //匹配对象名称
      mapObjectName: function (object, name) {
          var value = null;
          for (var i in object) {
              //过滤原型属性
              if (object.hasOwnProperty(i)) {
                  if (i == name) {
                      value = object[i];
                      break;
                  }
              }
          }
          return value;
      },
      deserializeParams: function (params) {
          var newParams = {};
          if (typeof params === 'string') {
              var arr = params.split('&');
              for (var i = 0; i < arr.length; i++) {
                  var a = arr[i].split('=');
                  newParams[a[0]] = decodeURIComponent(a[1]);
              }
          } else if (params instanceof Array) {
              for (var i = 0; i < params.length; i++) {
                  newParams[params[i].name] = decodeURIComponent(params[i].value);
              }
          } else if (typeof params === 'object') {
              newParams = params;
          }
          return newParams;
      }
  }

  $.fn.pagination = function (option) {
      if (typeof option === 'undefined') {
          return $(this).data('pagination') || false;
      } else {
          var result;
          var args = arguments;
          this.each(function () {
              var $this = $(this);
              var data = $this.data('pagination');
              if (!data && typeof option === 'string') {
                  throw new Error('MricodePagination is uninitialized.');
              } else if (data && typeof option === 'object') {
                  throw new Error('MricodePagination is initialized.');
              }
              //初始化
              else if (!data && typeof option === 'object') {
                  var options = typeof option == 'object' && option;
                  var data_api_options = $this.data();
                  options = $.extend(options, data_api_options);
                  $this.data('pagination', (data = new Page(this, options)));
              } else if (data && typeof option === 'string') {
                  result = data[option].apply(data, Array.prototype.slice.call(args, 1));
              }
          });
          return typeof result === 'undefined' ? this : result;
      }
  }
});

+ function ($) {
  'use strict';

  // radioCheck CLASS DEFINITION
  // ======================

  var Radio = function (element, options) {
    this.$options = options
    this.$body = $(document.body)
    this.$el = $(element)
  }

  Radio.VERSION = '0.0.1'

  Radio.DEFAULTS = {
    show: true
  }

  Radio.prototype.init = function () {
    console.log(this.$el)
    this.render()
    this.bind()
  }

  Radio.prototype.render = function () {
    var $el = this.$el;

    $el.find('input').each(function (index, val) {
      var title = $(val).attr('title');
      var htmlVal = "<div class='hz-form-radio'><i class='hz-radio-unselect'></i><div>" + title + "</div></div>";
      $(val).after(htmlVal);
    });
  }

  Radio.prototype.bind = function () {
    var that = this;
    this.$el.find('input').each(function (index, val) {
      if ($(val).attr('checked')) {
        $(val).next().children('i').addClass('hz-radioed')
      }

      $(val).next('.hz-form-radio').click(function (e) {
        var radioSelect = $(this).children('i');
        if (radioSelect.hasClass('hz-radioed')) {
          return;
        } else {
          $('.hz-radio-unselect').removeClass('hz-radioed');
          radioSelect.addClass('hz-radioed');
          that.$options.click($(val).val(),e);
        }
      })
    })
  }


  // Radio PLUGIN DEFINITION
  // =======================

  function Plugin(options) {
    return this.each(function () {
      var rd = new Radio(this, options);
      rd.init();
    })
  }

  var old = $.fn.radio

  $.fn.radio = Plugin
  $.fn.radio.Constructor = Radio


  // radio NO CONFLICT
  // =================

  $.fn.radio.noConflict = function () {
    $.fn.radio = old
    return this
  }


  // Radio DATA-API
  // ==============

  $(document).on('click.bs.Radio.data-api', '[data-toggle="Radio"]', function (e) {
    var $this = $(this)
    var href = $this.attr('href')
    var $target = $($this.attr('data-target') || (href && href.replace(/.*(?=#[^\s]+$)/, ''))) // strip for ie7
    var option = $target.data('bs.Radio') ? 'toggle' : $.extend({
      remote: !/#/.test(href) && href
    }, $target.data(), $this.data())

    if ($this.is('a')) e.preventDefault()

    $target.one('show.bs.Radio', function (showEvent) {
      if (showEvent.isDefaultPrevented()) return // only register focus restorer if Radio will actually get shown
      $target.one('hidden.bs.Radio', function () {
        $this.is(':visible') && $this.trigger('focus')
      })
    })
    Plugin.call($target, option, this)
  })

}(jQuery);
/**
 * switch开关组件
 * 参数可选
 * 用法 el.swithc({
 * 	click: function (onoff, el, event) {}
 * })
 * 
 * click参数说明：
 * onoff 开关状态  true 代表开  fault 代表关
 * el 当前点击的节点
 * event 触发事件对象
 * 
 */

+function ($) {
  'use strict';

  var Switch = function (el, options) {
    this.$el = el
    this.defaults = {};
    this.options = $.extend({}, this.defaults, options)
  }

  // 初始化函数
  Switch.prototype.init = function () {
    this.options.onoff = this.$el.hasClass('switch-on')
    this.changeColor()
  }

  // 初始化函数
  Switch.prototype.changeColor = function () {
    var el = this.$el
    if (this.options.onoff && el.attr('themeColor')) {
      el.css('background', el.attr('themeColor'))
    }
    this.bind()
  }

  // 更改switch颜色，状态
  Switch.prototype.bind = function () {
    var el = this.$el
    var options = this.options
    el.click(function (event) {
      options.onoff = !options.onoff
      if (options.onoff) {
        el.removeClass('switch-off').addClass('switch-on')
        if (el.attr('themeColor')) {
          el.css('background', el.attr('themeColor'))
        }
      } else {
        el.removeClass('switch-on').addClass('switch-off')
        if (el.attr('themeColor')) {
          el.css('background', '#D1D1D1')
        }
      }
      if (options.click && typeof options.click === 'function') {
        options.click(options.onoff, el, event)
      }
    })
  }

  function Plugin(options) {
    this.each(function (i, node) {
      var sw = new Switch($(node), options)
      sw.init();
    })
  }

  var old = $.fn.switch

  $.fn.switch = Plugin
  $.fn.switch.Constructor = Switch


  $.fn.switch.noConflict = function () {
    $.fn.switch = old
    return this
  }

}(jQuery);

/**
 * tab选项卡组件
 * 参数可选
 * 用法 el.tab({
 * 	onClickTab: function (event, el, index) {}
 * })
 * 
 * onClickTab参数说明：
 * 
 * index 当前活动展示的tab的索引值
 * el 当前点击的节点
 * event 触发事件对象
 * 
 */

+function ($) {
  'use strict';

  var Tab = function (el, options) {
    this.$el = el
    this.defaults = {};
    this.options = $.extend({}, this.defaults, options)
  }

  // 初始化函数
  Tab.prototype.init = function () {
    this.bind()
  }

  // 初始化函数
  Tab.prototype.bind = function () {
    var self = this
    var options = this.options
    var clickHandler = function (e) {
      if ($(this).attr('href') && $(this).attr('href')[0] !== '#') return
      e.preventDefault();
      var $this = $(this);
      var parent = $this.parent('li');
      if (parent.is('[disabled]')) return

      var selector = $this.attr('href');
      if (parent.hasClass('active')) return
      //切换tabpane
      $(selector).siblings('.tab-pane').removeClass('active');
      if ($(selector).hasClass('tab-pane')) {
        $(selector).addClass('active');
      }
      parent.siblings().removeClass('active');
      parent.addClass('active');
      if (options) {
        if (options.onClickTab && typeof options.onClickTab == 'function') {
          options.onClickTab($this, self, parent.index());
        }
      }
      self.$el.trigger('hz.tab.click', {
        event: $this,
        el: self,
        index: parent.index()
      })
    };

    $(document).on('click', '.sub-tabs a, .nav-tabs a, .nav-tabs-card a, [role="tablist"] a[role="tab"]', clickHandler)
  }


  function Plugin(option) {
    this.each(function (i, node) {
      var tab = new Tab($(node), option)
      tab.init();
    })
  }

  var old = $.fn.tab

  $.fn.tab = Plugin
  $.fn.tab.Constructor = Tab


  $.fn.tab.noConflict = function () {
    $.fn.switch = old
    return this
  }

}(jQuery);

/**
 * table组件
 * 参数可选
 * 用法 el.tableRender({
 * 	width: 600,
		height: 300,
		headerFixed: true, // 是否开启头部固定
		cols: [],
		data: []
 * })
 * 
 * onClickTab参数说明：
 * 
 * width 表格宽度
 * height 表格高度
 * headerFixed 是否开启头部固定
 * cols 列配置
 * data 渲染数据数组
 */

+function ($) {
  'use strict';

  var Table = function (el, options) {
    this.$el = el;
    this.width = options.width || $el.width()
    this.height = parseInt(options.height) ? parseInt(options.height) + 'px' : 'auto'
    this.fixedRightCol = []
    this.defaults = {
      cols: [],
      data: []
    };
    this.options = $.extend({}, this.defaults, options)
  }

  // 初始化函数
  Table.prototype.init = function (options) {
    this.render()
    this.bind()
    return $('.hz-table')
  }
  // 渲染表格函数
  Table.prototype.render = function () {
    var cols = this.options.cols
    var data = this.options.data
    var self = this
    var colgroup = ''
    var colArr = []
    var thstr = ''
    var allWidth = 0

    for (var i = 0, len = cols.length; i < len; i++) {
      var width = parseInt(cols[i].width)
      allWidth += width
      colArr.push('<col width="' + width + '"></col>')
      thstr += '<th><div>' + cols[i].label + '</div></th>'
      if (cols[i].fixed === 'right') {
        this.fixedRightCol.push(cols[i])
      }
    }
    if (allWidth < this.width) {
      var dis = this.width - allWidth
      var str = ''
      var width = 0
      if (cols[i - 1].type || cols[i - 1].fixed) {
        str = colArr[i - 2]
        width = parseInt(cols[i - 2].width) + dis
        str = str.replace(/\d+/g, width + '')
        colArr.splice(i - 2, 1, str)
      } else {
        str = colArr[i - 1]
        width = parseInt(cols[i - 1].width) + dis
        str = str.replace(/\d+/g, width + '')
        colArr.splice(i - 1, 1, str)
      }
      allWidth = this.width
    }
    $.each(colArr, function (i, item) {
      if (item.fixed === 'fixed') {
        self.fixedRightCol.push(item)
      }
    })

    var colstr = colArr.join('')
    var colgroup = '<colgroup>' + colstr + '</colgroup>'
    var theader = '<thead><tr>' + thstr + '</tr></thead>'

    var trs = ''

    for (var j = 0, leng = data.length; j < leng; j++) {
      var tds = ''
      for (var z = 0; z < cols.length; z++) {
        var text = data[j][cols[z].key] || ''
        if (cols[z].render && typeof cols[z].render === 'function') {
          text = cols[z].render(data[j])
        }
        if (cols[z].type === 'link') {
          if (toType(cols[z].text) === 'Array') {
            var btns = toOparations(cols[z].text, j)
            tds += '<td><div>' + btns + '</div></td>'
          } else {
            tds += '<td><div><a href="#" data-rowsIndex="' + j + '" data-btnText="' + cols[z].text + '">' + cols[z].text + '</a></div></td>'
          }
        } else {
          tds += '<td><div>' + text + '</div></td>'
        }
      }
      trs += '<tr>' + tds + '</tr>'
    }

    var tbody = '<tbody>' + trs + '</tbody>'
    var fixedRightStr = ''
    if (this.fixedRightCol.length > 0) {
      fixedRightStr = this.toFixedRightTemplate()
    }
    var tempalte = this.toTemplate({
      allWidth: allWidth,
      colgroup: colgroup,
      theader: theader,
      tbody: tbody,
      fixedRightStr: fixedRightStr
    })
    this.$el.html(tempalte)
  }
  // 渲染右边固定列
  Table.prototype.toFixedRightTemplate = function () {
    var cols = this.fixedRightCol
    var data = this.options.data
    var colstr = ''
    var thstr = ''
    var trs = ''
    var allwidth = 0
    for (var i = 0, len = cols.length; i < len; i++) {
      var width = parseInt(cols[i].width)
      allwidth += width
      colstr += '<col width="' + width + '"></col>'
      thstr += '<th><div>' + cols[i].label + '</div></th>'
    }
    var colgroup = '<colgroup>' + colstr + '</colgroup>'
    var theader = '<thead><tr>' + thstr + '</tr></thead>'

    for (var j = 0, leng = data.length; j < leng; j++) {
      var tds = ''
      for (var z = 0; z < cols.length; z++) {
        if (cols[z].type === 'link') {
          if (toType(cols[z].text) === 'Array') {
            var btns = toOparations(cols[z].text, j)
            tds += '<td><div>' + btns + '</div></td>'
          } else {
            tds += '<td><div><a href="#" data-rowsIndex="' + j + '">' + cols[z].text + '</a></div></td>'
          }
        } else {
          tds += '<td><div></div></td>'
        }
      }
      trs += '<tr>' + tds + '</tr>'
    }
    var tbody = '<tbody>' + trs + '</tbody>'

    var headerFixed = ''
    if (this.options.headerFixed) {
      headerFixed = '<table class="fixedRightHeaderTable" width="' + allwidth + '" cellspacing="0" cellpadding="0" border="0"> \
											'+ colgroup + theader + ' \
										</table>'
    }

    var fixedRightTemplate = '<div class="fixedRight" style="width: ' + allwidth + 'px; height: ' + this.height + '"> \
				{{_headerFixed}}\
				<table class="fixedRightTable" width="'+ allwidth + '" cellspacing="0" cellpadding="0" border="0">\
					'+ colgroup + theader + tbody + '\
				</table>\
			</div>'
    fixedRightTemplate = fixedRightTemplate.replace('{{_headerFixed}}', headerFixed)
    return fixedRightTemplate
  },
    // 输出固定头部
    Table.prototype.toHeanderTemplate = function (params) {
      var headerStr = ''
      if (this.options.headerFixed) {
        headerStr = '<div class="theader"> \
										<table class="fixedHeaderTable" width="'+ params.allWidth + '" cellspacing="0" cellpadding="0" border="0"> \
											'+ params.colgroup + params.theader + ' \
										</table>\
									</div>'
      }
      return headerStr
    }
  // 输出模板函数
  Table.prototype.toTemplate = function (params) {
    var headerStr = this.toHeanderTemplate(params)
    var tmp = ''
    var tmp = '<div class="hz-table"> \
					{{_headerTemplate}} \
					<div class="tbody" style="height: '+ this.height + '"> \
						<table class="tableBody" width="'+ params.allWidth + '" cellspacing="0" cellpadding="0" border="0"> \
						'+ params.colgroup + params.theader + params.tbody + '\
						</table>\
					</div>\
					{{_fixedRight}}\
			</div>'

    tmp = tmp.replace('{{_headerTemplate}}', headerStr)
    tmp = tmp.replace('{{_fixedRight}}', params.fixedRightStr)
    return tmp
  }

  Table.prototype.bind = function () {
    var self = this
    var fixedHeaderTable = this.$el.find('.fixedHeaderTable').eq(0)
    var trs = this.$el.find('.tableBody').find('tbody').find('tr')
    var fixedRightTable = this.$el.find('.fixedRightTable').eq(0)
    var rightTrs = $(fixedRightTable).find('tbody').find('tr')
    var hzTable = $('.hz-table')
    var tbodyBox = this.$el.find('.tbody')
    tbodyBox.on('scroll', function (e) {
      $(fixedHeaderTable).css('left', -$(this).scrollLeft() + 'px')
      $(fixedRightTable).css('top', -$(this).scrollTop() + 'px')
    })

    $(trs).on('mouseenter', function (e) {
      rowEnter.call(this, rightTrs)
    })
    $(trs).on('mouseleave', function (e) {
      rowLeave.call(this, rightTrs)
    })
    $(rightTrs).on('mouseenter', function (e) {
      rowEnter.call(this, trs)
    })
    $(rightTrs).on('mouseleave', function (e) {
      rowLeave.call(this, trs)
    })
    hzTable.click(function (e) {
      console.log(e.target)
      var type = $(e.target).data('type')
      var obj = {}
      if (type === 'btn') {
        var rowIndex = $(e.target).data('rowsindex')
        var btnText = $(e.target).data('btntext')
        obj.row = self.options.data[rowIndex]
        obj.btnText = btnText
      }
      hzTable.trigger('hz.click', obj)
    })
  }

  // 判断类型工具函数
  function toType(type) {
    return Object.prototype.toString.call(type).match(/\[object (\w+)\]/)[1]
  }
  // 拼接操作栏字符串函数
  function toOparations(arr, j) {
    var str = ''
    for (var i = 0; i < arr.length; i++) {
      str += '<a href="#" data-type="btn" data-rowsIndex="' + j + '" data-btnText="' + arr[i] + '">' + arr[i] + '</a>'
    }
    return str
  }
  // 当鼠标移入tr的时候
  function rowEnter(els) {
    var _this = $(this)
    var index = _this.index()
    _this.addClass('rowActive')
    $(els).eq(index).addClass('rowActive')
  }
  // 当鼠标移出tr的时候
  function rowLeave(els) {
    var _this = $(this)
    var index = _this.index()
    _this.removeClass('rowActive')
    $(els).eq(index).removeClass('rowActive')
  }

  // 插件函数
  function Plugin(option) {
    var table = new Table(this, option)
    return table.init();
  }
  // 绑定到$上面
  $.fn.tableRender = Plugin
  $.fn.tableRender.Constructor = Table

}(jQuery);
