/** 
* jQuery HzUI V0.0.2 
*/
/*
 * Jquery Message插件
 * 使用例子 :
 * 1. $.alert("提示内容",{
 *      title : "标题",
 *      position : ['left', [-0.1,0]]
 * })
 * 2. $.alert("提示内容","标题");
 * 位置请使用
 * top-left,top-right,bottom-left,bottom-right,center 大小写都可以哦
 */

(function($) {
  $.alertExt = {
    // 默认配置
    defaults: {
      id: "",
      autoClose: true, // 自动关闭
      closeTime: 3000, // 自动关闭时间，不少于1000
      withTime: false, // 添加计时  会在文字后面添加  ...10
      type: "success", // 提示类型, ['success', 'danger', 'warning', 'info']
      isDetail: true, // 是否为详细类型弹窗
      position: ["center", [-0.42, 0]], // 位置,第一个写位置; 后面是相对于window的偏移，如果是1跟-1之间为百分比, [top, right]
      title: "", // 标题
      close: "", // 需绑定关闭事件的按钮
      speed: "normal", // fadeIn / fadeOut 速度
      isOnly: true, // 是否只出现一个
      minTop: 10, //最小Top
      onShow: function() {}, // 打开后回调
      onClose: function() {}, // 关闭后回调
      onNoMoreAlert: function() {} // 点击不再显示后回调
    },

    // 提示框模版
    detailTmpl:
      '<div class="alert alert-dismissable alert-detail ${Type}">' +
      '<i class="type-icon iconfont icon-${State}"></i>' +
      '<button type="button" class="close" data-dismiss="alert" aria-hidden="true">' +
      '<i class="close-white-icon iconfont icon-close2"></i>' +
      "</button>" +
      '<h4 style="white-space: nowrap;">' +
      "${Title}" +
      "</h4>" +
      "<p>" +
      '<span class="msg">${Content}</span>' +
      '<span class="no-more-alert"><i class="checkbox iconfont icon-non-check"></i>不再提示</span>' +
      "</p>" +
      "</div>",
    basicTmpl:
      '<div class="alert alert-dismissable alert-basic ${Type}">' +
      '<p class="msg">' +
      "${Content}" +
      "</p>" +
      '<button type="button" class="close" data-dismiss="alert" aria-hidden="true">' +
      '<i class="close-white-icon iconfont icon-close2"></i>' +
      "</button>" +
      "</div>",

    noMoreAlertList: [],

    // 初始化函数
    init: function(msg, options) {
      if (this.noMoreAlertList.indexOf(options.id) > -1) return false;

      this.options = $.extend({}, this.defaults, options);

      this.create(msg);
      this.setCss();

      this.bindEvent();

      return this.alertDiv;
    },

    template: function(tmpl, data) {
      $.each(data, function(k, v) {
        tmpl = tmpl.replace("${" + k + "}", v);
      });
      return $(tmpl);
    },

    // 创建提示框
    create: function(msg) {
      var tmpl = this.options.isDetail ? this.detailTmpl : this.basicTmpl;
      this.alertDiv = this.template(tmpl, {
        State: this.options.type,
        Type: "alert-" + this.options.type,
        Title: this.options.title,
        Content: msg
      }).hide();
      if (this.options.isOnly) {
        $("body > .alert").remove();
      }
      this.alertDiv.appendTo($("body"));
    },

    // 设置样式
    setCss: function() {
      var alertDiv = this.alertDiv;

      // 初始化样式
      alertDiv.css({
        position: "fixed",
        "z-index": 10001 + $(".alert").length
      });

      // IE6兼容
      var ie6 = 0;
      if ($.browser && $.browser.msie && $.browser.version == "6.0") {
        alertDiv.css("position", "absolute");
        ie6 = $(window).scrollTop();
      }

      // 位置设置提取
      var position = this.options.position,
        posStr = position[0].split("-"),
        pos = [0, 0];
      if (position.length > 1) {
        pos = position[1];
      }

      // 偏移百分比检测
      if (pos[0] > -1 && pos[0] < 1) {
        pos[0] = pos[0] * $(window).height();
      }
      if (pos[1] > -1 && pos[1] < 1) {
        pos[1] = pos[1] * $(window).width();
      }

      // 位置设置
      for (var i in posStr) {
        if ($.type(posStr[i]) !== "string") {
          continue;
        }
        var str = posStr[i].toLowerCase();

        if ($.inArray(str, ["left", "right"]) > -1) {
          alertDiv.css(str, pos[1]);
        } else if ($.inArray(str, ["top", "bottom"]) > -1) {
          alertDiv.css(str, pos[0] + ie6);
        } else {
          alertDiv.css({
            top:
              ($(window).height() - alertDiv.outerHeight()) / 2 + pos[0] + ie6,
            left: ($(window).width() - alertDiv.outerWidth()) / 2 + pos[1]
          });
        }
      }

      if (parseInt(alertDiv.css("top")) < this.options.minTop) {
        alertDiv.css("top", this.options.minTop);
      }
    },

    // 绑定事件
    bindEvent: function() {
      this.bindShow();
      this.bindClose();
      this.bindNoMoreAlert();

      if ($.browser && $.browser.msie && $.browser.version == "6.0") {
        this.bindScroll();
      }
    },

    // 显示事件
    bindShow: function() {
      var ops = this.options;
      this.alertDiv.fadeIn(ops.speed, function() {
        ops.onShow($(this));
      });
    },

    // 关闭事件
    bindClose: function() {
      var alertDiv = this.alertDiv,
        ops = this.options,
        closeBtn = $(".close", alertDiv).add($(this.options.close, alertDiv));

      closeBtn.bind("click", function(e) {
        alertDiv.fadeOut(ops.speed, function() {
          $(this).remove();
          ops.onClose($(this));
        });
        e.stopPropagation();
      });

      // 自动关闭绑定
      if (ops.autoClose) {
        var time = parseInt(ops.closeTime / 1000);
        if (ops.withTime) {
          $("p", alertDiv).append(
            '<span>...<em class="close-time-count">' + time + "</em></span>"
          );
        }
        var timer = setInterval(function() {
          $(".close-time-count", alertDiv).text(--time);
          if (!time) {
            clearInterval(timer);
            closeBtn.trigger("click");
          }
        }, 1000);
      }
    },

    // 不再提示事件
    bindNoMoreAlert: function() {
      var ops = this.options;
      var _this = this;
      $(".no-more-alert", this.alertDiv).on("click", function(e) {
        var $checkbox = $(this).find(".checkbox");
        var idx = _this.noMoreAlertList.indexOf(ops.id);
        if ($checkbox.hasClass("icon-check_focus")) {
          $checkbox.removeClass("icon-check_focus").addClass("icon-non-check");
          if (idx > -1) {
            _this.noMoreAlertList.splice(idx, 1);
          }
        } else {
          $checkbox.addClass("icon-check_focus").removeClass("icon-non-check");
          if (idx === -1) {
            _this.noMoreAlertList.push(ops.id);
          }
        }
        ops.onNoMoreAlert($(this));
      });
    },

    // IE6滚动跟踪
    bindScroll: function() {
      var alertDiv = this.alertDiv,
        top = alertDiv.offset().top - $(window).scrollTop();
      $(window).scroll(function() {
        alertDiv.css("top", top + $(window).scrollTop());
      });
    },

    // 检测是否为手机浏览器
    checkMobile: function() {
      var userAgent = navigator.userAgent;
      var keywords = [
        "Android",
        "iPhone",
        "iPod",
        "iPad",
        "Windows Phone",
        "MQQBrowser"
      ];
      for (var i in keywords) {
        if (userAgent.indexOf(keywords[i]) > -1) {
          return keywords[i];
        }
      }
      return false;
    }
  };

  $.alert = function(msg, arg) {
    arg = arg || {};
    if ($.alertExt.checkMobile()) {
      alert(msg);
      return;
    }
    if (!$.trim(msg).length) {
      return false;
    }
    if ($.type(arg) === "string") {
      arg = {
        title: arg
      };
    }
    // 对于详细消息提示，如果没有title，将msg作为title
    if (arg.isDetail !== false && !arg.title) {
      arg.title = msg;
      msg = "";
    }
    // 统一消息类型
    if (arg) {
      switch (arg.type) {
        case "error":
          arg.type = "danger";
          break;
        case "positive":
          arg.type = "info";
          break;
        default:
          break;
      }
    }
    if (arg.id === undefined) {
      arg.id = arg.title + "-" + msg;
    }

    return $.alertExt.init(msg, arg);
  };
})(jQuery);

/**
 * button-top组件
 * 参数可选
 * 用法 el.buttonTop({
 * })
 *
 * options参数说明：
 *  class:       样式          |  default(hz-btn-top)     | 类型（String）
 *  toTop：      滚动目的地      | default(0)              | 类型（Number）
 *  scrollDom：  指定某个dom滚动 | default($('html，body')) | 类型（jq对象）
 *  delay:       滚动时长       | default(200)             | 类型（Number）
 *  menu:        增加其他按钮    | default(false)           | 类型(Array)
 *
 * 事件：
 * menuClick:
 */
+ function($) {
  var ButtonTop = function(el, options) {
    this.$el = el;
    this.options = $.extend(true, {}, options)
  }
  ButtonTop.prototype = {
    init: function() {
      var _element = this.$el
      var _this = this
      if (_this.options.menu) {
        _this.createMenuDom()
        _element.addClass('hz-btn-menu-box ' + _this.options.class)
      } else {
        _element.addClass('hz-btn-top ' + _this.options.class)
      }
      _element.on('click', function(e) {
        e.stopPropagation()
        _this.scrollToTop()
      })
    },
    scrollToTop: function() {
      var _toTop = this.options.toTop || 0,
        $scrollDom = this.options.scrollDom,
        _delay = this.options.delay || 200;
      if (!$scrollDom || !_isJquery($scrollDom)) {
        $scrollDom = $('html,body')
      }
      $scrollDom.animate({
        scrollTop: _toTop
      }, _delay);
    },
    createMenuDom: function() {
      var _this = this
      var _element = _this.$el,
        options = _this.options,
        menu = _this.options.menu,
        dom = '<div class="hz-btn-top-menu"><p class="hz-btn-top-add">+</p><ul>';
      _element.wrap('<div class="hz-btn-top-box"></div>');
      for (var m = 0, mLen = menu.length; m < mLen; m++) {
        dom = dom + '<li class=' + menu[m].class + ' key=' + menu[m].key + '>' + menu[m].name + '</li>'
      }
      _element.parent().prepend(dom + '</ul></div>')
      _element.parent().on('click', 'li', function(e) {
        e.stopPropagation();
        var txt = $(this).text();
        var key = $(this).attr("key");
        if (options && options.menuClick && typeof options.menuClick == 'function') {
          options.menuClick.call(this, key, txt, this);
        }
      })
    }
  }

  function _isJquery(obj) {
    if (obj instanceof jQuery) {
      return true
    } else {
      return false
    }
  }

  function Plugin(options) {
    var buttonTop = new ButtonTop(this, options)
    return buttonTop.init();
  }

  $.fn.buttonTop = Plugin
  $.fn.buttonTop.Constructor = ButtonTop
}(jQuery)

+function ($) {
  'use strict';

  /**
   *
   * @param element，容器
   * @param options，配置表，包含以下：
   * * @param options.list，checkbox列表，{key: 1, value: '键1', checked: false, disabled: false}，必需
   * * @param options.checkedHandle，选中当前框的触发
   */
  var CheckBox = function (element, options) {
    this.options = options
    this.$element = $(element)

    init(this.$element, this.options)
  }

  // 获取已选中的数据
  CheckBox.prototype.getChecked = function () {
    var res = []
    var _this = this
    _this.$element.find('.hzui-checkbox.active').each(function(){
      var index = $(this).data('index')

      res.push(_this.options.list[index])
    })

    return res
  }

  /**
   * @description 手动操作checkBox的选中状态
   * @param keyArray，需要修改的键与状态：[{key: 1, checked: true}]
   */
  CheckBox.prototype.active = function (keyArray) {
    var _this = this
    keyArray.forEach(function (item) {
      var ele = _this.$element.find('.hzui-checkbox[data-key="' + item.key + '"]')

      if (item.checked) {
        ele.addClass('active')
        ele.find('.hzui-checkbox-icon').removeClass('icon-non-check')
        ele.find('.hzui-checkbox-icon').addClass('icon-check_focus')
      } else {
        ele.removeClass('active')
        ele.find('.hzui-checkbox-icon').addClass('icon-non-check')
        ele.find('.hzui-checkbox-icon').removeClass('icon-check_focus')
      }

      var index = ele.data('index')
      var option = _this.options.list[index]
      option.isChecked = item.checked

      if (option.checkedHandle && typeof option.checkedHandle == 'function') {
        option.checkedHandle(option, ele)
      }
    })
  }

  /**
   * @description 手动操作checkBox的禁用状态
   * @param keyArray，需要修改的键与状态：[{key: 1, disabled: true}]
   */
  CheckBox.prototype.disabled = function (keyArray) {
    var _this = this
    keyArray.forEach(function (item) {
      var ele = _this.$element.find('.hzui-checkbox[data-key="' + item.key + '"]')
      
      if (item.disabled) {
        ele.addClass('disabled')
      } else {
        ele.removeClass('disabled')
      }

      var index = ele.data('index')
      var option = _this.options.list[index]
      option.disabled = item.disabled
    })
  }

  function init ($element, options) {
    var html = ''
    for (var i = 0; i < options.list.length; i++) {
      var item = options.list[i]
      html += '<div class="hzui-checkbox'

      if (item.checked) {
        html += ' active'
      }

      if (item.disabled) {
        html += ' disabled'
      }

      html += '" data-index="' + i + '"' + ' data-key="' + item.key + '"' + '><i class="hzui-checkbox-icon icon iconfont'
      if (item.checked) {
        html += ' icon-check_focus'
      } else {
        html += ' icon-non-check'
      }

      html += '"></i>'
      html += '<span class="hzui-checkbox-label">' + item.value + '</span>'
      html += '</div>'
    }

    $element.append(html)

    checkBoxSelectHandle($element, options)
  }

  function checkBoxSelectHandle ($element, options) {
    $element.on('click', '.hzui-checkbox', function(){
      var index = $(this).index()
      var item = options.list[index]
      if (item.disabled) {
        return
      }

      if (!$(this).hasClass('active')) {
        $(this).addClass('active')
        $(this).find('.icon').removeClass('icon-non-check')
        $(this).find('.icon').addClass('icon-check_focus')
      } else {
        $(this).removeClass('active')
        $(this).find('.icon').addClass('icon-non-check')
        $(this).find('.icon').removeClass('icon-check_focus')
      }

      if (item.checkedHandle && typeof item.checkedHandle == 'function') {
        item.checkedHandle(item, $(this))
      }
    })
  }

  $.fn._checkBox = function(arg1, arg2, arg3) {
    var results = []

    this.each(function () {
      var checkBox = $(this).data('_checkBox')
      //初始化新的checkBox
      if (!checkBox) {
        checkBox = new CheckBox(this, arg1)
        $(this).data('_checkBox', checkBox)
        results.push(checkBox)
        $(this).val($(this).val())
      } else if (!arg1 && !arg2) {
        results.push(checkBox)
      } else if (checkBox[arg1] !== undefined) {
        var retVal = undefined
        if (checkBox[arg1].length === 3 && arg3 !== undefined) {
          retVal = checkBox[arg1](arg2, null, arg3)
        } else {
          retVal = checkBox[arg1](arg2)
        }
        if (retVal !== undefined)
          results.push(retVal)
      }
    });

    if (typeof arg1 == 'string') {
      return results.length > 1 ? results : results[0]
    } else {
      return results
    }
  }

  $.fn._checkBox.Constructor = CheckBox
}(jQuery)

/* =========================================================
 * bootstrap-datetimepicker.js
 * =========================================================
 * Copyright 2012 Stefan Petre
 *
 * Improvements by Andrew Rowls
 * Improvements by Sébastien Malot
 * Improvements by Yun Lai
 * Improvements by Kenneth Henderick
 * Improvements by CuGBabyBeaR
 * Improvements by Christian Vaas <auspex@auspex.eu>
 *
 * Project URL : http://www.malot.fr/bootstrap-datetimepicker
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================= */

;(function (factory) {
  if (typeof define === 'function' && define.amd)
    define(['jquery'], factory);
  else if (typeof exports === 'object')
    factory(require('jquery'));
  else
    factory(jQuery);

}(function ($, undefined) {

  // Add ECMA262-5 Array methods if not supported natively (IE8)
  if (!('indexOf' in Array.prototype)) {
    Array.prototype.indexOf = function (find, i) {
      if (i === undefined) i = 0;
      if (i < 0) i += this.length;
      if (i < 0) i = 0;
      for (var n = this.length; i < n; i++) {
        if (i in this && this[i] === find) {
          return i;
        }
      }
      return -1;
    }
  }

  // Add timezone abbreviation support for ie6+, Chrome, Firefox
  function timeZoneAbbreviation () {
    var abbreviation, date, formattedStr, i, len, matchedStrings, ref, str;
    date = (new Date()).toString();
    formattedStr = ((ref = date.split('(')[1]) != null ? ref.slice(0, -1) : 0) || date.split(' ');
    if (formattedStr instanceof Array) {
      matchedStrings = [];
      for (var i = 0, len = formattedStr.length; i < len; i++) {
        str = formattedStr[i];
        if ((abbreviation = (ref = str.match(/\b[A-Z]+\b/)) !== null) ? ref[0] : 0) {
          matchedStrings.push(abbreviation);
        }
      }
      formattedStr = matchedStrings.pop();
    }
    return formattedStr;
  }

  function UTCDate () {
    return new Date(Date.UTC.apply(Date, arguments));
  }

  // Picker object
  var Datetimepicker = function (element, options) {
    var that = this;

    this.element = $(element);

    // add container for single page application
    // when page switch the datetimepicker div will be removed also.
    this.container = options.container || 'body';

    this.language = options.language || this.element.data('date-language') || 'en';
    this.language = this.language in dates ? this.language : this.language.split('-')[0]; // fr-CA fallback to fr
    this.language = this.language in dates ? this.language : 'en';
    this.isRTL = dates[this.language].rtl || false;
    this.formatType = options.formatType || this.element.data('format-type') || 'standard';
    this.format = DPGlobal.parseFormat(options.format || this.element.data('date-format') || dates[this.language].format || DPGlobal.getDefaultFormat(this.formatType, 'input'), this.formatType);
    this.isInline = false;
    this.isVisible = false;
    this.isInput = this.element.is('input');
    this.fontAwesome = options.fontAwesome || this.element.data('font-awesome') || false;

    this.bootcssVer = options.bootcssVer || (this.isInput ? (this.element.is('.form-control') ? 3 : 2) : (this.bootcssVer = this.element.is('.input-group') ? 3 : 2));

    this.component = this.element.is('.date') ? (this.bootcssVer === 3 ? this.element.find('.input-group-addon .glyphicon-th, .input-group-addon .glyphicon-time, .input-group-addon .glyphicon-remove, .input-group-addon .glyphicon-calendar, .input-group-addon .fa-calendar, .input-group-addon .fa-clock-o').parent() : this.element.find('.add-on .icon-th, .add-on .icon-time, .add-on .icon-calendar, .add-on .fa-calendar, .add-on .fa-clock-o').parent()) : false;
    this.componentReset = this.element.is('.date') ? (this.bootcssVer === 3 ? this.element.find('.input-group-addon .glyphicon-remove, .input-group-addon .fa-times').parent() : this.element.find('.add-on .icon-remove, .add-on .fa-times').parent()) : false;
    this.hasInput = this.component && this.element.find('input').length;
    if (this.component && this.component.length === 0) {
      this.component = false;
    }
    this.linkField = options.linkField || this.element.data('link-field') || false;
    this.linkFormat = DPGlobal.parseFormat(options.linkFormat || this.element.data('link-format') || DPGlobal.getDefaultFormat(this.formatType, 'link'), this.formatType);
    this.minuteStep = options.minuteStep || this.element.data('minute-step') || 5;
    this.pickerPosition = options.pickerPosition || this.element.data('picker-position') || 'bottom-right';
    this.showMeridian = options.showMeridian || this.element.data('show-meridian') || false;
    this.initialDate = options.initialDate || new Date();
    this.zIndex = options.zIndex || this.element.data('z-index') || undefined;
    this.title = typeof options.title === 'undefined' ? false : options.title;
    this.timezone = options.timezone || timeZoneAbbreviation();

    this.icons = {
      leftArrow: this.fontAwesome ? 'fa-arrow-left' : (this.bootcssVer === 3 ? 'glyphicon-arrow-left' : 'icon-arrow-left'),
      rightArrow: this.fontAwesome ? 'fa-arrow-right' : (this.bootcssVer === 3 ? 'glyphicon-arrow-right' : 'icon-arrow-right')
    }
    this.icontype = this.fontAwesome ? 'fa' : 'glyphicon';

    this._attachEvents();

    this.clickedOutside = function (e) {
      // Clicked outside the datetimepicker, hide it
      if ($(e.target).closest('.datetimepicker').length === 0) {
        that.hide();
      }
    }

    this.formatViewType = 'datetime';
    if ('formatViewType' in options) {
      this.formatViewType = options.formatViewType;
    } else if ('formatViewType' in this.element.data()) {
      this.formatViewType = this.element.data('formatViewType');
    }

    this.minView = 0;
    if ('minView' in options) {
      this.minView = options.minView;
    } else if ('minView' in this.element.data()) {
      this.minView = this.element.data('min-view');
    }
    this.minView = DPGlobal.convertViewMode(this.minView);

    this.maxView = DPGlobal.modes.length - 1;
    if ('maxView' in options) {
      this.maxView = options.maxView;
    } else if ('maxView' in this.element.data()) {
      this.maxView = this.element.data('max-view');
    }
    this.maxView = DPGlobal.convertViewMode(this.maxView);

    this.wheelViewModeNavigation = false;
    if ('wheelViewModeNavigation' in options) {
      this.wheelViewModeNavigation = options.wheelViewModeNavigation;
    } else if ('wheelViewModeNavigation' in this.element.data()) {
      this.wheelViewModeNavigation = this.element.data('view-mode-wheel-navigation');
    }

    this.wheelViewModeNavigationInverseDirection = false;

    if ('wheelViewModeNavigationInverseDirection' in options) {
      this.wheelViewModeNavigationInverseDirection = options.wheelViewModeNavigationInverseDirection;
    } else if ('wheelViewModeNavigationInverseDirection' in this.element.data()) {
      this.wheelViewModeNavigationInverseDirection = this.element.data('view-mode-wheel-navigation-inverse-dir');
    }

    this.wheelViewModeNavigationDelay = 100;
    if ('wheelViewModeNavigationDelay' in options) {
      this.wheelViewModeNavigationDelay = options.wheelViewModeNavigationDelay;
    } else if ('wheelViewModeNavigationDelay' in this.element.data()) {
      this.wheelViewModeNavigationDelay = this.element.data('view-mode-wheel-navigation-delay');
    }

    this.startViewMode = 2;
    if ('startView' in options) {
      this.startViewMode = options.startView;
    } else if ('startView' in this.element.data()) {
      this.startViewMode = this.element.data('start-view');
    }
    this.startViewMode = DPGlobal.convertViewMode(this.startViewMode);
    this.viewMode = this.startViewMode;

    this.viewSelect = this.minView;
    if ('viewSelect' in options) {
      this.viewSelect = options.viewSelect;
    } else if ('viewSelect' in this.element.data()) {
      this.viewSelect = this.element.data('view-select');
    }
    this.viewSelect = DPGlobal.convertViewMode(this.viewSelect);

    this.forceParse = true;
    if ('forceParse' in options) {
      this.forceParse = options.forceParse;
    } else if ('dateForceParse' in this.element.data()) {
      this.forceParse = this.element.data('date-force-parse');
    }
    var template = this.bootcssVer === 3 ? DPGlobal.templateV3 : DPGlobal.template;
    while (template.indexOf('{iconType}') !== -1) {
      template = template.replace('{iconType}', this.icontype);
    }
    while (template.indexOf('{leftArrow}') !== -1) {
      template = template.replace('{leftArrow}', this.icons.leftArrow);
    }
    while (template.indexOf('{rightArrow}') !== -1) {
      template = template.replace('{rightArrow}', this.icons.rightArrow);
    }
    this.picker = $(template)
      .appendTo(this.isInline ? this.element : this.container) // 'body')
      .on({
        click: $.proxy(this.click, this),
        mousedown: $.proxy(this.mousedown, this)
      });

    if (this.wheelViewModeNavigation) {
      if ($.fn.mousewheel) {
        this.picker.on({ mousewheel: $.proxy(this.mousewheel, this) });
      } else {
        console.log('Mouse Wheel event is not supported. Please include the jQuery Mouse Wheel plugin before enabling this option');
      }
    }

    if (this.isInline) {
      this.picker.addClass('datetimepicker-inline');
    } else {
      this.picker.addClass('datetimepicker-dropdown-' + this.pickerPosition + ' dropdown-menu');
    }
    if (this.isRTL) {
      this.picker.addClass('datetimepicker-rtl');
      var selector = this.bootcssVer === 3 ? '.prev span, .next span' : '.prev i, .next i';
      this.picker.find(selector).toggleClass(this.icons.leftArrow + ' ' + this.icons.rightArrow);
    }

    $(document).on('mousedown touchend', this.clickedOutside);

    this.autoclose = false;
    if ('autoclose' in options) {
      this.autoclose = options.autoclose;
    } else if ('dateAutoclose' in this.element.data()) {
      this.autoclose = this.element.data('date-autoclose');
    }

    this.keyboardNavigation = true;
    if ('keyboardNavigation' in options) {
      this.keyboardNavigation = options.keyboardNavigation;
    } else if ('dateKeyboardNavigation' in this.element.data()) {
      this.keyboardNavigation = this.element.data('date-keyboard-navigation');
    }

    this.todayBtn = (options.todayBtn || this.element.data('date-today-btn') || false);
    this.clearBtn = (options.clearBtn || this.element.data('date-clear-btn') || false);
    this.todayHighlight = (options.todayHighlight || this.element.data('date-today-highlight') || false);

    this.weekStart = 0;
    if (typeof options.weekStart !== 'undefined') {
      this.weekStart = options.weekStart;
    } else if (typeof this.element.data('date-weekstart') !== 'undefined') {
      this.weekStart = this.element.data('date-weekstart');
    } else if (typeof dates[this.language].weekStart !== 'undefined') {
      this.weekStart = dates[this.language].weekStart;
    }
    this.weekStart = this.weekStart % 7;
    this.weekEnd = ((this.weekStart + 6) % 7);
    this.onRenderDay = function (date) {
      var render = (options.onRenderDay || function () { return []; })(date);
      if (typeof render === 'string') {
        render = [render];
      }
      var res = ['day'];
      return res.concat((render ? render : []));
    };
    this.onRenderHour = function (date) {
      var render = (options.onRenderHour || function () { return []; })(date);
      var res = ['hour'];
      if (typeof render === 'string') {
        render = [render];
      }
      return res.concat((render ? render : []));
    };
    this.onRenderMinute = function (date) {
      var render = (options.onRenderMinute || function () { return []; })(date);
      var res = ['minute'];
      if (typeof render === 'string') {
        render = [render];
      }
      if (date < this.startDate || date > this.endDate) {
        res.push('disabled');
      } else if (Math.floor(this.date.getUTCMinutes() / this.minuteStep) === Math.floor(date.getUTCMinutes() / this.minuteStep)) {
        res.push('active');
      }
      return res.concat((render ? render : []));
    };
    this.onRenderYear = function (date) {
      var render = (options.onRenderYear || function () { return []; })(date);
      var res = ['year'];
      if (typeof render === 'string') {
        render = [render];
      }
      if (this.date.getUTCFullYear() === date.getUTCFullYear()) {
        res.push('active');
      }
      var currentYear = date.getUTCFullYear();
      var endYear = this.endDate.getUTCFullYear();
      if (date < this.startDate || currentYear > endYear) {
        res.push('disabled');
      }
      return res.concat((render ? render : []));
    }
    this.onRenderMonth = function (date) {
      var render = (options.onRenderMonth || function () { return []; })(date);
      var res = ['month'];
      if (typeof render === 'string') {
        render = [render];
      }
      return res.concat((render ? render : []));
    }
    this.startDate = new Date(-8639968443048000);
    this.endDate = new Date(8639968443048000);
    this.datesDisabled = [];
    this.daysOfWeekDisabled = [];
    this.setStartDate(options.startDate || this.element.data('date-startdate'));
    this.setEndDate(options.endDate || this.element.data('date-enddate'));
    this.setDatesDisabled(options.datesDisabled || this.element.data('date-dates-disabled'));
    this.setDaysOfWeekDisabled(options.daysOfWeekDisabled || this.element.data('date-days-of-week-disabled'));
    this.setMinutesDisabled(options.minutesDisabled || this.element.data('date-minute-disabled'));
    this.setHoursDisabled(options.hoursDisabled || this.element.data('date-hour-disabled'));
    this.fillDow();
    this.fillMonths();
    this.update();
    this.showMode();

    if (this.isInline) {
      this.show();
    }
  };

  Datetimepicker.prototype = {
    constructor: Datetimepicker,

    _events: [],
    _attachEvents: function () {
      this._detachEvents();
      if (this.isInput) { // single input
        this._events = [
          [this.element, {
            focus: $.proxy(this.show, this),
            keyup: $.proxy(this.update, this),
            keydown: $.proxy(this.keydown, this)
          }]
        ];
      }
      else if (this.component && this.hasInput) { // component: input + button
        this._events = [
          // For components that are not readonly, allow keyboard nav
          [this.element.find('input'), {
            focus: $.proxy(this.show, this),
            keyup: $.proxy(this.update, this),
            keydown: $.proxy(this.keydown, this)
          }],
          [this.component, {
            click: $.proxy(this.show, this)
          }]
        ];
        if (this.componentReset) {
          this._events.push([
            this.componentReset,
            { click: $.proxy(this.reset, this) }
          ]);
        }
      }
      else if (this.element.is('div')) {  // inline datetimepicker
        this.isInline = true;
      }
      else {
        this._events = [
          [this.element, {
            click: $.proxy(this.show, this)
          }]
        ];
      }
      for (var i = 0, el, ev; i < this._events.length; i++) {
        el = this._events[i][0];
        ev = this._events[i][1];
        el.on(ev);
      }
    },

    _detachEvents: function () {
      for (var i = 0, el, ev; i < this._events.length; i++) {
        el = this._events[i][0];
        ev = this._events[i][1];
        el.off(ev);
      }
      this._events = [];
    },

    show: function (e) {
      this.picker.show();
      this.height = this.component ? this.component.outerHeight() : this.element.outerHeight();
      if (this.forceParse) {
        this.update();
      }
      this.place();
      $(window).on('resize', $.proxy(this.place, this));
      if (e) {
        e.stopPropagation();
        e.preventDefault();
      }
      this.isVisible = true;
      this.element.trigger({
        type: 'show',
        date: this.date
      });
    },

    hide: function () {
      if (!this.isVisible) return;
      if (this.isInline) return;
      this.picker.hide();
      $(window).off('resize', this.place);
      this.viewMode = this.startViewMode;
      this.showMode();
      if (!this.isInput) {
        $(document).off('mousedown', this.hide);
      }

      if (
        this.forceParse &&
        (
          this.isInput && this.element.val() ||
          this.hasInput && this.element.find('input').val()
        )
      )
        this.setValue();
      this.isVisible = false;
      this.element.trigger({
        type: 'hide',
        date: this.date
      });
    },

    remove: function () {
      this._detachEvents();
      $(document).off('mousedown', this.clickedOutside);
      this.picker.remove();
      delete this.picker;
      delete this.element.data().datetimepicker;
    },

    getDate: function () {
      var d = this.getUTCDate();
      if (d === null) {
        return null;
      }
      return new Date(d.getTime() + (d.getTimezoneOffset() * 60000));
    },

    getUTCDate: function () {
      return this.date;
    },

    getInitialDate: function () {
      return this.initialDate
    },

    setInitialDate: function (initialDate) {
      this.initialDate = initialDate;
    },

    setDate: function (d) {
      this.setUTCDate(new Date(d.getTime() - (d.getTimezoneOffset() * 60000)));
    },

    setUTCDate: function (d) {
      if (d >= this.startDate && d <= this.endDate) {
        this.date = d;
        this.setValue();
        this.viewDate = this.date;
        this.fill();
      } else {
        this.element.trigger({
          type: 'outOfRange',
          date: d,
          startDate: this.startDate,
          endDate: this.endDate
        });
      }
    },

    setFormat: function (format) {
      this.format = DPGlobal.parseFormat(format, this.formatType);
      var element;
      if (this.isInput) {
        element = this.element;
      } else if (this.component) {
        element = this.element.find('input');
      }
      if (element && element.val()) {
        this.setValue();
      }
    },

    setValue: function () {
      var formatted = this.getFormattedDate();
      if (!this.isInput) {
        if (this.component) {
          this.element.find('input').val(formatted);
        }
        this.element.data('date', formatted);
      } else {
        this.element.val(formatted);
      }
      if (this.linkField) {
        $('#' + this.linkField).val(this.getFormattedDate(this.linkFormat));
      }
    },

    getFormattedDate: function (format) {
      format = format || this.format;
      return DPGlobal.formatDate(this.date, format, this.language, this.formatType, this.timezone);
    },

    setStartDate: function (startDate) {
      this.startDate = startDate || this.startDate;
      if (this.startDate.valueOf() !== 8639968443048000) {
        this.startDate = DPGlobal.parseDate(this.startDate, this.format, this.language, this.formatType, this.timezone);
      }
      this.update();
      this.updateNavArrows();
    },

    setEndDate: function (endDate) {
      this.endDate = endDate || this.endDate;
      if (this.endDate.valueOf() !== 8639968443048000) {
        this.endDate = DPGlobal.parseDate(this.endDate, this.format, this.language, this.formatType, this.timezone);
      }
      this.update();
      this.updateNavArrows();
    },

    setDatesDisabled: function (datesDisabled) {
      this.datesDisabled = datesDisabled || [];
      if (!$.isArray(this.datesDisabled)) {
        this.datesDisabled = this.datesDisabled.split(/,\s*/);
      }
      var mThis = this;
      this.datesDisabled = $.map(this.datesDisabled, function (d) {
        return DPGlobal.parseDate(d, mThis.format, mThis.language, mThis.formatType, mThis.timezone).toDateString();
      });
      this.update();
      this.updateNavArrows();
    },

    setTitle: function (selector, value) {
      return this.picker.find(selector)
        .find('th:eq(1)')
        .text(this.title === false ? value : this.title);
    },

    setDaysOfWeekDisabled: function (daysOfWeekDisabled) {
      this.daysOfWeekDisabled = daysOfWeekDisabled || [];
      if (!$.isArray(this.daysOfWeekDisabled)) {
        this.daysOfWeekDisabled = this.daysOfWeekDisabled.split(/,\s*/);
      }
      this.daysOfWeekDisabled = $.map(this.daysOfWeekDisabled, function (d) {
        return parseInt(d, 10);
      });
      this.update();
      this.updateNavArrows();
    },

    setMinutesDisabled: function (minutesDisabled) {
      this.minutesDisabled = minutesDisabled || [];
      if (!$.isArray(this.minutesDisabled)) {
        this.minutesDisabled = this.minutesDisabled.split(/,\s*/);
      }
      this.minutesDisabled = $.map(this.minutesDisabled, function (d) {
        return parseInt(d, 10);
      });
      this.update();
      this.updateNavArrows();
    },

    setHoursDisabled: function (hoursDisabled) {
      this.hoursDisabled = hoursDisabled || [];
      if (!$.isArray(this.hoursDisabled)) {
        this.hoursDisabled = this.hoursDisabled.split(/,\s*/);
      }
      this.hoursDisabled = $.map(this.hoursDisabled, function (d) {
        return parseInt(d, 10);
      });
      this.update();
      this.updateNavArrows();
    },

    place: function () {
      if (this.isInline) return;

      if (!this.zIndex) {
        var index_highest = 0;
        $('div').each(function () {
          var index_current = parseInt($(this).css('zIndex'), 10);
          if (index_current > index_highest) {
            index_highest = index_current;
          }
        });
        this.zIndex = index_highest + 10;
      }

      var offset, top, left, containerOffset;
      if (this.container instanceof $) {
        containerOffset = this.container.offset();
      } else {
        containerOffset = $(this.container).offset();
      }

      if (this.component) {
        offset = this.component.offset();
        left = offset.left;
        if (this.pickerPosition === 'bottom-left' || this.pickerPosition === 'top-left') {
          left += this.component.outerWidth() - this.picker.outerWidth();
        }
      } else {
        offset = this.element.offset();
        left = offset.left;
        if (this.pickerPosition === 'bottom-left' || this.pickerPosition === 'top-left') {
          left += this.element.outerWidth() - this.picker.outerWidth();
        }
      }

      var bodyWidth = document.body.clientWidth || window.innerWidth;
      if (left + 220 > bodyWidth) {
        left = bodyWidth - 220;
      }

      if (this.pickerPosition === 'top-left' || this.pickerPosition === 'top-right') {
        top = offset.top - this.picker.outerHeight();
      } else {
        top = offset.top + this.height;
      }

      top = top - containerOffset.top;
      left = left - containerOffset.left;

      this.picker.css({
        top: top,
        left: left,
        zIndex: this.zIndex
      });
    },

    hour_minute: "^([0-9]|0[0-9]|1[0-9]|2[0-3]):[0-5][0-9]",

    update: function () {
      var date, fromArgs = false;
      if (arguments && arguments.length && (typeof arguments[0] === 'string' || arguments[0] instanceof Date)) {
        date = arguments[0];
        fromArgs = true;
      } else {
        date = (this.isInput ? this.element.val() : this.element.find('input').val()) || this.element.data('date') || this.initialDate;
        if (typeof date === 'string') {
          date = date.replace(/^\s+|\s+$/g, '');
        }
      }

      if (!date) {
        date = new Date();
        fromArgs = false;
      }

      if (typeof date === "string") {
        if (new RegExp(this.hour_minute).test(date) || new RegExp(this.hour_minute + ":[0-5][0-9]").test(date)) {
          date = this.getDate()
        }
      }

      this.date = DPGlobal.parseDate(date, this.format, this.language, this.formatType, this.timezone);

      if (fromArgs) this.setValue();

      if (this.date < this.startDate) {
        this.viewDate = new Date(this.startDate);
      } else if (this.date > this.endDate) {
        this.viewDate = new Date(this.endDate);
      } else {
        this.viewDate = new Date(this.date);
      }
      this.fill();
    },

    fillDow: function () {
      var dowCnt = this.weekStart,
        html = '<tr>';
      while (dowCnt < this.weekStart + 7) {
        html += '<th class="dow">' + dates[this.language].daysMin[(dowCnt++) % 7] + '</th>';
      }
      html += '</tr>';
      this.picker.find('.datetimepicker-days thead').append(html);
    },

    fillMonths: function () {
      var html = '';
      var d = new Date(this.viewDate);
      for (var i = 0; i < 12; i++) {
        d.setUTCMonth(i);
        var classes = this.onRenderMonth(d);
        html += '<span class="' + classes.join(' ') + '">' + dates[this.language].monthsShort[i] + '</span>';
      }
      this.picker.find('.datetimepicker-months td').html(html);
    },

    fill: function () {
      if (!this.date || !this.viewDate) {
        return;
      }
      var d = new Date(this.viewDate),
        year = d.getUTCFullYear(),
        month = d.getUTCMonth(),
        dayMonth = d.getUTCDate(),
        hours = d.getUTCHours(),
        startYear = this.startDate.getUTCFullYear(),
        startMonth = this.startDate.getUTCMonth(),
        endYear = this.endDate.getUTCFullYear(),
        endMonth = this.endDate.getUTCMonth() + 1,
        currentDate = (new UTCDate(this.date.getUTCFullYear(), this.date.getUTCMonth(), this.date.getUTCDate())).valueOf(),
        today = new Date();
      this.setTitle('.datetimepicker-days', dates[this.language].months[month] + ' ' + year)
      if (this.formatViewType === 'time') {
        var formatted = this.getFormattedDate();
        this.setTitle('.datetimepicker-hours', formatted);
        this.setTitle('.datetimepicker-minutes', formatted);
      } else {
        this.setTitle('.datetimepicker-hours', dayMonth + ' ' + dates[this.language].months[month] + ' ' + year);
        this.setTitle('.datetimepicker-minutes', dayMonth + ' ' + dates[this.language].months[month] + ' ' + year);
      }
      this.picker.find('tfoot th.today')
        .text(dates[this.language].today || dates['en'].today)
        .toggle(this.todayBtn !== false);
      this.picker.find('tfoot th.clear')
        .text(dates[this.language].clear || dates['en'].clear)
        .toggle(this.clearBtn !== false);
      this.updateNavArrows();
      this.fillMonths();
      var prevMonth = UTCDate(year, month - 1, 28, 0, 0, 0, 0),
        day = DPGlobal.getDaysInMonth(prevMonth.getUTCFullYear(), prevMonth.getUTCMonth());
      prevMonth.setUTCDate(day);
      prevMonth.setUTCDate(day - (prevMonth.getUTCDay() - this.weekStart + 7) % 7);
      var nextMonth = new Date(prevMonth);
      nextMonth.setUTCDate(nextMonth.getUTCDate() + 42);
      nextMonth = nextMonth.valueOf();
      var html = [];
      var classes;
      while (prevMonth.valueOf() < nextMonth) {
        if (prevMonth.getUTCDay() === this.weekStart) {
          html.push('<tr>');
        }
        classes = this.onRenderDay(prevMonth);
        if (prevMonth.getUTCFullYear() < year || (prevMonth.getUTCFullYear() === year && prevMonth.getUTCMonth() < month)) {
          classes.push('old');
        } else if (prevMonth.getUTCFullYear() > year || (prevMonth.getUTCFullYear() === year && prevMonth.getUTCMonth() > month)) {
          classes.push('new');
        }
        // Compare internal UTC date with local today, not UTC today
        if (this.todayHighlight &&
          prevMonth.getUTCFullYear() === today.getFullYear() &&
          prevMonth.getUTCMonth() === today.getMonth() &&
          prevMonth.getUTCDate() === today.getDate()) {
          classes.push('today');
        }
        if (prevMonth.valueOf() === currentDate) {
          classes.push('active');
        }
        if ((prevMonth.valueOf() + 86400000) <= this.startDate || prevMonth.valueOf() > this.endDate ||
          $.inArray(prevMonth.getUTCDay(), this.daysOfWeekDisabled) !== -1 ||
          $.inArray(prevMonth.toDateString(), this.datesDisabled) !== -1) {
          classes.push('disabled');
        }
        html.push('<td class="' + classes.join(' ') + '">' + prevMonth.getUTCDate() + '</td>');
        if (prevMonth.getUTCDay() === this.weekEnd) {
          html.push('</tr>');
        }
        prevMonth.setUTCDate(prevMonth.getUTCDate() + 1);
      }
      this.picker.find('.datetimepicker-days tbody').empty().append(html.join(''));

      html = [];
      var txt = '', meridian = '', meridianOld = '';
      var hoursDisabled = this.hoursDisabled || [];
      d = new Date(this.viewDate)
      for (var i = 0; i < 24; i++) {
        d.setUTCHours(i);
        classes = this.onRenderHour(d);
        if (hoursDisabled.indexOf(i) !== -1) {
          classes.push('disabled');
        }
        var actual = UTCDate(year, month, dayMonth, i);
        // We want the previous hour for the startDate
        if ((actual.valueOf() + 3600000) <= this.startDate || actual.valueOf() > this.endDate) {
          classes.push('disabled');
        } else if (hours === i) {
          classes.push('active');
        }
        if (this.showMeridian && dates[this.language].meridiem.length === 2) {
          meridian = (i < 12 ? dates[this.language].meridiem[0] : dates[this.language].meridiem[1]);
          if (meridian !== meridianOld) {
            if (meridianOld !== '') {
              html.push('</fieldset>');
            }
            html.push('<fieldset class="hour"><legend>' + meridian.toUpperCase() + '</legend>');
          }
          meridianOld = meridian;
          txt = (i % 12 ? i % 12 : 12);
          if (i < 12) {
            classes.push('hour_am');
          } else {
            classes.push('hour_pm');
          }
          html.push('<span class="' + classes.join(' ') + '">' + txt + '</span>');
          if (i === 23) {
            html.push('</fieldset>');
          }
        } else {
          txt = i + ':00';
          html.push('<span class="' + classes.join(' ') + '">' + txt + '</span>');
        }
      }
      this.picker.find('.datetimepicker-hours td').html(html.join(''));

      html = [];
      txt = '';
      meridian = '';
      meridianOld = '';
      var minutesDisabled = this.minutesDisabled || [];
      d = new Date(this.viewDate);
      for (var i = 0; i < 60; i += this.minuteStep) {
        if (minutesDisabled.indexOf(i) !== -1) continue;
        d.setUTCMinutes(i);
        d.setUTCSeconds(0);
        classes = this.onRenderMinute(d);
        if (this.showMeridian && dates[this.language].meridiem.length === 2) {
          meridian = (hours < 12 ? dates[this.language].meridiem[0] : dates[this.language].meridiem[1]);
          if (meridian !== meridianOld) {
            if (meridianOld !== '') {
              html.push('</fieldset>');
            }
            html.push('<fieldset class="minute"><legend>' + meridian.toUpperCase() + '</legend>');
          }
          meridianOld = meridian;
          txt = (hours % 12 ? hours % 12 : 12);
          html.push('<span class="' + classes.join(' ') + '">' + txt + ':' + (i < 10 ? '0' + i : i) + '</span>');
          if (i === 59) {
            html.push('</fieldset>');
          }
        } else {
          txt = i + ':00';
          html.push('<span class="' + classes.join(' ') + '">' + hours + ':' + (i < 10 ? '0' + i : i) + '</span>');
        }
      }
      this.picker.find('.datetimepicker-minutes td').html(html.join(''));

      var currentYear = this.date.getUTCFullYear();
      var months = this.setTitle('.datetimepicker-months', year)
        .end()
        .find('.month').removeClass('active');
      if (currentYear === year) {
        // getUTCMonths() returns 0 based, and we need to select the next one
        // To cater bootstrap 2 we don't need to select the next one
        months.eq(this.date.getUTCMonth()).addClass('active');
      }
      if (year < startYear || year > endYear) {
        months.addClass('disabled');
      }
      if (year === startYear) {
        months.slice(0, startMonth).addClass('disabled');
      }
      if (year === endYear) {
        months.slice(endMonth).addClass('disabled');
      }

      html = '';
      year = parseInt(year / 10, 10) * 10;
      var yearCont = this.setTitle('.datetimepicker-years', year + '-' + (year + 9))
        .end()
        .find('td');
      year -= 1;
      d = new Date(this.viewDate);
      for (var i = -1; i < 11; i++) {
        d.setUTCFullYear(year);
        classes = this.onRenderYear(d);
        if (i === -1 || i === 10) {
          classes.push(old);
        }
        html += '<span class="' + classes.join(' ') + '">' + year + '</span>';
        year += 1;
      }
      yearCont.html(html);
      this.place();
    },

    updateNavArrows: function () {
      var d = new Date(this.viewDate),
        year = d.getUTCFullYear(),
        month = d.getUTCMonth(),
        day = d.getUTCDate(),
        hour = d.getUTCHours();
      switch (this.viewMode) {
        case 0:
          if (year <= this.startDate.getUTCFullYear()
            && month <= this.startDate.getUTCMonth()
            && day <= this.startDate.getUTCDate()
            && hour <= this.startDate.getUTCHours()) {
            this.picker.find('.prev').css({ visibility: 'hidden' });
          } else {
            this.picker.find('.prev').css({ visibility: 'visible' });
          }
          if (year >= this.endDate.getUTCFullYear()
            && month >= this.endDate.getUTCMonth()
            && day >= this.endDate.getUTCDate()
            && hour >= this.endDate.getUTCHours()) {
            this.picker.find('.next').css({ visibility: 'hidden' });
          } else {
            this.picker.find('.next').css({ visibility: 'visible' });
          }
          break;
        case 1:
          if (year <= this.startDate.getUTCFullYear()
            && month <= this.startDate.getUTCMonth()
            && day <= this.startDate.getUTCDate()) {
            this.picker.find('.prev').css({ visibility: 'hidden' });
          } else {
            this.picker.find('.prev').css({ visibility: 'visible' });
          }
          if (year >= this.endDate.getUTCFullYear()
            && month >= this.endDate.getUTCMonth()
            && day >= this.endDate.getUTCDate()) {
            this.picker.find('.next').css({ visibility: 'hidden' });
          } else {
            this.picker.find('.next').css({ visibility: 'visible' });
          }
          break;
        case 2:
          if (year <= this.startDate.getUTCFullYear()
            && month <= this.startDate.getUTCMonth()) {
            this.picker.find('.prev').css({ visibility: 'hidden' });
          } else {
            this.picker.find('.prev').css({ visibility: 'visible' });
          }
          if (year >= this.endDate.getUTCFullYear()
            && month >= this.endDate.getUTCMonth()) {
            this.picker.find('.next').css({ visibility: 'hidden' });
          } else {
            this.picker.find('.next').css({ visibility: 'visible' });
          }
          break;
        case 3:
        case 4:
          if (year <= this.startDate.getUTCFullYear()) {
            this.picker.find('.prev').css({ visibility: 'hidden' });
          } else {
            this.picker.find('.prev').css({ visibility: 'visible' });
          }
          if (year >= this.endDate.getUTCFullYear()) {
            this.picker.find('.next').css({ visibility: 'hidden' });
          } else {
            this.picker.find('.next').css({ visibility: 'visible' });
          }
          break;
      }
    },

    mousewheel: function (e) {

      e.preventDefault();
      e.stopPropagation();

      if (this.wheelPause) {
        return;
      }

      this.wheelPause = true;

      var originalEvent = e.originalEvent;

      var delta = originalEvent.wheelDelta;

      var mode = delta > 0 ? 1 : (delta === 0) ? 0 : -1;

      if (this.wheelViewModeNavigationInverseDirection) {
        mode = -mode;
      }

      this.showMode(mode);

      setTimeout($.proxy(function () {

        this.wheelPause = false

      }, this), this.wheelViewModeNavigationDelay);

    },

    click: function (e) {
      e.stopPropagation();
      e.preventDefault();
      var target = $(e.target).closest('span, td, th, legend');
      if (target.is('.' + this.icontype)) {
        target = $(target).parent().closest('span, td, th, legend');
      }
      if (target.length === 1) {
        if (target.is('.disabled')) {
          this.element.trigger({
            type: 'outOfRange',
            date: this.viewDate,
            startDate: this.startDate,
            endDate: this.endDate
          });
          return;
        }
        switch (target[0].nodeName.toLowerCase()) {
          case 'th':
            switch (target[0].className) {
              case 'switch':
                this.showMode(1);
                break;
              case 'prev':
              case 'next':
                var dir = DPGlobal.modes[this.viewMode].navStep * (target[0].className === 'prev' ? -1 : 1);
                switch (this.viewMode) {
                  case 0:
                    this.viewDate = this.moveHour(this.viewDate, dir);
                    break;
                  case 1:
                    this.viewDate = this.moveDate(this.viewDate, dir);
                    break;
                  case 2:
                    this.viewDate = this.moveMonth(this.viewDate, dir);
                    break;
                  case 3:
                  case 4:
                    this.viewDate = this.moveYear(this.viewDate, dir);
                    break;
                }
                this.fill();
                this.element.trigger({
                  type: target[0].className + ':' + this.convertViewModeText(this.viewMode),
                  date: this.viewDate,
                  startDate: this.startDate,
                  endDate: this.endDate
                });
                break;
              case 'clear':
                this.reset();
                if (this.autoclose) {
                  this.hide();
                }
                break;
              case 'today':
                var date = new Date();
                date = UTCDate(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours(), date.getMinutes(), date.getSeconds(), 0);

                // Respect startDate and endDate.
                if (date < this.startDate) date = this.startDate;
                else if (date > this.endDate) date = this.endDate;

                this.viewMode = this.startViewMode;
                this.showMode(0);
                this._setDate(date);
                this.fill();
                if (this.autoclose) {
                  this.hide();
                }
                break;
            }
            break;
          case 'span':
            if (!target.is('.disabled')) {
              var year = this.viewDate.getUTCFullYear(),
                month = this.viewDate.getUTCMonth(),
                day = this.viewDate.getUTCDate(),
                hours = this.viewDate.getUTCHours(),
                minutes = this.viewDate.getUTCMinutes(),
                seconds = this.viewDate.getUTCSeconds();

              if (target.is('.month')) {
                this.viewDate.setUTCDate(1);
                month = target.parent().find('span').index(target);
                day = this.viewDate.getUTCDate();
                this.viewDate.setUTCMonth(month);
                this.element.trigger({
                  type: 'changeMonth',
                  date: this.viewDate
                });
                if (this.viewSelect >= 3) {
                  this._setDate(UTCDate(year, month, day, hours, minutes, seconds, 0));
                }
              } else if (target.is('.year')) {
                this.viewDate.setUTCDate(1);
                year = parseInt(target.text(), 10) || 0;
                this.viewDate.setUTCFullYear(year);
                this.element.trigger({
                  type: 'changeYear',
                  date: this.viewDate
                });
                if (this.viewSelect >= 4) {
                  this._setDate(UTCDate(year, month, day, hours, minutes, seconds, 0));
                }
              } else if (target.is('.hour')) {
                hours = parseInt(target.text(), 10) || 0;
                if (target.hasClass('hour_am') || target.hasClass('hour_pm')) {
                  if (hours === 12 && target.hasClass('hour_am')) {
                    hours = 0;
                  } else if (hours !== 12 && target.hasClass('hour_pm')) {
                    hours += 12;
                  }
                }
                this.viewDate.setUTCHours(hours);
                this.element.trigger({
                  type: 'changeHour',
                  date: this.viewDate
                });
                if (this.viewSelect >= 1) {
                  this._setDate(UTCDate(year, month, day, hours, minutes, seconds, 0));
                }
              } else if (target.is('.minute')) {
                minutes = parseInt(target.text().substr(target.text().indexOf(':') + 1), 10) || 0;
                this.viewDate.setUTCMinutes(minutes);
                this.element.trigger({
                  type: 'changeMinute',
                  date: this.viewDate
                });
                if (this.viewSelect >= 0) {
                  this._setDate(UTCDate(year, month, day, hours, minutes, seconds, 0));
                }
              }
              if (this.viewMode !== 0) {
                var oldViewMode = this.viewMode;
                this.showMode(-1);
                this.fill();
                if (oldViewMode === this.viewMode && this.autoclose) {
                  this.hide();
                }
              } else {
                this.fill();
                if (this.autoclose) {
                  this.hide();
                }
              }
            }
            break;
          case 'td':
            if (target.is('.day') && !target.is('.disabled')) {
              var day = parseInt(target.text(), 10) || 1;
              var year = this.viewDate.getUTCFullYear(),
                month = this.viewDate.getUTCMonth(),
                hours = this.viewDate.getUTCHours(),
                minutes = this.viewDate.getUTCMinutes(),
                seconds = this.viewDate.getUTCSeconds();
              if (target.is('.old')) {
                if (month === 0) {
                  month = 11;
                  year -= 1;
                } else {
                  month -= 1;
                }
              } else if (target.is('.new')) {
                if (month === 11) {
                  month = 0;
                  year += 1;
                } else {
                  month += 1;
                }
              }
              this.viewDate.setUTCFullYear(year);
              this.viewDate.setUTCMonth(month, day);
              this.element.trigger({
                type: 'changeDay',
                date: this.viewDate
              });
              if (this.viewSelect >= 2) {
                this._setDate(UTCDate(year, month, day, hours, minutes, seconds, 0));
              }
            }
            var oldViewMode = this.viewMode;
            this.showMode(-1);
            this.fill();
            if (oldViewMode === this.viewMode && this.autoclose) {
              this.hide();
            }
            break;
        }
      }
    },

    _setDate: function (date, which) {
      if (!which || which === 'date')
        this.date = date;
      if (!which || which === 'view')
        this.viewDate = date;
      this.fill();
      this.setValue();
      var element;
      if (this.isInput) {
        element = this.element;
      } else if (this.component) {
        element = this.element.find('input');
      }
      if (element) {
        element.change();
      }
      this.element.trigger({
        type: 'changeDate',
        date: this.getDate()
      });
      if (date === null)
        this.date = this.viewDate;
    },

    moveMinute: function (date, dir) {
      if (!dir) return date;
      var new_date = new Date(date.valueOf());
      //dir = dir > 0 ? 1 : -1;
      new_date.setUTCMinutes(new_date.getUTCMinutes() + (dir * this.minuteStep));
      return new_date;
    },

    moveHour: function (date, dir) {
      if (!dir) return date;
      var new_date = new Date(date.valueOf());
      //dir = dir > 0 ? 1 : -1;
      new_date.setUTCHours(new_date.getUTCHours() + dir);
      return new_date;
    },

    moveDate: function (date, dir) {
      if (!dir) return date;
      var new_date = new Date(date.valueOf());
      //dir = dir > 0 ? 1 : -1;
      new_date.setUTCDate(new_date.getUTCDate() + dir);
      return new_date;
    },

    moveMonth: function (date, dir) {
      if (!dir) return date;
      var new_date = new Date(date.valueOf()),
        day = new_date.getUTCDate(),
        month = new_date.getUTCMonth(),
        mag = Math.abs(dir),
        new_month, test;
      dir = dir > 0 ? 1 : -1;
      if (mag === 1) {
        test = dir === -1
          // If going back one month, make sure month is not current month
          // (eg, Mar 31 -> Feb 31 === Feb 28, not Mar 02)
          ? function () {
            return new_date.getUTCMonth() === month;
          }
          // If going forward one month, make sure month is as expected
          // (eg, Jan 31 -> Feb 31 === Feb 28, not Mar 02)
          : function () {
            return new_date.getUTCMonth() !== new_month;
          };
        new_month = month + dir;
        new_date.setUTCMonth(new_month);
        // Dec -> Jan (12) or Jan -> Dec (-1) -- limit expected date to 0-11
        if (new_month < 0 || new_month > 11)
          new_month = (new_month + 12) % 12;
      } else {
        // For magnitudes >1, move one month at a time...
        for (var i = 0; i < mag; i++)
          // ...which might decrease the day (eg, Jan 31 to Feb 28, etc)...
          new_date = this.moveMonth(new_date, dir);
        // ...then reset the day, keeping it in the new month
        new_month = new_date.getUTCMonth();
        new_date.setUTCDate(day);
        test = function () {
          return new_month !== new_date.getUTCMonth();
        };
      }
      // Common date-resetting loop -- if date is beyond end of month, make it
      // end of month
      while (test()) {
        new_date.setUTCDate(--day);
        new_date.setUTCMonth(new_month);
      }
      return new_date;
    },

    moveYear: function (date, dir) {
      return this.moveMonth(date, dir * 12);
    },

    dateWithinRange: function (date) {
      return date >= this.startDate && date <= this.endDate;
    },

    keydown: function (e) {
      if (this.picker.is(':not(:visible)')) {
        if (e.keyCode === 27) // allow escape to hide and re-show picker
          this.show();
        return;
      }
      var dateChanged = false,
        dir, newDate, newViewDate;
      switch (e.keyCode) {
        case 27: // escape
          this.hide();
          e.preventDefault();
          break;
        case 37: // left
        case 39: // right
          if (!this.keyboardNavigation) break;
          dir = e.keyCode === 37 ? -1 : 1;
          var viewMode = this.viewMode;
          if (e.ctrlKey) {
            viewMode += 2;
          } else if (e.shiftKey) {
            viewMode += 1;
          }
          if (viewMode === 4) {
            newDate = this.moveYear(this.date, dir);
            newViewDate = this.moveYear(this.viewDate, dir);
          } else if (viewMode === 3) {
            newDate = this.moveMonth(this.date, dir);
            newViewDate = this.moveMonth(this.viewDate, dir);
          } else if (viewMode === 2) {
            newDate = this.moveDate(this.date, dir);
            newViewDate = this.moveDate(this.viewDate, dir);
          } else if (viewMode === 1) {
            newDate = this.moveHour(this.date, dir);
            newViewDate = this.moveHour(this.viewDate, dir);
          } else if (viewMode === 0) {
            newDate = this.moveMinute(this.date, dir);
            newViewDate = this.moveMinute(this.viewDate, dir);
          }
          if (this.dateWithinRange(newDate)) {
            this.date = newDate;
            this.viewDate = newViewDate;
            this.setValue();
            this.update();
            e.preventDefault();
            dateChanged = true;
          }
          break;
        case 38: // up
        case 40: // down
          if (!this.keyboardNavigation) break;
          dir = e.keyCode === 38 ? -1 : 1;
          viewMode = this.viewMode;
          if (e.ctrlKey) {
            viewMode += 2;
          } else if (e.shiftKey) {
            viewMode += 1;
          }
          if (viewMode === 4) {
            newDate = this.moveYear(this.date, dir);
            newViewDate = this.moveYear(this.viewDate, dir);
          } else if (viewMode === 3) {
            newDate = this.moveMonth(this.date, dir);
            newViewDate = this.moveMonth(this.viewDate, dir);
          } else if (viewMode === 2) {
            newDate = this.moveDate(this.date, dir * 7);
            newViewDate = this.moveDate(this.viewDate, dir * 7);
          } else if (viewMode === 1) {
            if (this.showMeridian) {
              newDate = this.moveHour(this.date, dir * 6);
              newViewDate = this.moveHour(this.viewDate, dir * 6);
            } else {
              newDate = this.moveHour(this.date, dir * 4);
              newViewDate = this.moveHour(this.viewDate, dir * 4);
            }
          } else if (viewMode === 0) {
            newDate = this.moveMinute(this.date, dir * 4);
            newViewDate = this.moveMinute(this.viewDate, dir * 4);
          }
          if (this.dateWithinRange(newDate)) {
            this.date = newDate;
            this.viewDate = newViewDate;
            this.setValue();
            this.update();
            e.preventDefault();
            dateChanged = true;
          }
          break;
        case 13: // enter
          if (this.viewMode !== 0) {
            var oldViewMode = this.viewMode;
            this.showMode(-1);
            this.fill();
            if (oldViewMode === this.viewMode && this.autoclose) {
              this.hide();
            }
          } else {
            this.fill();
            if (this.autoclose) {
              this.hide();
            }
          }
          e.preventDefault();
          break;
        case 9: // tab
          this.hide();
          break;
      }
      if (dateChanged) {
        var element;
        if (this.isInput) {
          element = this.element;
        } else if (this.component) {
          element = this.element.find('input');
        }
        if (element) {
          element.change();
        }
        this.element.trigger({
          type: 'changeDate',
          date: this.getDate()
        });
      }
    },

    showMode: function (dir) {
      if (dir) {
        var newViewMode = Math.max(0, Math.min(DPGlobal.modes.length - 1, this.viewMode + dir));
        if (newViewMode >= this.minView && newViewMode <= this.maxView) {
          this.element.trigger({
            type: 'changeMode',
            date: this.viewDate,
            oldViewMode: this.viewMode,
            newViewMode: newViewMode
          });

          this.viewMode = newViewMode;
        }
      }
      /*
       vitalets: fixing bug of very special conditions:
       jquery 1.7.1 + webkit + show inline datetimepicker in bootstrap popover.
       Method show() does not set display css correctly and datetimepicker is not shown.
       Changed to .css('display', 'block') solve the problem.
       See https://github.com/vitalets/x-editable/issues/37

       In jquery 1.7.2+ everything works fine.
       */
      //this.picker.find('>div').hide().filter('.datetimepicker-'+DPGlobal.modes[this.viewMode].clsName).show();
      this.picker.find('>div').hide().filter('.datetimepicker-' + DPGlobal.modes[this.viewMode].clsName).css('display', 'block');
      this.updateNavArrows();
    },

    reset: function () {
      this._setDate(null, 'date');
    },

    convertViewModeText: function (viewMode) {
      switch (viewMode) {
        case 4:
          return 'decade';
        case 3:
          return 'year';
        case 2:
          return 'month';
        case 1:
          return 'day';
        case 0:
          return 'hour';
      }
    }
  };

  var old = $.fn.datetimepicker;
  $.fn.datetimepicker = function (option) {
    var args = Array.apply(null, arguments);
    args.shift();
    var internal_return;
    this.each(function () {
      var $this = $(this),
        data = $this.data('datetimepicker'),
        options = typeof option === 'object' && option;
      if (!data) {
        $this.data('datetimepicker', (data = new Datetimepicker(this, $.extend({}, $.fn.datetimepicker.defaults, options))));
      }
      if (typeof option === 'string' && typeof data[option] === 'function') {
        internal_return = data[option].apply(data, args);
        if (internal_return !== undefined) {
          return false;
        }
      }
    });
    if (internal_return !== undefined)
      return internal_return;
    else
      return this;
  };

  $.fn.datetimepicker.defaults = {
  };
  $.fn.datetimepicker.Constructor = Datetimepicker;
  var dates = $.fn.datetimepicker.dates = {
    en: {
      days: ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday'],
      daysShort: ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'],
      daysMin: ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa', 'Su'],
      months: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
      monthsShort: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'],
      meridiem: ['am', 'pm'],
      suffix: ['st', 'nd', 'rd', 'th'],
      today: 'Today',
      clear: 'Clear'
    }
  };

  var DPGlobal = {
    modes: [
      {
        clsName: 'minutes',
        navFnc: 'Hours',
        navStep: 1
      },
      {
        clsName: 'hours',
        navFnc: 'Date',
        navStep: 1
      },
      {
        clsName: 'days',
        navFnc: 'Month',
        navStep: 1
      },
      {
        clsName: 'months',
        navFnc: 'FullYear',
        navStep: 1
      },
      {
        clsName: 'years',
        navFnc: 'FullYear',
        navStep: 10
      }
    ],
    isLeapYear: function (year) {
      return (((year % 4 === 0) && (year % 100 !== 0)) || (year % 400 === 0))
    },
    getDaysInMonth: function (year, month) {
      return [31, (DPGlobal.isLeapYear(year) ? 29 : 28), 31, 30, 31, 30, 31, 31, 30, 31, 30, 31][month]
    },
    getDefaultFormat: function (type, field) {
      if (type === 'standard') {
        if (field === 'input')
          return 'yyyy-mm-dd hh:ii';
        else
          return 'yyyy-mm-dd hh:ii:ss';
      } else if (type === 'php') {
        if (field === 'input')
          return 'Y-m-d H:i';
        else
          return 'Y-m-d H:i:s';
      } else {
        throw new Error('Invalid format type.');
      }
    },
    validParts: function (type) {
      if (type === 'standard') {
        return /t|hh?|HH?|p|P|z|Z|ii?|ss?|dd?|DD?|mm?|MM?|yy(?:yy)?/g;
      } else if (type === 'php') {
        return /[dDjlNwzFmMnStyYaABgGhHis]/g;
      } else {
        throw new Error('Invalid format type.');
      }
    },
    nonpunctuation: /[^ -\/:-@\[-`{-~\t\n\rTZ]+/g,
    parseFormat: function (format, type) {
      // IE treats \0 as a string end in inputs (truncating the value),
      // so it's a bad format delimiter, anyway
      var separators = format.replace(this.validParts(type), '\0').split('\0'),
        parts = format.match(this.validParts(type));
      if (!separators || !separators.length || !parts || parts.length === 0) {
        throw new Error('Invalid date format.');
      }
      return { separators: separators, parts: parts };
    },
    parseDate: function (date, format, language, type, timezone) {
      if (date instanceof Date) {
        var dateUTC = new Date(date.valueOf() - date.getTimezoneOffset() * 60000);
        dateUTC.setMilliseconds(0);
        return dateUTC;
      }
      if (/^\d{4}\-\d{1,2}\-\d{1,2}$/.test(date)) {
        format = this.parseFormat('yyyy-mm-dd', type);
      }
      if (/^\d{4}\-\d{1,2}\-\d{1,2}[T ]\d{1,2}\:\d{1,2}$/.test(date)) {
        format = this.parseFormat('yyyy-mm-dd hh:ii', type);
      }
      if (/^\d{4}\-\d{1,2}\-\d{1,2}[T ]\d{1,2}\:\d{1,2}\:\d{1,2}[Z]{0,1}$/.test(date)) {
        format = this.parseFormat('yyyy-mm-dd hh:ii:ss', type);
      }
      if (/^[-+]\d+[dmwy]([\s,]+[-+]\d+[dmwy])*$/.test(date)) {
        var part_re = /([-+]\d+)([dmwy])/,
          parts = date.match(/([-+]\d+)([dmwy])/g),
          part, dir;
        date = new Date();
        for (var i = 0; i < parts.length; i++) {
          part = part_re.exec(parts[i]);
          dir = parseInt(part[1]);
          switch (part[2]) {
            case 'd':
              date.setUTCDate(date.getUTCDate() + dir);
              break;
            case 'm':
              date = Datetimepicker.prototype.moveMonth.call(Datetimepicker.prototype, date, dir);
              break;
            case 'w':
              date.setUTCDate(date.getUTCDate() + dir * 7);
              break;
            case 'y':
              date = Datetimepicker.prototype.moveYear.call(Datetimepicker.prototype, date, dir);
              break;
          }
        }
        return UTCDate(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(), date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds(), 0);
      }
      var parts = date && date.toString().match(this.nonpunctuation) || [],
        date = new Date(0, 0, 0, 0, 0, 0, 0),
        parsed = {},
        setters_order = ['hh', 'h', 'ii', 'i', 'ss', 's', 'yyyy', 'yy', 'M', 'MM', 'm', 'mm', 'D', 'DD', 'd', 'dd', 'H', 'HH', 'p', 'P', 'z', 'Z'],
        setters_map = {
          hh: function (d, v) {
            return d.setUTCHours(v);
          },
          h: function (d, v) {
            return d.setUTCHours(v);
          },
          HH: function (d, v) {
            return d.setUTCHours(v === 12 ? 0 : v);
          },
          H: function (d, v) {
            return d.setUTCHours(v === 12 ? 0 : v);
          },
          ii: function (d, v) {
            return d.setUTCMinutes(v);
          },
          i: function (d, v) {
            return d.setUTCMinutes(v);
          },
          ss: function (d, v) {
            return d.setUTCSeconds(v);
          },
          s: function (d, v) {
            return d.setUTCSeconds(v);
          },
          yyyy: function (d, v) {
            return d.setUTCFullYear(v);
          },
          yy: function (d, v) {
            return d.setUTCFullYear(2000 + v);
          },
          m: function (d, v) {
            v -= 1;
            while (v < 0) v += 12;
            v %= 12;
            d.setUTCMonth(v);
            while (d.getUTCMonth() !== v)
              if (isNaN(d.getUTCMonth()))
                return d;
              else
                d.setUTCDate(d.getUTCDate() - 1);
            return d;
          },
          d: function (d, v) {
            return d.setUTCDate(v);
          },
          p: function (d, v) {
            return d.setUTCHours(v === 1 ? d.getUTCHours() + 12 : d.getUTCHours());
          },
          z: function () {
            return timezone
          }
        },
        val, filtered, part;
      setters_map['M'] = setters_map['MM'] = setters_map['mm'] = setters_map['m'];
      setters_map['dd'] = setters_map['d'];
      setters_map['P'] = setters_map['p'];
      setters_map['Z'] = setters_map['z'];
      date = UTCDate(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours(), date.getMinutes(), date.getSeconds());
      if (parts.length === format.parts.length) {
        for (var i = 0, cnt = format.parts.length; i < cnt; i++) {
          val = parseInt(parts[i], 10);
          part = format.parts[i];
          if (isNaN(val)) {
            switch (part) {
              case 'MM':
                filtered = $(dates[language].months).filter(function () {
                  var m = this.slice(0, parts[i].length),
                    p = parts[i].slice(0, m.length);
                  return m === p;
                });
                val = $.inArray(filtered[0], dates[language].months) + 1;
                break;
              case 'M':
                filtered = $(dates[language].monthsShort).filter(function () {
                  var m = this.slice(0, parts[i].length),
                    p = parts[i].slice(0, m.length);
                  return m.toLowerCase() === p.toLowerCase();
                });
                val = $.inArray(filtered[0], dates[language].monthsShort) + 1;
                break;
              case 'p':
              case 'P':
                val = $.inArray(parts[i].toLowerCase(), dates[language].meridiem);
                break;
              case 'z':
              case 'Z':
                timezone;
                break;

            }
          }
          parsed[part] = val;
        }
        for (var i = 0, s; i < setters_order.length; i++) {
          s = setters_order[i];
          if (s in parsed && !isNaN(parsed[s]))
            setters_map[s](date, parsed[s])
        }
      }
      return date;
    },
    formatDate: function (date, format, language, type, timezone) {
      if (date === null) {
        return '';
      }
      var val;
      if (type === 'standard') {
        val = {
          t: date.getTime(),
          // year
          yy: date.getUTCFullYear().toString().substring(2),
          yyyy: date.getUTCFullYear(),
          // month
          m: date.getUTCMonth() + 1,
          M: dates[language].monthsShort[date.getUTCMonth()],
          MM: dates[language].months[date.getUTCMonth()],
          // day
          d: date.getUTCDate(),
          D: dates[language].daysShort[date.getUTCDay()],
          DD: dates[language].days[date.getUTCDay()],
          p: (dates[language].meridiem.length === 2 ? dates[language].meridiem[date.getUTCHours() < 12 ? 0 : 1] : ''),
          // hour
          h: date.getUTCHours(),
          // minute
          i: date.getUTCMinutes(),
          // second
          s: date.getUTCSeconds(),
          // timezone
          z: timezone
        };

        if (dates[language].meridiem.length === 2) {
          val.H = (val.h % 12 === 0 ? 12 : val.h % 12);
        }
        else {
          val.H = val.h;
        }
        val.HH = (val.H < 10 ? '0' : '') + val.H;
        val.P = val.p.toUpperCase();
        val.Z = val.z;
        val.hh = (val.h < 10 ? '0' : '') + val.h;
        val.ii = (val.i < 10 ? '0' : '') + val.i;
        val.ss = (val.s < 10 ? '0' : '') + val.s;
        val.dd = (val.d < 10 ? '0' : '') + val.d;
        val.mm = (val.m < 10 ? '0' : '') + val.m;
      } else if (type === 'php') {
        // php format
        val = {
          // year
          y: date.getUTCFullYear().toString().substring(2),
          Y: date.getUTCFullYear(),
          // month
          F: dates[language].months[date.getUTCMonth()],
          M: dates[language].monthsShort[date.getUTCMonth()],
          n: date.getUTCMonth() + 1,
          t: DPGlobal.getDaysInMonth(date.getUTCFullYear(), date.getUTCMonth()),
          // day
          j: date.getUTCDate(),
          l: dates[language].days[date.getUTCDay()],
          D: dates[language].daysShort[date.getUTCDay()],
          w: date.getUTCDay(), // 0 -> 6
          N: (date.getUTCDay() === 0 ? 7 : date.getUTCDay()),       // 1 -> 7
          S: (date.getUTCDate() % 10 <= dates[language].suffix.length ? dates[language].suffix[date.getUTCDate() % 10 - 1] : ''),
          // hour
          a: (dates[language].meridiem.length === 2 ? dates[language].meridiem[date.getUTCHours() < 12 ? 0 : 1] : ''),
          g: (date.getUTCHours() % 12 === 0 ? 12 : date.getUTCHours() % 12),
          G: date.getUTCHours(),
          // minute
          i: date.getUTCMinutes(),
          // second
          s: date.getUTCSeconds()
        };
        val.m = (val.n < 10 ? '0' : '') + val.n;
        val.d = (val.j < 10 ? '0' : '') + val.j;
        val.A = val.a.toString().toUpperCase();
        val.h = (val.g < 10 ? '0' : '') + val.g;
        val.H = (val.G < 10 ? '0' : '') + val.G;
        val.i = (val.i < 10 ? '0' : '') + val.i;
        val.s = (val.s < 10 ? '0' : '') + val.s;
      } else {
        throw new Error('Invalid format type.');
      }
      var date = [],
        seps = $.extend([], format.separators);
      for (var i = 0, cnt = format.parts.length; i < cnt; i++) {
        if (seps.length) {
          date.push(seps.shift());
        }
        date.push(val[format.parts[i]]);
      }
      if (seps.length) {
        date.push(seps.shift());
      }
      return date.join('');
    },
    convertViewMode: function (viewMode) {
      switch (viewMode) {
        case 4:
        case 'decade':
          viewMode = 4;
          break;
        case 3:
        case 'year':
          viewMode = 3;
          break;
        case 2:
        case 'month':
          viewMode = 2;
          break;
        case 1:
        case 'day':
          viewMode = 1;
          break;
        case 0:
        case 'hour':
          viewMode = 0;
          break;
      }

      return viewMode;
    },
    headTemplate: '<thead>' +
      '<tr>' +
      '<th class="prev"><i class="{iconType} {leftArrow}"/></th>' +
      '<th colspan="5" class="switch"></th>' +
      '<th class="next"><i class="{iconType} {rightArrow}"/></th>' +
      '</tr>' +
      '</thead>',
    headTemplateV3: '<thead>' +
      '<tr>' +
      '<th class="prev"><span class="{iconType} {leftArrow}"></span> </th>' +
      '<th colspan="5" class="switch"></th>' +
      '<th class="next"><span class="{iconType} {rightArrow}"></span> </th>' +
      '</tr>' +
      '</thead>',
    contTemplate: '<tbody><tr><td colspan="7"></td></tr></tbody>',
    footTemplate: '<tfoot>' +
      '<tr><th colspan="7" class="today"></th></tr>' +
      '<tr><th colspan="7" class="clear"></th></tr>' +
      '</tfoot>'
  };
  DPGlobal.template = '<div class="datetimepicker">' +
    '<div class="datetimepicker-minutes">' +
    '<table class=" table-condensed">' +
    DPGlobal.headTemplate +
    DPGlobal.contTemplate +
    DPGlobal.footTemplate +
    '</table>' +
    '</div>' +
    '<div class="datetimepicker-hours">' +
    '<table class=" table-condensed">' +
    DPGlobal.headTemplate +
    DPGlobal.contTemplate +
    DPGlobal.footTemplate +
    '</table>' +
    '</div>' +
    '<div class="datetimepicker-days">' +
    '<table class=" table-condensed">' +
    DPGlobal.headTemplate +
    '<tbody></tbody>' +
    DPGlobal.footTemplate +
    '</table>' +
    '</div>' +
    '<div class="datetimepicker-months">' +
    '<table class="table-condensed">' +
    DPGlobal.headTemplate +
    DPGlobal.contTemplate +
    DPGlobal.footTemplate +
    '</table>' +
    '</div>' +
    '<div class="datetimepicker-years">' +
    '<table class="table-condensed">' +
    DPGlobal.headTemplate +
    DPGlobal.contTemplate +
    DPGlobal.footTemplate +
    '</table>' +
    '</div>' +
    '</div>';
  DPGlobal.templateV3 = '<div class="datetimepicker">' +
    '<div class="datetimepicker-minutes">' +
    '<table class=" table-condensed">' +
    DPGlobal.headTemplateV3 +
    DPGlobal.contTemplate +
    DPGlobal.footTemplate +
    '</table>' +
    '</div>' +
    '<div class="datetimepicker-hours">' +
    '<table class=" table-condensed">' +
    DPGlobal.headTemplateV3 +
    DPGlobal.contTemplate +
    DPGlobal.footTemplate +
    '</table>' +
    '</div>' +
    '<div class="datetimepicker-days">' +
    '<table class=" table-condensed">' +
    DPGlobal.headTemplateV3 +
    '<tbody></tbody>' +
    DPGlobal.footTemplate +
    '</table>' +
    '</div>' +
    '<div class="datetimepicker-months">' +
    '<table class="table-condensed">' +
    DPGlobal.headTemplateV3 +
    DPGlobal.contTemplate +
    DPGlobal.footTemplate +
    '</table>' +
    '</div>' +
    '<div class="datetimepicker-years">' +
    '<table class="table-condensed">' +
    DPGlobal.headTemplateV3 +
    DPGlobal.contTemplate +
    DPGlobal.footTemplate +
    '</table>' +
    '</div>' +
    '</div>';
  $.fn.datetimepicker.DPGlobal = DPGlobal;

  /* DATETIMEPICKER NO CONFLICT
   * =================== */

  $.fn.datetimepicker.noConflict = function () {
    $.fn.datetimepicker = old;
    return this;
  };

  /* DATETIMEPICKER DATA-API
   * ================== */

  $(document).on(
    'focus.datetimepicker.data-api click.datetimepicker.data-api',
    '[data-provide="datetimepicker"]',
    function (e) {
      var $this = $(this);
      if ($this.data('datetimepicker')) return;
      e.preventDefault();
      // component click requires us to explicitly show it
      $this.datetimepicker('show');
    }
  );
  $(function () {
    $('[data-provide="datetimepicker-inline"]').datetimepicker();
  });

}));


/**
 * Simplified Chinese translation for bootstrap-datetimepicker
 * Yuan Cheung <advanimal@gmail.com>
 */
;(function($){
	$.fn.datetimepicker.dates['zh-CN'] = {
			days: ["星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六", "星期日"],
			daysShort: ["周日", "周一", "周二", "周三", "周四", "周五", "周六", "周日"],
			daysMin:  ["日", "一", "二", "三", "四", "五", "六", "日"],
			months: ["一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"],
			monthsShort: ["一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"],
			today: "今天",
			suffix: [],
			meridiem: ["上午", "下午"]
	};
}(jQuery));

+ function ($) {
  'use strict';

  var is_IE = function (ver) {
    var b = document.createElement('b')
    b.innerHTML = '<!--[if IE ' + ver + ']><i></i><![endif]-->'
    return b.getElementsByTagName('i').length === 1
  }
  // input ie9 兼容
  // ======================

  if (is_IE(9)) {
    $("[placeholder]")
      .focus(function () {
        var input = $(this);
        if (input.val() == input.attr("placeholder")) {
          input.val("");
          input.removeClass("placeholderIe");
        }
      })
      .blur(function () {
        var input = $(this);
        if (input.val() == "" || input.val() == input.attr("placeholder")) {
          input.addClass("placeholderIe");
          input.val(input.attr("placeholder")).css('color', '#bdbdbd');
        }
      })
      .blur()
      .parents("form")
      .submit(function () {
        $(this).find("[placeholder]").each(function () {
          var input = $(this);
          if (input.val() == input.attr("placeholder")) {
            input.val("");
          }
        });
      });
  }



}(jQuery);

+function ($) {
  'use strict';

  if (!window.loading) {
    window.loading = {};
  };

  loading.start = function (options) {
    loading.backdrop = document.createElement('div');
    loading.body = document.createElement('div');
    loading.body.setAttribute('class', 'loading-body');
    loading.backdrop.setAttribute('class', 'loading-back-drop');
    if (!options) {
      options = {
        type: 'loading'
      };
    }
    switch (options.type) {
      case 'part':
        loading.icon = document.createElement('div');
        loading.text = document.createElement('span');
        loading.icon.setAttribute('class', 'part-icon');
        loading.text.setAttribute('class', 'part-text');
        $(loading.text).html('正在加载，请稍候...');
        loading.body.appendChild(loading.icon);
        loading.body.appendChild(loading.text);
        $(options.ele).append(loading.backdrop);
        $(options.ele).append(loading.body);
        break;
      default:
        loading.icon = document.createElement('div');
        loading.icon.setAttribute('class', 'static-icon');
        $(loading.body).addClass('loading-static-body');
        loading.body.appendChild(loading.icon);
        document.body.appendChild(loading.backdrop);
        document.body.appendChild(loading.body);
        break;
    }
  }

  loading.end = function (ele, options) {
    $(loading.backdrop).remove();
    $(loading.body).remove();
    $(loading.icon).remove();
    if (!options) {
      options = {
        type: 'loading'
      };
    }
    switch (options.type) {
      case 'part':
        $(loading.text).remove();
        break;
    }
  }
}(jQuery);

/* ========================================================================
 * Bootstrap: modal.js v3.3.7
 * http://getbootstrap.com/javascript/#modals
 * ========================================================================
 * Copyright 2011-2016 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // MODAL CLASS DEFINITION
  // ======================

  var Modal = function (element, options) {
    this.options             = options
    this.$body               = $(document.body)
    this.$element            = $(element)
    this.$dialog             = this.$element.find('.modal-dialog')
    this.$backdrop           = null
    this.isShown             = null
    this.originalBodyPad     = null
    this.scrollbarWidth      = 0
    this.ignoreBackdropClick = false

    if (this.options.remote) {
      this.$element
        .find('.modal-content')
        .load(this.options.remote, $.proxy(function () {
          this.$element.trigger('loaded.bs.modal')
        }, this))
    }
  }

  Modal.VERSION  = '3.3.7'

  Modal.TRANSITION_DURATION = 300
  Modal.BACKDROP_TRANSITION_DURATION = 150

  Modal.DEFAULTS = {
    backdrop: true,
    keyboard: true,
    show: true
  }

  Modal.prototype.toggle = function (_relatedTarget) {
    return this.isShown ? this.hide() : this.show(_relatedTarget)
  }

  Modal.prototype.show = function (_relatedTarget) {
    var that = this
    var e    = $.Event('show.bs.modal', { relatedTarget: _relatedTarget })

    this.$element.trigger(e)

    if (this.isShown || e.isDefaultPrevented()) return

    this.isShown = true

    this.checkScrollbar()
    this.setScrollbar()
    // this.$body.addClass('modal-open')

    this.escape()
    this.resize()

    this.$element.on('click.dismiss.bs.modal', '[data-dismiss="modal"]', $.proxy(this.hide, this))

    this.$dialog.on('mousedown.dismiss.bs.modal', function () {
      that.$element.one('mouseup.dismiss.bs.modal', function (e) {
        if ($(e.target).is(that.$element)) that.ignoreBackdropClick = true
      })
    })

    this.backdrop(function () {
      var transition = $.support.transition && that.$element.hasClass('fade')

      if (!that.$element.parent().length) {
        that.$element.appendTo(that.$body) // don't move modals dom position
      }

      that.$element
        .show()
        .scrollTop(0)

      that.adjustDialog()

      if (transition) {
        that.$element[0].offsetWidth // force reflow
      }

      that.$element.addClass('in')

      that.enforceFocus()

      var e = $.Event('shown.bs.modal', { relatedTarget: _relatedTarget })

      transition ?
        that.$dialog // wait for modal to slide in
          .one('bsTransitionEnd', function () {
            that.$element.trigger('focus').trigger(e)
          })
          .emulateTransitionEnd(Modal.TRANSITION_DURATION) :
        that.$element.trigger('focus').trigger(e)
    })
  }

  Modal.prototype.hide = function (e) {
    if (e) e.preventDefault()

    e = $.Event('hide.bs.modal')

    this.$element.trigger(e)

    if (!this.isShown || e.isDefaultPrevented()) return

    this.isShown = false

    this.escape()
    this.resize()

    $(document).off('focusin.bs.modal')

    this.$element
      .removeClass('in')
      .off('click.dismiss.bs.modal')
      .off('mouseup.dismiss.bs.modal')

    this.$dialog.off('mousedown.dismiss.bs.modal')

    $.support.transition && this.$element.hasClass('fade') ?
      this.$element
        .one('bsTransitionEnd', $.proxy(this.hideModal, this))
        .emulateTransitionEnd(Modal.TRANSITION_DURATION) :
      this.hideModal()
  }

  Modal.prototype.enforceFocus = function () {
    $(document)
      .off('focusin.bs.modal') // guard against infinite focus loop
      .on('focusin.bs.modal', $.proxy(function (e) {
        if (document !== e.target &&
            this.$element[0] !== e.target &&
            !this.$element.has(e.target).length) {
          this.$element.trigger('focus')
        }
      }, this))
  }

  Modal.prototype.escape = function () {
    if (this.isShown && this.options.keyboard) {
      this.$element.on('keydown.dismiss.bs.modal', $.proxy(function (e) {
        e.which == 27 && this.hide()
      }, this))
    } else if (!this.isShown) {
      this.$element.off('keydown.dismiss.bs.modal')
    }
  }

  Modal.prototype.resize = function () {
    if (this.isShown) {
      $(window).on('resize.bs.modal', $.proxy(this.handleUpdate, this))
    } else {
      $(window).off('resize.bs.modal')
    }
  }

  Modal.prototype.hideModal = function () {
    var that = this
    this.$element.hide()
    this.backdrop(function () {
      that.$body.removeClass('modal-open')
      that.resetAdjustments()
      that.resetScrollbar()
      that.$element.trigger('hidden.bs.modal')
    })
  }

  Modal.prototype.removeBackdrop = function () {
    this.$backdrop && this.$backdrop.remove()
    this.$backdrop = null
  }

  Modal.prototype.backdrop = function (callback) {
    var that = this
    var animate = this.$element.hasClass('fade') ? 'fade' : ''

    if (this.isShown && this.options.backdrop) {
      var doAnimate = $.support.transition && animate

      this.$backdrop = $(document.createElement('div'))
        .addClass('modal-backdrop ' + animate)
        .appendTo(this.$body)

      this.$element.on('click.dismiss.bs.modal', $.proxy(function (e) {
        if (this.ignoreBackdropClick) {
          this.ignoreBackdropClick = false
          return
        }
        if (e.target !== e.currentTarget) return
        this.options.backdrop == 'static'
          ? this.$element[0].focus()
          : this.hide()
      }, this))

      if (doAnimate) this.$backdrop[0].offsetWidth // force reflow

      this.$backdrop.addClass('in')

      if (!callback) return

      doAnimate ?
        this.$backdrop
          .one('bsTransitionEnd', callback)
          .emulateTransitionEnd(Modal.BACKDROP_TRANSITION_DURATION) :
        callback()

    } else if (!this.isShown && this.$backdrop) {
      this.$backdrop.removeClass('in')

      var callbackRemove = function () {
        that.removeBackdrop()
        callback && callback()
      }
      $.support.transition && this.$element.hasClass('fade') ?
        this.$backdrop
          .one('bsTransitionEnd', callbackRemove)
          .emulateTransitionEnd(Modal.BACKDROP_TRANSITION_DURATION) :
        callbackRemove()

    } else if (callback) {
      callback()
    }
  }

  // these following methods are used to handle overflowing modals

  Modal.prototype.handleUpdate = function () {
    this.adjustDialog()
  }

  Modal.prototype.adjustDialog = function () {
    var modalIsOverflowing = this.$element[0].scrollHeight > document.documentElement.clientHeight

    this.$element.css({
      paddingLeft:  !this.bodyIsOverflowing && modalIsOverflowing ? this.scrollbarWidth : '',
      paddingRight: this.bodyIsOverflowing && !modalIsOverflowing ? this.scrollbarWidth : ''
    })
  }

  Modal.prototype.resetAdjustments = function () {
    this.$element.css({
      paddingLeft: '',
      paddingRight: ''
    })
  }

  Modal.prototype.checkScrollbar = function () {
    var fullWindowWidth = window.innerWidth
    if (!fullWindowWidth) { // workaround for missing window.innerWidth in IE8
      var documentElementRect = document.documentElement.getBoundingClientRect()
      fullWindowWidth = documentElementRect.right - Math.abs(documentElementRect.left)
    }
    this.bodyIsOverflowing = document.body.clientWidth < fullWindowWidth
    this.scrollbarWidth = this.measureScrollbar()
  }

  Modal.prototype.setScrollbar = function () {
    // var bodyPad = parseInt((this.$body.css('padding-right') || 0), 10)
    this.originalBodyPad = document.body.style.paddingRight || ''
    // if (this.bodyIsOverflowing) this.$body.css('padding-right', bodyPad + this.scrollbarWidth)
  }

  Modal.prototype.resetScrollbar = function () {
    // this.$body.css('padding-right', this.originalBodyPad)
  }

  Modal.prototype.measureScrollbar = function () { // thx walsh
    var scrollDiv = document.createElement('div')
    scrollDiv.className = 'modal-scrollbar-measure'
    this.$body.append(scrollDiv)
    var scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth
    this.$body[0].removeChild(scrollDiv)
    return scrollbarWidth
  }


  // MODAL PLUGIN DEFINITION
  // =======================

  function Plugin(option, _relatedTarget) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.modal')
      var options = $.extend({}, Modal.DEFAULTS, $this.data(), typeof option == 'object' && option)

      if (!data) $this.data('bs.modal', (data = new Modal(this, options)))
      if (typeof option == 'string') data[option](_relatedTarget)
      else if (options.show) data.show(_relatedTarget)
    })
  }

  var old = $.fn.modal

  $.fn.modal             = Plugin
  $.fn.modal.Constructor = Modal


  // MODAL NO CONFLICT
  // =================

  $.fn.modal.noConflict = function () {
    $.fn.modal = old
    return this
  }


  // MODAL DATA-API
  // ==============

  $(document).on('click.bs.modal.data-api', '[data-toggle="modal"]', function (e) {
    var $this   = $(this)
    var href    = $this.attr('href')
    var $target = $($this.attr('data-target') || (href && href.replace(/.*(?=#[^\s]+$)/, ''))) // strip for ie7
    var option  = $target.data('bs.modal') ? 'toggle' : $.extend({ remote: !/#/.test(href) && href }, $target.data(), $this.data())

    if ($this.is('a')) e.preventDefault()

    $target.one('show.bs.modal', function (showEvent) {
      if (showEvent.isDefaultPrevented()) return // only register focus restorer if modal will actually get shown
      $target.one('hidden.bs.modal', function () {
        $this.is(':visible') && $this.trigger('focus')
      })
    })
    Plugin.call($target, option, this)
  })

}(jQuery);

/*!
 * Mricode Pagination Plugin
 * Github: https://github.com/momopig/Mricode.Pagination
 * Version: 1.4.4
 *
 * Required jQuery
 *
 * Copyright 2016 Mricle
 * Released under the MIT license
 * Modified By momopig
 */

(function(factory) {
  if (typeof define === "function" && define.amd) {
    // AMD
    define(["jquery"], factory);
  } else if (typeof exports === "object") {
    // Node / CommonJS
    factory(require("jquery"));
  } else {
    // Globals
    factory(jQuery);
  }
})(function($) {
  "use strict";

  var Page = function(element, options) {
    var defaultOption = {
      pageIndex: 0,
      pageSize: 10,
      total: 0,
      pageBtnCount: 11,
      showFirstLastBtn: true,
      firstBtnText: null,
      lastBtnText: null,
      prevBtnText: "&laquo;",
      nextBtnText: "&raquo;",
      loadFirstPage: true,
      initSuccess: function() {
        if (this.debug && console) {
          console.info("init success! just run once!");
        }
      },
      remote: {
        url: null,
        params: null,
        pageParams: null,
        success: null,
        beforeSend: null,
        complete: null,
        pageIndexName: "pageIndex",
        pageSizeName: "pageSize",
        totalName: "total",
        traditional: false
      },
      pageElementSort: ["$page", "$size", "$jump", "$info"],
      showInfo: false,
      infoFormat: "{start} ~ {end} of {total} entires",
      noInfoText: "0 entires",
      showJump: false,
      jumpBtnText: "Go",
      showPageSizes: false,
      pageSizeItems: [5, 10, 15, 20],
      debug: false
    };
    this.$element = $(element);
    this.$page = $('<ul class="m-pagination-page"></ul>').hide();
    this.$size = $('<div class="m-pagination-size"></div>').hide();
    this.$jump = $('<div class="m-pagination-jump"></div>').hide();
    this.$info = $('<div class="m-pagination-info"></div>').hide();
    this.options = $.extend(
      true,
      {},
      defaultOption,
      $.fn.pagination.defaults,
      options
    );

    if (options.pageElementSort) {
      this.options.pageElementSort = options.pageElementSort;
    } else {
      if (
        $.fn.pagination.defaults &&
        $.fn.pagination.defaults.pageElementSort
      ) {
        this.options.pageElementSort = $.fn.pagination.defaults.pageElementSort;
      } else {
        this.options.pageElementSort = defaultOption.pageElementSort;
      }
    }
    this.options.pageSizeItems =
      options.pageSizeItems ||
      ($.fn.pagination.defaults && $.fn.pagination.defaults.pageSizeItems
        ? $.fn.pagination.defaults.pageSizeItems
        : defaultOption.pageSizeItems);
    this.total = this.options.total;
    this.currentUrl = this.options.remote.url;
    this.currentPageIndex = utility.convertInt(this.options.pageIndex);
    this.currentPageSize = utility.convertInt(this.options.pageSize);
    this.currentParams = utility.deserializeParams(this.options.remote.params) || {};
    this.getLastPageNum = function() {
      return pagination.core.calLastPageNum(this.total, this.currentPageSize);
    };
    if (this.options.remote.success === null) {
      this.options.remote.success = this.options.remote.callback;
    }
    this.init();
  };

  Page.prototype = {
    getPageIndex: function() {
      return this.currentPageIndex;
    },
    getPageSize: function() {
      return this.currentPageSize;
    },
    getParams: function() {
      return this.currentParams;
    },
    setPageIndex: function(pageIndex) {
      if (pageIndex !== undefined && pageIndex !== null) {
        this.currentPageIndex = utility.convertInt(pageIndex);
      }
      this.renderPagination();
    },
    setPageSize: function(pageSize) {
      if (pageSize !== undefined && pageSize !== null) {
        this.currentPageSize = utility.convertInt(pageSize);
      }
      this.renderPagination();
    },
    setRemoteUrl: function(url) {
      if (url) {
        this.currentUrl = url;
      }
    },
    setTotal: function(total) {
      if (total !== undefined && total !== null) {
        this.total = utility.convertInt(total);
      }
      this.renderPagination();
    },
    setParams: function(params) {
      if (params) {
        this.currentParams = utility.deserializeParams(params);
      }
      this.renderPagination();
    },
    render: function(total) {
      if (total !== undefined && total !== null) {
        this.total = utility.convertInt(total);
      }
      this.renderPagination();
      this.debug("pagination rendered");
    },
    init: function() {
      this.initHtml();
      this.initEvent();
      if (this.currentUrl && this.options.loadFirstPage) {
        this.remote();
      } else {
        this.renderPagination();
      }
      this.debug("pagination inited");
      this.options.initSuccess();
    },
    initHtml: function() {
      //init size module
      var sizeHtml = $('<select data-page-btn="size"></select>');
      var list = "";
      for (var i = 0; i < this.options.pageSizeItems.length; i++) {
        sizeHtml.append(
          '<option value="' +
            this.options.pageSizeItems[i] +
            '" ' +
            (this.currentPageSize == this.options.pageSizeItems[i]
              ? "selected"
              : "") +
            " >" +
            this.options.pageSizeItems[i] +
            "</option>"
        );
        //自定义样式切换pagesize下拉列表
        list +=
          '<li><a href="javascript:;" key=' +
          this.options.pageSizeItems[i] +
          ">" +
          this.options.pageSizeItems[i] +
          "条/页" +
          "</a></li>";
      }
      sizeHtml.val(this.currentPageSize);
      this.$size.append(sizeHtml);

      this.$size.append(
        '<div class="list-container"><ul>' + list + "</ul></div>"
      );
      //自定义样式的pagesize框
      this.$size.append(
        '<div class="size-container"><span class="label">' +
          this.currentPageSize +
          "条/页" +
          '</span><span class="iconfont icon-dropdown-copy"></span><span class="iconfont icon-pickup2"></span></div>'
      );

      //init jump module
      //添加自定义样式label字段
      var prefix = "跳转 ";
      var endfix = " 页";
      var jumpHtml =
        '<span class="jump-prefix">' +
        prefix +
        '</span><div class="m-pagination-group"><input data-page-btn="jump" type="text"><button data-page-btn="jump" type="button">' +
        this.options.jumpBtnText +
        '</button></div><span class="jump-endfix">' +
        endfix +
        "</span>";
      this.$jump.append(jumpHtml);
      this.$jump.find("input").val(1);
      for (var i = 0; i < this.options.pageElementSort.length; i++) {
        this.$element.append(this[this.options.pageElementSort[i]]);
      }
    },
    initEvent: function() {
      this.$element
        .on(
          "click",
          {
            page: this
          },
          function(event) {
            event.stopPropagation();
            if ($(event.target).is("button")) {
              if ($(event.target).data("pageBtn") == "jump") {
                var $input = event.data.page.$element.find(
                  "input[data-page-btn=jump]"
                );
                event.data.page.jumpEventHandler($input.val(), event);
              }
            } else {
              if ($(event.target).data("pageIndex") !== undefined)
                eventHandler.call(event.data.page, event);

              //自定义样式pagesize下拉选择事件绑定
              if (
                $(event.target)
                  .parent()
                  .attr("class") == "size-container"
              ) {
                var sizeContainer = $(event.target).parent();
                var paginationSize = sizeContainer.parent();
                var ul = paginationSize.find("ul");
                if (!ul.is(":visible")) {
                  ul.slideDown("fast");
                  sizeContainer.find(".icon-dropdown-copy").hide();
                  sizeContainer
                    .find(".icon-pickup2")
                    .css("display", "inline-block");
                } else {
                  ul.slideUp("fast");
                  sizeContainer.find(".icon-pickup2").hide();
                  sizeContainer
                    .find(".icon-dropdown-copy")
                    .css("display", "inline-block");
                }
              }
              if (
                $(event.target)
                  .closest("div")
                  .attr("class") == "list-container"
              ) {
                var $this = $(event.target);
                var $paginationSize = $this.closest(".m-pagination-size");
                var txt = $this.text();
                var key = $this.attr("key");
                $paginationSize
                  .find(".size-container .label")
                  .html(txt)
                  .attr("key", key);
                $this.closest("ul").hide();
                $paginationSize.find(".icon-pickup2").hide();
                $paginationSize
                  .find(".icon-dropdown-copy")
                  .css("display", "inline-block");
                var originSelect = $paginationSize.find(
                  '[data-page-btn="size"]'
                );
                originSelect.val(key);
                originSelect.trigger("change");
              }
            }
          }
        )
        .on(
          "change",
          {
            page: this
          },
          function(event) {
            var $this = $(event.target);
            if ($this.data("pageBtn") == "jump") {
              event.data.page.jumpEventHandler($this.val(), event);
            }
            if ($this.data("pageBtn") == "size") {
              eventHandler.call(event.data.page, event);
            }
          }
        )
        .on(
          "keypress",
          {
            page: this
          },
          function(event) {
            if (event.keyCode == "13") {
              var $input = event.data.page.$element.find(
                "input[data-page-btn=jump]"
              );
              var pageIndex = $input.val();
              event.data.page.jumpEventHandler($input.val(), event);
              $input.val(pageIndex);
            }
          }
        );
      //点击空白处 隐藏pagesize下拉列表
      $(document).on("click", function(e) {
        if ($(".m-pagination .list-container ul").is(":visible")) {
          var ul = $(".m-pagination .list-container ul");
          var paginationSize = ul.closest(".m-pagination-size");
          ul.hide();
          paginationSize.find(".size-container .icon-pickup2").hide();
          paginationSize
            .find(".size-container .icon-dropdown-copy")
            .css("display", "inline-block");
        }
      });
    },
    jumpEventHandler: function(inputValue, event) {
      if (!inputValue) {
        this.$jump.removeClass("error");
      } else if (!pagination.check.checkJumpPage(inputValue)) {
        this.$jump.addClass("error");
      } else if (utility.convertInt(inputValue) > this.getLastPageNum()) {
        this.$jump.addClass("error");
      } else {
        this.$jump.removeClass("error");
        eventHandler.call(this, event);
      }
    },

    doPagination: function() {
      if (this.currentUrl) {
        this.remote();
      } else {
        this.renderPagination();
      }
    },
    remote: function() {
      if (typeof this.options.remote.pageParams === "function") {
        var pageParams = this.options.remote.pageParams.call(this, {
          pageIndex: this.currentPageIndex,
          pageSize: this.currentPageSize
        });
        this.currentParams = $.extend({}, this.currentParams, pageParams);
      } else {
        this.currentParams[
          this.options.remote.pageIndexName
        ] = this.currentPageIndex;
        this.currentParams[
          this.options.remote.pageSizeName
        ] = this.currentPageSize;
      }
      pagination.remote.getAjax(
        this,
        this.currentUrl,
        this.currentParams,
        this.ajaxCallBack,
        this.options.remote.beforeSend,
        this.options.remote.complete,
        this.options.remote.traditional
      );
    },

    ajaxCallBack: function(result) {
      var total = utility.mapObjectNameRecursion(
        result,
        this.options.remote.totalName
      );
      if (total == null || total == undefined) {
        // throw new Error("the response of totalName :  '" + this.options.remote.totalName + "'  not found.");
        total = 0;
      }
      total = utility.convertInt(total);
      this.total = total;
      var lastPageNum = this.getLastPageNum();
      if (
        this.currentPageIndex > 0 &&
        lastPageNum - 1 < this.currentPageIndex
      ) {
        this.setPageIndex(lastPageNum - 1);
        this.remote();
      } else {
        if (typeof this.options.remote.success === "function")
          this.options.remote.success(result);
        this.renderPagination();
      }
    },

    onEvent: function(eventName, pageIndex, pageSize) {
      if (pageIndex != null)
        this.currentPageIndex = utility.convertInt(pageIndex);
      if (pageSize != null) this.currentPageSize = utility.convertInt(pageSize);
      this.doPagination();
      this.$element.trigger(eventName, {
        pageIndex: this.currentPageIndex,
        pageSize: this.currentPageSize
      });
      this.debug("pagination " + eventName);
    },
    //生成分页
    renderPagination: function() {
      var option = {
        showFirstLastBtn: this.options.showFirstLastBtn,
        firstBtnText: this.options.firstBtnText,
        prevBtnText: this.options.prevBtnText,
        nextBtnText: this.options.nextBtnText,
        lastBtnText: this.options.lastBtnText
      };
      var lastPageNum = this.getLastPageNum();
      this.currentPageIndex =
        lastPageNum > 0 && this.currentPageIndex > lastPageNum - 1
          ? lastPageNum - 1
          : this.currentPageIndex;
      this.$page
        .empty()
        .append(
          pagination.core.renderPages(
            this.currentPageIndex,
            this.currentPageSize,
            this.total,
            this.options.pageBtnCount,
            option
          )
        )
        .show();
      if (this.options.showPageSizes && lastPageNum !== 0) this.$size.show();
      else this.$size.hide();
      if (this.options.showJump && lastPageNum !== 0) this.$jump.show();
      else this.$jump.hide();
      this.$info.text(
        pagination.core.renderInfo(
          this.currentPageIndex,
          this.currentPageSize,
          this.total,
          this.options.infoFormat,
          this.options.noInfoText
        )
      );
      if (this.options.showInfo) this.$info.show();
      else this.$info.hide();
    },
    //销毁分页
    destroy: function() {
      this.$element
        .unbind()
        .data("pagination", null)
        .empty();
      this.debug("pagination destroyed");
    },
    debug: function(message, data) {
      if (this.options.debug && console) {
        message &&
          console.info(
            message +
              " : pageIndex = " +
              this.currentPageIndex +
              " , pageSize = " +
              this.currentPageSize +
              " , total = " +
              this.total
          );
        data && console.info(data);
      }
    }
  };

  var eventHandler = function(event) {
    var that = event.data.page;
    var $target = $(event.target);

    if (
      event.type === "click" &&
      $target.data("pageIndex") !== undefined &&
      !$target.parent().hasClass("active")
    ) {
      that.onEvent(
        pagination.event.pageClicked,
        $target.data("pageIndex"),
        null
      );
    } else if (
      (event.type === "click" || event.type === "keypress") &&
      $target.data("pageBtn") === "jump"
    ) {
      var pageIndexStr = that.$jump.find("input").val();
      if (utility.convertInt(pageIndexStr) <= that.getLastPageNum()) {
        that.onEvent(pagination.event.jumpClicked, pageIndexStr - 1, null);
        that.$jump.find("input").val(null);
      }
    } else if (event.type === "change" && $target.data("pageBtn") === "size") {
      var newPageSize = that.$size.find("select").val();
      var lastPageNum = pagination.core.calLastPageNum(that.total, newPageSize);
      if (lastPageNum > 0 && that.currentPageIndex > lastPageNum - 1) {
        that.currentPageIndex = lastPageNum - 1;
      }
      that.onEvent(
        pagination.event.pageSizeChanged,
        that.currentPageIndex,
        newPageSize
      );
    }
  };

  var pagination = {};
  pagination.event = {
    pageClicked: "pageClicked",
    jumpClicked: "jumpClicked",
    pageSizeChanged: "pageSizeChanged"
  };
  pagination.remote = {
    getAjax: function(
      pagination,
      url,
      data,
      success,
      beforeSend,
      complate,
      traditional
    ) {
      $.ajax({
        url: url,
        dataType: "json",
        data: data,
        cache: false,
        traditional: traditional,
        contentType: "application/Json",
        beforeSend: function(XMLHttpRequest) {
          if (typeof beforeSend === "function")
            beforeSend.call(this, XMLHttpRequest);
        },
        complete: function(XMLHttpRequest, textStatue) {
          if (typeof complate === "function")
            complate.call(this, XMLHttpRequest, textStatue);
        },
        success: function(result) {
          success.call(pagination, result);
        }
      });
    }
  };
  pagination.core = {
    /*
      options : {
          showFirstLastBtn
          firstBtnText:
      }
      */
    renderPages: function(pageIndex, pageSize, total, pageBtnCount, options) {
      options = options || {};
      var pageNumber = pageIndex + 1;
      var lastPageNumber = this.calLastPageNum(total, pageSize);
      var html = [];

      if (lastPageNumber <= pageBtnCount) {
        html = this.renderGroupPages(1, lastPageNumber, pageNumber);
      } else {
        var firstPage = this.renderPerPage(options.firstBtnText || 1, 0);
        var lastPage = this.renderPerPage(
          options.lastBtnText || lastPageNumber,
          lastPageNumber - 1
        );

        //button count of  both sides
        var symmetryBtnCount = (pageBtnCount - 1 - 4) / 2;
        if (!options.showFirstLastBtn) symmetryBtnCount = symmetryBtnCount + 1;
        var frontBtnNum = (pageBtnCount + 1) / 2;
        var behindBtnNum = lastPageNumber - (pageBtnCount + 1) / 2;

        var prevPage = this.renderPerPage(options.prevBtnText, pageIndex - 1);
        var nextPage = this.renderPerPage(options.nextBtnText, pageIndex + 1);

        symmetryBtnCount =
          symmetryBtnCount.toString().indexOf(".") == -1
            ? symmetryBtnCount
            : symmetryBtnCount + 0.5;
        frontBtnNum =
          frontBtnNum.toString().indexOf(".") == -1
            ? frontBtnNum
            : frontBtnNum + 0.5;
        behindBtnNum =
          behindBtnNum.toString().indexOf(".") == -1
            ? behindBtnNum
            : behindBtnNum + 0.5;
        if (pageNumber < frontBtnNum) {
          if (options.showFirstLastBtn) {
            html = this.renderGroupPages(1, pageBtnCount - 2, pageNumber);
            html.push(nextPage);
            html.push(lastPage);
          } else {
            html = this.renderGroupPages(1, pageBtnCount - 1, pageNumber);
            html.push(nextPage);
          }
        } else if (pageNumber > behindBtnNum) {
          if (options.showFirstLastBtn) {
            html = this.renderGroupPages(
              lastPageNumber - pageBtnCount + 3,
              pageBtnCount - 2,
              pageNumber
            );
            html.unshift(prevPage);
            html.unshift(firstPage);
          } else {
            html = this.renderGroupPages(
              lastPageNumber - pageBtnCount + 2,
              pageBtnCount - 1,
              pageNumber
            );
            html.unshift(prevPage);
          }
        } else {
          if (options.showFirstLastBtn) {
            html = this.renderGroupPages(
              pageNumber - symmetryBtnCount,
              pageBtnCount - 4,
              pageNumber
            );
            html.unshift(prevPage);
            html.push(nextPage);
            html.unshift(firstPage);
            html.push(lastPage);
          } else {
            html = this.renderGroupPages(
              pageNumber - symmetryBtnCount,
              pageBtnCount - 2,
              pageNumber
            );
            html.unshift(prevPage);
            html.push(nextPage);
          }
        }
      }
      return html;
    },
    renderGroupPages: function(beginPageNum, count, currentPage) {
      var html = [];
      for (var i = 0; i < count; i++) {
        var page = this.renderPerPage(beginPageNum, beginPageNum - 1);
        if (beginPageNum === currentPage) page.addClass("active");
        html.push(page);
        beginPageNum++;
      }
      return html;
    },
    renderPerPage: function(text, value) {
      return $("<li><a data-page-index='" + value + "'>" + text + "</a></li>");
    },
    renderInfo: function(
      currentPageIndex,
      currentPageSize,
      total,
      infoFormat,
      noInfoText
    ) {
      if (total <= 0) {
        return noInfoText;
      } else {
        var startNum = currentPageIndex * currentPageSize + 1;
        var endNum = (currentPageIndex + 1) * currentPageSize;
        endNum = endNum >= total ? total : endNum;
        return infoFormat
          .replace("{start}", startNum)
          .replace("{end}", endNum)
          .replace("{total}", total);
      }
    },
    //计算最大分页数
    calLastPageNum: function(total, pageSize) {
      total = utility.convertInt(total);
      pageSize = utility.convertInt(pageSize);
      var i = total / pageSize;
      return utility.isDecimal(i) ? parseInt(i) + 1 : i;
    }
  };
  pagination.check = {
    //校验跳转页数有效性
    checkJumpPage: function(pageIndex) {
      var reg = /^\+?[1-9][0-9]*$/;
      return reg.test(pageIndex);
    }
  };

  var utility = {
    //转换为int
    convertInt: function(i) {
      if (typeof i === "number") {
        return i;
      } else {
        var j = parseInt(i);
        if (!isNaN(j)) {
          return j;
        } else {
          throw new Error(
            "convertInt Type Error.  Type is " + typeof i + ", value = " + i
          );
        }
      }
    },
    //返回是否小数
    isDecimal: function(i) {
      return parseInt(i) !== i;
    },
    //匹配对象名称（递归）
    mapObjectNameRecursion: function(object, name) {
      var obj = object;
      var arr = name.split(".");
      for (var i = 0; i < arr.length; i++) {
        obj = utility.mapObjectName(obj, arr[i]);
      }
      return obj;
    },
    //匹配对象名称
    mapObjectName: function(object, name) {
      var value = null;
      for (var i in object) {
        //过滤原型属性
        if (object.hasOwnProperty(i)) {
          if (i == name) {
            value = object[i];
            break;
          }
        }
      }
      return value;
    },
    deserializeParams: function(params) {
      var newParams = {};
      if (typeof params === "string") {
        var arr = params.split("&");
        for (var i = 0; i < arr.length; i++) {
          var a = arr[i].split("=");
          newParams[a[0]] = decodeURIComponent(a[1]);
        }
      } else if (params instanceof Array) {
        for (var i = 0; i < params.length; i++) {
          newParams[params[i].name] = decodeURIComponent(params[i].value);
        }
      } else if (typeof params === "object") {
        newParams = params;
      }
      return newParams;
    }
  };

  $.fn.pagination = function(option) {
    if (typeof option === "undefined") {
      return $(this).data("pagination") || false;
    } else {
      var result;
      var args = arguments;
      this.each(function() {
        var $this = $(this);
        var data = $this.data("pagination");
        if (!data && typeof option === "string") {
          throw new Error("MricodePagination is uninitialized.");
        } else if (data && typeof option === "object") {
          throw new Error("MricodePagination is initialized.");
        }
        //初始化
        else if (!data && typeof option === "object") {
          var options = typeof option == "object" && option;
          var data_api_options = $this.data();
          options = $.extend(options, data_api_options);
          $this.data("pagination", (data = new Page(this, options)));
        } else if (data && typeof option === "string") {
          result = data[option].apply(
            data,
            Array.prototype.slice.call(args, 1)
          );
        }
      });
      return typeof result === "undefined" ? this : result;
    }
  };
});

+ function ($) {
  'use strict';

  // radioCheck CLASS DEFINITION
  // ======================

  var Radio = function (element, options) {
    this.$options = options
    this.$body = $(document.body)
    this.$el = $(element)
  }

  Radio.VERSION = '0.0.1'

  Radio.DEFAULTS = {
    show: true
  }

  Radio.prototype.init = function () {
    console.log(this.$el)
    this.render()
    this.bind()
  }

  Radio.prototype.render = function () {
    var $el = this.$el;

    $el.find('input').each(function (index, val) {
      var title = $(val).attr('title');
      var htmlVal = "<div class='hz-form-radio'><i class='hz-radio-unselect'></i><div>" + title + "</div></div>";
      $(val).after(htmlVal);
    });
  }

  Radio.prototype.bind = function () {
    var that = this;
    this.$el.find('input').each(function (index, val) {
      if ($(val).attr('checked')) {
        $(val).next().children('i').addClass('hz-radioed')
      }

      $(val).next('.hz-form-radio').click(function (e) {
        window.e ? window.e.cancelBubble = true : e.stopPropagation();
        window.e ? window.e.returnValue = false : e.preventDefault();
        var radioSelect = $(this).children('i');
        if (radioSelect.hasClass('hz-radioed')) {
          return;
        } else {
          $('.hz-radio-unselect').removeClass('hz-radioed');
          radioSelect.addClass('hz-radioed');
          if (that.$options) {
            if (that.$options.click && typeof that.$options.click == 'function') {
              that.$options.click($(val).val(), e);
            } else {
              console.log('请输入click处理函数')
            }
          }
        }
      })
    })
  }


  // Radio PLUGIN DEFINITION
  // =======================

  function Plugin(options) {
    return this.each(function () {
      var rd = new Radio(this, options);
      rd.init();
    })
  }

  var old = $.fn.radio

  $.fn.radio = Plugin
  $.fn.radio.Constructor = Radio


  // radio NO CONFLICT
  // =================

  $.fn.radio.noConflict = function () {
    $.fn.radio = old
    return this
  }


  // Radio DATA-API
  // ==============

  $(document).on('click.bs.Radio.data-api', '[data-toggle="Radio"]', function (e) {
    var $this = $(this)
    var href = $this.attr('href')
    var $target = $($this.attr('data-target') || (href && href.replace(/.*(?=#[^\s]+$)/, ''))) // strip for ie7
    var option = $target.data('bs.Radio') ? 'toggle' : $.extend({
      remote: !/#/.test(href) && href
    }, $target.data(), $this.data())

    if ($this.is('a')) e.preventDefault()

    $target.one('show.bs.Radio', function (showEvent) {
      if (showEvent.isDefaultPrevented()) return // only register focus restorer if Radio will actually get shown
      $target.one('hidden.bs.Radio', function () {
        $this.is(':visible') && $this.trigger('focus')
      })
    })
    Plugin.call($target, option, this)
  })

}(jQuery);

/*
  初始化配置参数 params:options
  {
    width: 100, // 宽度
    height: 30, // 高度
    placeholder:'请选择',
    defaultText:'选项1',
    defaultValue:'1',
    noBorder: true, // 添加class no-border 
    data:[
      {value:'1',text:'选项1', ，click: function(value, text){}},
      {value:'2',text:'选项2'，click: function(value, text){}},
    ],
    onChange:changeHandler(value,text),
    onClick: clickHandler(value, text)
  }
*/
+function ($) {
  'use strict'

  var Select = function (el, options) {
    this.$el = el
    this.defaults = {
      placeholder:'请选择',
      height: 30,
      data: []
    };
    this.options = $.extend({}, this.defaults, options)
    return this
  }

  // 初始化函数
  Select.prototype.init = function () {
    this.create()
    this.setData()
    this.bind()
  }
  Select.prototype.create = function () {
    var options = this.options
    var el = this.$el
    if (options.noBorder) {
      el.addClass('no-border')
    }
    if (!options.height) {
      options.height = 30
    }
    el.addClass('dropdown-select')
    var placeholder = '请选择'
    if (options && options.placeholder) {
      placeholder = options.placeholder
    }
    el.append('<div class="select-container"><span class="label">' + placeholder + '</span><span class="iconfont icon-dropdown-copy"></span><span class="iconfont icon-pickup2"></span></div>')

    if (options && options.defaultText) {
      el.find('.label').html(options.defaultText)
    }
    if (options.width) {
      el.css('width', options.width)
      el.find('.select-container').css('width', options.width - 2)
      el.find('.list-container').css('width', options.width - 2)
    }
    if (options.height) {
      el.css('height', options.height)
      el.find('.select-container')
        .css({
          height: options.height + 'px',
          lineHeight: options.height + 'px'
        })
      el.find('.list-container').css('height', options.height + 'px')
    }
  }
  // 设置下拉选项数据
  Select.prototype.setData = function () {
    var options = this.options
    var el = this.$el
    var listArray = options.data
    el.find('.list-container').remove()
    var list = '无选项'
    var label = el.find('.label')
    if (listArray.length > 0) {
      list = ''
      for (var index = 0; index < listArray.length; index++) {
        var item = listArray[index]
        var htmlLi = '<li><a title="' + item.text + '" href="javascript:;" value=' + item.value + ">" + item.text + "</a></li>";
        list += htmlLi
      }
      if (options.defaultValue) {
        var matchItem = listArray.filter(function(item) {
          return item.value == options.defaultValue;
        })
        if (matchItem.length > 0) {
          label.html(matchItem[0].text)
          label.attr('value', matchItem[0].value)
        } else {
          label.html('请选择')
          label.attr('value', '')
        }
      }
    }
    el.append('<div class="list-container" style="width:' + el.width() + 'px;"><ul>' + list + "</ul></div>")
  }

  // 更改switch颜色，状态
  Select.prototype.bind = function () {
    var el = this.$el
    var options = this.options
    var ul = el.find('ul')
    var listContanier = el.find('.list-container')
    el.on('mouseenter', function() {
      listContanier.show()
      ul.slideDown('fast')
    })
    el.on('mouseleave', function() {
      ul.slideUp('fast')
    })
    el.on('click', 'a', function() {
      var txt = $(this).text()
      var value = $(this).attr('value')
      var $label = $(this).closest('.dropdown-select').find('.label')
      var originText = $label.html()
      $label.html(txt).attr('value', value)
      
      if (options && options.onChange && typeof options.onChange == 'function' && txt !== originText) {
        options.onChange.call(this, value, txt)
      }
      if (options && options.onClick && typeof options.onClick == 'function') {
        options.onClick.call(this, value, txt)
      }

      var matchItem = options.data.filter(function(item) {
        return item.value == value;
      })

      if (matchItem.length > 0 && matchItem[0].click && typeof matchItem[0].click === 'function') {
        matchItem[0].click(value, txt)
      }
      $(this).closest('.dropdown-select').find('.list-container').hide()
    })
  }
  //获取选择的选项对应的文本值
  Select.prototype.getText = function() {
    var el = this.$el
    var text = ''
    if (el.find('.label').attr('value')) {
      text = el.find('.label').text()
    }
    return text
  }
  //获取选择的选项对应的value值
  Select.prototype.getValue = function() {
    var value = this.$el.find('.label').attr('value')
    return value ? value : ''
  }
  //重置
  Select.prototype.reset = function(val, value) {
    var el = this.el
    val = val || ''
    value = value || ''
    el.find('.label').text(val || this.options.placeholder)
    el.find('.label').attr('value', value)
  }
  // 销毁
  Select.prototype.destroy = function() {
    this.el.off().children().remove()
  }
  // 插件函数
  function Plugin(options, type) {
    if (arguments.length > 1) {
      options = type
    }
    var $ele = this.map(function (i, node) {
      var select = new Select($(node), options)
      select.init()
      return select
    })
    return this.length == 1 ? $ele[0] : $ele
  }

  var old = $.fn.select

  $.fn.select = Plugin
  $.fn.select.Constructor = Select

  $.fn.select.noConflict = function () {
    $.fn.select = old
    return this
  }

}(jQuery)

/**
 * switch开关组件
 * 参数可选
 * 用法 el.swithc({
 * 	click: function (onoff, el, event) {}
 * })
 * 
 * click参数说明：
 * onoff 开关状态  true 代表开  fault 代表关
 * el 当前点击的节点
 * event 触发事件对象
 * 
 */

+function ($) {
  'use strict';

  var Switch = function (el, options) {
    this.$el = el
    this.defaults = {};
    this.options = $.extend({}, this.defaults, options)
  }

  // 初始化函数
  Switch.prototype.init = function () {
    this.options.onoff = this.$el.hasClass('switch-on')
    this.changeColor()
  }

  // 初始化函数
  Switch.prototype.changeColor = function () {
    var el = this.$el
    if (this.options.onoff && el.attr('themeColor')) {
      el.css('background', el.attr('themeColor'))
    }
    this.bind()
  }

  // 更改switch颜色，状态
  Switch.prototype.bind = function () {
    var el = this.$el
    var options = this.options
    el.click(function (event) {
      options.onoff = !options.onoff
      if (options.onoff) {
        el.removeClass('switch-off').addClass('switch-on')
        if (el.attr('themeColor')) {
          el.css('background', el.attr('themeColor'))
        }
      } else {
        el.removeClass('switch-on').addClass('switch-off')
        if (el.attr('themeColor')) {
          el.css('background', '#D1D1D1')
        }
      }
      if (options.click && typeof options.click === 'function') {
        options.click(options.onoff, el, event)
      }
    })
  }

  function Plugin(options) {
    this.each(function (i, node) {
      var sw = new Switch($(node), options)
      sw.init();
    })
  }

  var old = $.fn.switch

  $.fn.switch = Plugin
  $.fn.switch.Constructor = Switch


  $.fn.switch.noConflict = function () {
    $.fn.switch = old
    return this
  }

}(jQuery);

/**
 * tab选项卡组件
 * 参数可选
 * 用法 el.tab({
 * 	onClickTab: function (event, el, index) {}
 * })
 * 
 * onClickTab参数说明：
 * 
 * index 当前活动展示的tab的索引值
 * el 当前点击的节点
 * event 触发事件对象
 * 
 */

+function ($) {
  'use strict';

  var Tab = function (el, options) {
    this.$el = el
    this.defaults = {};
    this.options = $.extend({}, this.defaults, options)
  }

  // 初始化函数
  Tab.prototype.init = function () {
    this.bind()
  }

  // 初始化函数
  Tab.prototype.bind = function () {
    var self = this
    var options = this.options
    var clickHandler = function (e) {
      if ($(this).attr('href') && $(this).attr('href')[0] !== '#') return
      e.preventDefault();
      var $this = $(this);
      var parent = $this.parent('li');
      if (parent.is('[disabled]')) return

      var selector = $this.attr('href');
      if (parent.hasClass('active')) return
      //切换tabpane
      $(selector).siblings('.tab-pane').removeClass('active');
      if ($(selector).hasClass('tab-pane')) {
        $(selector).addClass('active');
      }
      parent.siblings().removeClass('active');
      parent.addClass('active');
      if (options) {
        if (options.onClickTab && typeof options.onClickTab == 'function') {
          options.onClickTab($this, self, parent.index());
        }
      }
      self.$el.trigger('hz.tab.click', {
        event: $this,
        el: self,
        index: parent.index()
      })
    };

    $(document).on('click', '.sub-tabs a, .nav-tabs a, .nav-tabs-card a, [role="tablist"] a[role="tab"]', clickHandler)
  }


  function Plugin(option) {
    this.each(function (i, node) {
      var tab = new Tab($(node), option)
      tab.init();
    })
  }

  var old = $.fn.tab

  $.fn.tab = Plugin
  $.fn.tab.Constructor = Tab


  $.fn.tab.noConflict = function () {
    $.fn.switch = old
    return this
  }

}(jQuery);

/**
 * table组件
 * 参数可选
 * 用法 el.tableRender({
 * 	width: 600,
		height: 300,
		headerFixed: true, // 是否开启头部固定
		cols: [],
		data: []
 * })
 *
 * onClickTab参数说明：
 *
 * width 表格宽度
 * height 表格高度
 * headerFixed 是否开启头部固定
 * cols 列配置
 * data 渲染数据数组
 */


 /*!
 * jQuery Mousewheel 3.1.13
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 */

(function (factory) {
  if ( typeof define === 'function' && define.amd ) {
      // AMD. Register as an anonymous module.
      define(['jquery'], factory);
  } else if (typeof exports === 'object') {
      // Node/CommonJS style for Browserify
      module.exports = factory;
  } else {
      // Browser globals
      factory(jQuery);
  }
}(function ($) {

  var toFix  = ['wheel', 'mousewheel', 'DOMMouseScroll', 'MozMousePixelScroll'],
      toBind = ( 'onwheel' in document || document.documentMode >= 9 ) ?
                  ['wheel'] : ['mousewheel', 'DomMouseScroll', 'MozMousePixelScroll'],
      slice  = Array.prototype.slice,
      nullLowestDeltaTimeout, lowestDelta;

  if ( $.event.fixHooks ) {
      for ( var i = toFix.length; i; ) {
          $.event.fixHooks[ toFix[--i] ] = $.event.mouseHooks;
      }
  }

  var special = $.event.special.mousewheel = {
      version: '3.1.12',

      setup: function() {
          if ( this.addEventListener ) {
              for ( var i = toBind.length; i; ) {
                  this.addEventListener( toBind[--i], handler, false );
              }
          } else {
              this.onmousewheel = handler;
          }
          // Store the line height and page height for this particular element
          $.data(this, 'mousewheel-line-height', special.getLineHeight(this));
          $.data(this, 'mousewheel-page-height', special.getPageHeight(this));
      },

      teardown: function() {
          if ( this.removeEventListener ) {
              for ( var i = toBind.length; i; ) {
                  this.removeEventListener( toBind[--i], handler, false );
              }
          } else {
              this.onmousewheel = null;
          }
          // Clean up the data we added to the element
          $.removeData(this, 'mousewheel-line-height');
          $.removeData(this, 'mousewheel-page-height');
      },

      getLineHeight: function(elem) {
          var $elem = $(elem),
              $parent = $elem['offsetParent' in $.fn ? 'offsetParent' : 'parent']();
          if (!$parent.length) {
              $parent = $('body');
          }
          return parseInt($parent.css('fontSize'), 10) || parseInt($elem.css('fontSize'), 10) || 16;
      },

      getPageHeight: function(elem) {
          return $(elem).height();
      },

      settings: {
          adjustOldDeltas: true, // see shouldAdjustOldDeltas() below
          normalizeOffset: true  // calls getBoundingClientRect for each event
      }
  };

  $.fn.extend({
      mousewheel: function(fn) {
          return fn ? this.bind('mousewheel', fn) : this.trigger('mousewheel');
      },

      unmousewheel: function(fn) {
          return this.unbind('mousewheel', fn);
      }
  });


  function handler(event) {
      var orgEvent   = event || window.event,
          args       = slice.call(arguments, 1),
          delta      = 0,
          deltaX     = 0,
          deltaY     = 0,
          absDelta   = 0,
          offsetX    = 0,
          offsetY    = 0;
      event = $.event.fix(orgEvent);
      event.type = 'mousewheel';

      // Old school scrollwheel delta
      if ( 'detail'      in orgEvent ) { deltaY = orgEvent.detail * -1;      }
      if ( 'wheelDelta'  in orgEvent ) { deltaY = orgEvent.wheelDelta;       }
      if ( 'wheelDeltaY' in orgEvent ) { deltaY = orgEvent.wheelDeltaY;      }
      if ( 'wheelDeltaX' in orgEvent ) { deltaX = orgEvent.wheelDeltaX * -1; }

      // Firefox < 17 horizontal scrolling related to DOMMouseScroll event
      if ( 'axis' in orgEvent && orgEvent.axis === orgEvent.HORIZONTAL_AXIS ) {
          deltaX = deltaY * -1;
          deltaY = 0;
      }

      // Set delta to be deltaY or deltaX if deltaY is 0 for backwards compatabilitiy
      delta = deltaY === 0 ? deltaX : deltaY;

      // New school wheel delta (wheel event)
      if ( 'deltaY' in orgEvent ) {
          deltaY = orgEvent.deltaY * -1;
          delta  = deltaY;
      }
      if ( 'deltaX' in orgEvent ) {
          deltaX = orgEvent.deltaX;
          if ( deltaY === 0 ) { delta  = deltaX * -1; }
      }

      // No change actually happened, no reason to go any further
      if ( deltaY === 0 && deltaX === 0 ) { return; }

      // Need to convert lines and pages to pixels if we aren't already in pixels
      // There are three delta modes:
      //   * deltaMode 0 is by pixels, nothing to do
      //   * deltaMode 1 is by lines
      //   * deltaMode 2 is by pages
      if ( orgEvent.deltaMode === 1 ) {
          var lineHeight = $.data(this, 'mousewheel-line-height');
          delta  *= lineHeight;
          deltaY *= lineHeight;
          deltaX *= lineHeight;
      } else if ( orgEvent.deltaMode === 2 ) {
          var pageHeight = $.data(this, 'mousewheel-page-height');
          delta  *= pageHeight;
          deltaY *= pageHeight;
          deltaX *= pageHeight;
      }

      // Store lowest absolute delta to normalize the delta values
      absDelta = Math.max( Math.abs(deltaY), Math.abs(deltaX) );

      if ( !lowestDelta || absDelta < lowestDelta ) {
          lowestDelta = absDelta;

          // Adjust older deltas if necessary
          if ( shouldAdjustOldDeltas(orgEvent, absDelta) ) {
              lowestDelta /= 40;
          }
      }

      // Adjust older deltas if necessary
      if ( shouldAdjustOldDeltas(orgEvent, absDelta) ) {
          // Divide all the things by 40!
          delta  /= 40;
          deltaX /= 40;
          deltaY /= 40;
      }

      // Get a whole, normalized value for the deltas
      delta  = Math[ delta  >= 1 ? 'floor' : 'ceil' ](delta  / lowestDelta);
      deltaX = Math[ deltaX >= 1 ? 'floor' : 'ceil' ](deltaX / lowestDelta);
      deltaY = Math[ deltaY >= 1 ? 'floor' : 'ceil' ](deltaY / lowestDelta);

      // Normalise offsetX and offsetY properties
      if ( special.settings.normalizeOffset && this.getBoundingClientRect ) {
          var boundingRect = this.getBoundingClientRect();
          offsetX = event.clientX - boundingRect.left;
          offsetY = event.clientY - boundingRect.top;
      }

      // Add information to the event object
      event.deltaX = deltaX;
      event.deltaY = deltaY;
      event.deltaFactor = lowestDelta;
      event.offsetX = offsetX;
      event.offsetY = offsetY;
      // Go ahead and set deltaMode to 0 since we converted to pixels
      // Although this is a little odd since we overwrite the deltaX/Y
      // properties with normalized deltas.
      event.deltaMode = 0;

      // Add event and delta to the front of the arguments
      args.unshift(event, delta, deltaX, deltaY);

      // Clearout lowestDelta after sometime to better
      // handle multiple device types that give different
      // a different lowestDelta
      // Ex: trackpad = 3 and mouse wheel = 120
      if (nullLowestDeltaTimeout) { clearTimeout(nullLowestDeltaTimeout); }
      nullLowestDeltaTimeout = setTimeout(nullLowestDelta, 200);

      return ($.event.dispatch || $.event.handle).apply(this, args);
  }

  function nullLowestDelta() {
      lowestDelta = null;
  }

  function shouldAdjustOldDeltas(orgEvent, absDelta) {
      // If this is an older event and the delta is divisable by 120,
      // then we are assuming that the browser is treating this as an
      // older mouse wheel event and that we should divide the deltas
      // by 40 to try and get a more usable deltaFactor.
      // Side note, this actually impacts the reported scroll distance
      // in older browsers and can cause scrolling to be slower than native.
      // Turn this off by setting $.event.special.mousewheel.settings.adjustOldDeltas to false.
      return special.settings.adjustOldDeltas && orgEvent.type === 'mousewheel' && absDelta % 120 === 0;
  }

}));

+ function($) {
  'use strict';

  var Table = function(el, options) {
    this.$el = el;
    this.width = options.width || el.width()
    this.fixedRightCol = []
    this.defaults = {
      cols: [],
      data: []
    };
    this.options = $.extend({}, this.defaults, options)
    var h = ((Number(this.options.data.length) + 1) * 36) + 'px'
    this.height = parseInt(options.height) ? parseInt(options.height) + 'px' : h
  }

  // 初始化函数
  Table.prototype.init = function(options) {
    this.render()
    this.createScroll()
    this.bind()
    return $('.hz-table')
  }
  // 渲染表格函数
  Table.prototype.render = function() {
    var cols = this.options.cols
    var data = this.options.data
    var self = this
    var colgroup = ''
    var colArr = []
    var thstr = ''
    var allWidth = 0

    for (var i = 0, len = cols.length; i < len; i++) {
      var width = parseInt(cols[i].width)
      allWidth += width
      colArr.push('<col width="' + width + '"></col>')
      thstr += '<th><div>' + cols[i].label + '</div></th>'
      if (cols[i].fixed === 'right') {
        this.fixedRightCol.push(cols[i])
      }
    }
    if (allWidth < this.width) {
      var dis = this.width - allWidth
      var str = ''
      var width = 0
      if (cols[i - 1].type || cols[i - 1].fixed) {
        str = colArr[i - 2]
        width = parseInt(cols[i - 2].width) + dis
        str = str.replace(/\d+/g, width + '')
        colArr.splice(i - 2, 1, str)
      } else {
        str = colArr[i - 1]
        width = parseInt(cols[i - 1].width) + dis
        str = str.replace(/\d+/g, width + '')
        colArr.splice(i - 1, 1, str)
      }
      allWidth = this.width
    }
    $.each(colArr, function(i, item) {
      if (item.fixed === 'fixed') {
        self.fixedRightCol.push(item)
      }
    })

    var colstr = colArr.join('')
    var colgroup = '<colgroup>' + colstr + '</colgroup>'
    var theader = '<thead><tr>' + thstr + '</tr></thead>'

    var trs = ''

    for (var j = 0, leng = data.length; j < leng; j++) {
      var tds = ''
      for (var z = 0; z < cols.length; z++) {
        var text = data[j][cols[z].key] || ''
        if (cols[z].render && typeof cols[z].render === 'function') {
          text = cols[z].render(data[j])
        }
        if (cols[z].type === 'link') {
          if (toType(cols[z].text) === 'Array') {
            var btns = toOparations(cols[z].text, j)
            tds += '<td data-rowsIndex="' + j + '"><div>' + btns + '</div></td>'
          } else {
            tds += '<td data-rowsIndex="' + j + '"><div><a href="javascript:void(0)" data-btnText="' + cols[z].text + '">' + cols[z].text + '</a></div></td>'
          }
        } else {
          tds += '<td data-rowsIndex="' + j + '"><div>' + text + '</div></td>'
        }
      }
      trs += '<tr>' + tds + '</tr>'
    }

    var tbody = '<tbody>' + trs + '</tbody>'
    var fixedRightStr = ''
    if (this.fixedRightCol.length > 0) {
      fixedRightStr = this.toFixedRightTemplate()
    }
    var tempalte = this.toTemplate({
      allWidth: allWidth,
      colgroup: colgroup,
      theader: theader,
      tbody: tbody,
      fixedRightStr: fixedRightStr
    })
    this.$el.html(tempalte)
  }
  // 渲染右边固定列
  Table.prototype.toFixedRightTemplate = function() {
    var cols = this.fixedRightCol
    var data = this.options.data
    var colstr = ''
    var thstr = ''
    var trs = ''
    var allwidth = 0
    for (var i = 0, len = cols.length; i < len; i++) {
      var width = parseInt(cols[i].width)
      allwidth += width
      colstr += '<col width="' + width + '"></col>'
      thstr += '<th><div>' + cols[i].label + '</div></th>'
    }
    var colgroup = '<colgroup>' + colstr + '</colgroup>'
    var theader = '<thead><tr>' + thstr + '</tr></thead>'

    for (var j = 0, leng = data.length; j < leng; j++) {
      var tds = ''
      for (var z = 0; z < cols.length; z++) {
        var text = cols[z].text
        if (cols[z].render && typeof cols[z].render === 'function') {
          text = cols[z].render(data[j])
        }
        if (cols[z].type === 'link') {
          if (toType(cols[z].text) === 'Array') {
            var btns = toOparations(cols[z].text, j)
            tds += '<td data-rowsIndex="' + j + '"><div>' + btns + '</div></td>'
          } else {
            tds += '<td data-rowsIndex="' + j + '"><div><a href="#">' + text + '</a></div></td>'
          }
        } else {
          tds += '<td data-rowsIndex="' + j + '"><div>' + text + '</div></td>'
        }
      }
      trs += '<tr>' + tds + '</tr>'
    }
    var tbody = '<tbody>' + trs + '</tbody>'

    var headerFixed = ''
    if (this.options.headerFixed) {
      headerFixed = '<table class="fixedRightHeaderTable" width="' + allwidth + '" cellspacing="0" cellpadding="0" border="0"> \
                    ' + colgroup + theader + ' \
                  </table>'
    }

    var fixedRightTemplate = '<div class="fixedRight" style="width: ' + allwidth + 'px; height: ' + this.height + '"> \
      {{_headerFixed}}\
      <table class="fixedRightTable" width="' + allwidth + '" cellspacing="0" cellpadding="0" border="0">\
        ' + colgroup + theader + tbody + '\
      </table>\
    </div>'
    fixedRightTemplate = fixedRightTemplate.replace('{{_headerFixed}}', headerFixed)
    return fixedRightTemplate
  },
  // 输出固定头部
  Table.prototype.toHeanderTemplate = function(params) {
    var headerStr = ''
    if (this.options.headerFixed) {
      headerStr = '<div class="theader"> \
                  <table class="fixedHeaderTable" width="' + params.allWidth + '" cellspacing="0" cellpadding="0" border="0" style="width: ' +  params.allWidth + 'px"> \
                    ' + params.colgroup + params.theader + ' \
                  </table>\
                </div>'
    }
    return headerStr
  }
  // 输出模板函数
  Table.prototype.toTemplate = function(params) {
    var data = this.options.data
    var headerStr = this.toHeanderTemplate(params)
    var tmp = ''
    // var style = 'width: '+ this.width +'px; height: ' + this.height
    var noDataStyle = "display: none"
    var tbodyStyle = "display: block"
    if (data.length === 0) {
      noDataStyle = "display: block"
      tbodyStyle = "display: none"
    }
    var tmp = '<div class="hz-table"> \
          {{_headerTemplate}} \
          <div class="nodeData" style="'+ noDataStyle +'">暂时没有数据</div> \
					<div class="tbody" style="height: ' + this.height + ';'+ tbodyStyle +'"> \
						<table class="tableBody" width="' + params.allWidth + '" cellspacing="0" cellpadding="0" border="0" style="width: ' +  params.allWidth + 'px"> \
						' + params.colgroup + params.theader + params.tbody + '\
						</table>\
					</div>\
					{{_fixedRight}}\
			</div>'

    tmp = tmp.replace('{{_headerTemplate}}', headerStr)
    tmp = tmp.replace('{{_fixedRight}}', params.fixedRightStr)
    return tmp
  }
  // 创建滚动条
  Table.prototype.createScroll = function() {
    var el = this.$el.find('.hz-table')
    var tableBody = el.find('.tableBody')
    var tbody = el.find('.tbody')
    this.boxWidth = el.width()
    this.tableWidth = tableBody.width()
    this.boxHeight = el.height()
    this.tableHeight = tableBody.height()
    if (this.tableWidth > this.boxWidth) {
      var sw = this.boxWidth * (this.boxWidth / this.tableWidth)
      this.disScrollWidth = this.boxWidth - sw // 滚动条跟滚动盒子的距离

      // var scrollW = this.boxHeight

      // if (this.tableHeight > this.boxHeight) {
      //   this.isHscroll = true
      //   scrollW = this.boxWidth - 10
      //   sw = scrollW * (this.boxWidth / this.tableWidth)
      //   this.disScrollWidth = scrollW - sw
      //   tbody.css('height', tbody.height() - 10)
      // }

      var hScrollHtml = '<div class="hscroll"><div class="hscrollbar" style="width: '+ sw +'px"></div></div>'
      el.append(hScrollHtml)
    }
    if (this.tableHeight > this.boxHeight) {
      var sh = this.boxHeight * (this.boxHeight / this.tableHeight)
      this.isVscroll = true
      this.disScrollHeight = this.boxHeight - sh

      // if (this.tableWidth > this.boxWidth) {
      //   scrollH = this.boxHeight - 10
      //   sh = scrollH * (this.boxHeight / this.tableHeight)
      //   this.disScrollHeight = scrollH - sh
      //   // tbody.css('width', tbody.width() - 10)
      // }
      var vScrollHtml = '<div class="vscroll"><div class="vscrollbar" style="height: '+ sh +'px"></div></div>'
      el.append(vScrollHtml)
    }

  }

  Table.prototype.bind = function() {
    var self = this
    var el = this.$el
    var fixedHeaderTable = el.find('.fixedHeaderTable').eq(0)
    var tableBody = el.find('.tableBody')
    var trs = tableBody.find('tbody').find('tr')
    var fixedRightTable = el.find('.fixedRightTable').eq(0)
    var rightTrs = $(fixedRightTable).find('tbody').find('tr')
    var hzTable = $('.hz-table')

    // 滚动条
    var hscroll = el.find('.hscroll')
    var vscroll = el.find('.vscroll')
    var hscrollbar = el.find('.hscrollbar')
    var vscrollbar = el.find('.vscrollbar')
    var startX = 0
    var startY = 0
    var disx = 0
    var disy = 0
    var disHscroll = hscroll.width() - hscrollbar.width()

    if (this.isVscroll) {
      hzTable.on('mousewheel', function(event, delta) {
        event.stopPropagation()
        event.preventDefault()
        delta = delta > 0 ? 6 : -6
        if (Math.abs(event.deltaX) > Math.abs(event.deltaY)) {
          // disx = hscrollbar.position().left - delta
          // scrollXfn(disx)
        } else {
          disy = vscrollbar.position().top - delta
          scrollYfn(disy)
        }
      })
    }

    var mousemovefn = function (e) {
      disx = e.clientX - startX
      disx = hscrollbar.position().left +  disx
      scrollXfn(disx)
      startX = e.clientX 
    }

    var scrollXfn = function (disx) {
      if (disx <= 0) {
        disx = 0
      } else if (disx >= self.disScrollWidth) {
        disx = self.disScrollWidth
      }
      var tableLeft = (disx / self.boxWidth) * self.tableWidth
      hscrollbar.css('left', disx)
      tableBody.css('left', -tableLeft)
      if (self.options.headerFixed) {
        fixedHeaderTable.css('left', -tableLeft)
      }
    }

    var mouseupfn = function (e) {
      hscrollbar.removeClass('active')
      $(document).off('mousemove', mousemovefn)
      $(document).off('mouseup', mouseupfn)
    }

    hscrollbar.on('mousedown', function(e) {
      startX = e.clientX
      hscrollbar.addClass('active')
      $(document).on('mousemove', mousemovefn)
      $(document).on('mouseup', mouseupfn)
    })

    var scrollYfn = function (disy) {
      var sy = 0
      if (disy <= 0) {
        disy = 0
      } else if (disy >= self.disScrollHeight) {
        disy = self.disScrollHeight
      }

      // if (self.isHscroll) {
      //   var tableTop = (disy / (self.boxHeight - 10)) * self.tableHeight
      // } else {
      //   var tableTop = (disy / self.boxHeight) * self.tableHeight
      // }

      var tableTop = (disy / self.boxHeight) * self.tableHeight
      
      vscrollbar.css('top', disy)
      tableBody.css('top', -tableTop)
      if (self.fixedRightCol.length > 0) {
        fixedRightTable.css('top', -tableTop)
      }
    }

    var mousemoveYfn = function (e) {
      disy = e.clientY - startY
      disy = vscrollbar.position().top + disy
      scrollYfn(disy)
      startY = e.clientY 
    }

    var mouseupYfn = function (e) {
      vscrollbar.removeClass('active')
      $(document).off('mousemove', mousemoveYfn)
      $(document).off('mouseup', mouseupYfn)
    }

    vscrollbar.on('mousedown', function(e) {
      startY = e.clientY
      vscrollbar.addClass('active')
      $(document).on('mousemove', mousemoveYfn)
      $(document).on('mouseup', mouseupYfn)
    })


    $(trs).on('mouseenter', function(e) {
      rowEnter.call(this, rightTrs)
    })
    $(trs).on('mouseleave', function(e) {
      rowLeave.call(this, rightTrs)
    })
    $(rightTrs).on('mouseenter', function(e) {
      rowEnter.call(this, trs)
    })
    $(rightTrs).on('mouseleave', function(e) {
      rowLeave.call(this, trs)
    })

    hzTable.click(function(e) {
      var target = $(e.target)
      var type = target.data('type')
      var rowIndex = target.data('rowsindex')
      if (!rowIndex && typeof rowIndex !== 'number') {
        rowIndex = target.parent("td").data('rowsindex')
      }
      var obj = {}
      if (type === 'btn') {
        var btnText = $(e.target).data('btntext')
        obj.btnText = btnText
      }
      obj.row = self.options.data[rowIndex]
      hzTable.trigger('hz.table.click', obj)
    })
  }

  // 判断类型工具函数
  function toType(type) {
    return Object.prototype.toString.call(type).match(/\[object (\w+)\]/)[1]
  }
  // 拼接操作栏字符串函数
  function toOparations(arr, j) {
    var str = ''
    for (var i = 0; i < arr.length; i++) {
      str += '<a href="javascript:void(0)" data-type="btn" data-rowsIndex="' + j + '" data-btnText="' + arr[i] + '">' + arr[i] + '</a>'
    }
    return str
  }
  // 当鼠标移入tr的时候
  function rowEnter(els) {
    var _this = $(this)
    var index = _this.index()
    _this.addClass('rowActive')
    $(els).eq(index).addClass('rowActive')
  }
  // 当鼠标移出tr的时候
  function rowLeave(els) {
    var _this = $(this)
    var index = _this.index()
    _this.removeClass('rowActive')
    $(els).eq(index).removeClass('rowActive')
  }

  // 插件函数
  function Plugin(option) {
    var table = new Table(this, option)
    return table.init();
  }
  // 绑定到$上面
  $.fn.tableRender = Plugin
  $.fn.tableRender.Constructor = Table

}(jQuery);
(function ($) {
  "use strict";
  /*
  模糊搜索远程接口配置
  ajax:{
    requestData: {offset: 0, count: 10},
    requestPath: UserNameUrl,
    searchKey: 'customerName',
    requestMethod: 'GET',
  },
  */
  var defaultOptions = {
    fuzzySearch: false,
    data: null, //模糊搜索列表数组[{keyUnit:xx,valUnit:yy}]
    width: 500,
    placeholder: '',
    inputFontSize: 12, // 用于适配placeholder宽度
    keyUnit: '', //模糊搜索列表key
    valUnit: '', //模糊搜索列表value
    ajax: null,
    onAfterRemote: null,
    tagClass: function (item) {
      return 'label label-info';
    },
    focusClass: 'focus',
    itemValue: function (item) {
      return item ? item.toString() : item;
    },
    itemText: function (item) {
      return this.itemValue(item);
    },
    itemTitle: function (item) {
      return null;
    },
    itemAttrs: [],
    freeInput: true,
    addOnBlur: true,
    maxTags: undefined,
    maxChars: undefined,
    confirmKeys: [],  //确认添加，键盘按键码
    delimiter: ',',
    delimiterRegex: null,
    cancelConfirmKeysOnEmpty: false,
    onTagExists: function (item, $tag) {
      $tag.hide().fadeIn();
    },
    trimValue: false,
    allowDuplicates: false,
    triggerChange: true,
    attach: false,
    attachIcon: 'find iconfont icon-SearchBox',
    onAttachHandler: function () {

    },
    additionSelect: false,
    removeIcon: true,
  };

  /**
   * Constructor function
   */
  function TagsInput(element, options) {
    this.options = $.extend({}, defaultOptions, options);
    this.isInit = true;
    this.itemsArray = [];

    this.$element = $(element);
    this.$element.hide();

    this.isSelect = (element.tagName === 'SELECT');
    this.multiple = (this.isSelect && element.hasAttribute('multiple'));
    this.objectItems = this.options && this.options.itemValue;
    this.placeholderText = element.hasAttribute('placeholder') ? this.$element.attr('placeholder') : '';
    this.inputSize = Math.max(1, this.placeholderText.length);

    this.$container = $('<div class="bootstrap-tagsinput"></div>');
    this.$input = $('<input type="text" placeholder="' + this.placeholderText + '"/>').appendTo(this.$container);
    if (this.options && this.options.attach) {
      this.$attach = $('<div class="attach"><i class="' + String(this.options.attachIcon) + '"></i></div>')
    }
    else {
      this.$attach = ''
    }
    this.$tagsContainer = $('<div class="tags-container"></div>');

    this.$element.before(this.$container);
    this.$container.append(this.$attach)
    this.$container.append(this.$tagsContainer)

    this.build();
    this.isInit = false;
  }

  TagsInput.prototype = {
    constructor: TagsInput,

    /**
     * Adds the given item as a new tag. Pass true to dontPushVal to prevent
     * updating the elements val()
     */
    add: function (item, key, dontPushVal, options) {
      var self = this;

      if (self.options.maxTags && self.itemsArray.length >= self.options.maxTags)
        return;

      // Ignore falsey values, except false
      if (item !== false && !item)
        return;

      // Trim value
      if (typeof item === "string" && self.options.trimValue) {
        item = $.trim(item);
      }

      // Throw an error when trying to add an object while the itemValue option was not set
      if (typeof item === "object" && !self.objectItems)
        throw ("Can't add objects when itemValue option is not set");

      // Ignore strings only containg whitespace
      if (item.toString().match(/^\s*$/))
        return;

      // If SELECT but not multiple, remove current tag
      if (self.isSelect && !self.multiple && self.itemsArray.length > 0)
        self.remove(self.itemsArray[0]);

      if (typeof item === "string" && this.$element[0].tagName === 'INPUT') {
        var delimiter = (self.options.delimiterRegex) ? self.options.delimiterRegex : self.options.delimiter;
        var items = item.split(delimiter);
        if (items.length > 1) {
          for (var i = 0; i < items.length; i++) {
            this.add(items[i], true);
          }

          if (!dontPushVal)
            self.pushVal(self.options.triggerChange);
          return;
        }
      }

      var itemValue = self.options.itemValue(item),
        itemText = self.options.itemText(item),
        tagClass = self.options.tagClass(item),
        itemTitle = self.options.itemTitle(item);

      // Ignore items allready added
      var existing = $.grep(self.itemsArray, function (item) {
        return self.options.itemValue(item) === itemValue;
      })[0];
      if (existing && !self.options.allowDuplicates) {
        // Invoke onTagExists
        if (self.options.onTagExists) {
          var $existingTag = $(".tag", self.$container).filter(function () {
            return $(this).data("item") === existing;
          });
          self.options.onTagExists(item, $existingTag);
        }
        return;
      }

      // if length greater than limit
      if (self.items().toString().length + item.length + 1 > self.options.maxInputLength)
        return;

      // raise beforeItemAdd arg
      var beforeItemAddEvent = $.Event('beforeItemAdd', {
        item: item,
        cancel: false,
        options: options
      });
      self.$element.trigger(beforeItemAddEvent);
      if (beforeItemAddEvent.cancel)
        return;

      // register item in internal array and map
      self.itemsArray.push(item);

      var additionalAction = ''
      var removeIcon = ''
      // add a tag element
      if (self.options.additionSelect) {
        //额外选项列表
        var additionOptions = ''
        var optionsContainerWidth = parseInt(self.$container.css('width')) + 12 + 'px';
        if (self.options.additionSelectOptions && Array.isArray(self.options.additionSelectOptions)) {
          var data = self.options.additionSelectOptions
          var listHtml = '';
          for (let index = 0; index < data.length; index++) {
            const element = data[index];
            listHtml += '<li class="addition-options-li"><a key=' + element + '>' + element + '</a></li>'
          }
          additionOptions = '<div class="addition-options-container" style="width:' + optionsContainerWidth + '">' +
            '<ul class="addition-options-ul">' +
            listHtml +
            '</ul>' +
            '</div>';
        }
        additionalAction = '<span class="tag ' + htmlEncode(tagClass) + '"><span class="content"></span><span class="iconfont icon-dropdown-copy" data-role="additionSelect"></span>' + additionOptions + '</span>';
      }
      if (self.options.removeIcon) {
        removeIcon = '<span data-role="remove"></span>'
      }
      var $tag = $('<span class="tag ' + htmlEncode(tagClass) + (itemTitle !== null ? ('" title="' + itemTitle) : '') + '" data-key="' + key + '"><span class="content">' + htmlEncode(itemText) + '</span>' + removeIcon + '</span>' + additionalAction);
      $tag.data('item', item);
      self.findInputWrapper().before($tag);
      // self.$tagsContainer.append($tag)
      // self.$tagsContainer.append(self.findInputWrapper())
      $tag.after(' ');

      // Check to see if the tag exists in its raw or uri-encoded form
      var optionExists = (
        $('option[value="' + encodeURIComponent(itemValue) + '"]', self.$element).length ||
        $('option[value="' + htmlEncode(itemValue) + '"]', self.$element).length
      );

      // add <option /> if item represents a value not present in one of the <select />'s options
      if (self.isSelect && !optionExists) {
        var $option = $('<option selected>' + htmlEncode(itemText) + '</option>');
        $option.data('item', item);
        $option.attr('value', itemValue);
        self.$element.append($option);
      }

      if (!dontPushVal)
        self.pushVal(self.options.triggerChange);

      // Add class when reached maxTags
      if (self.options.maxTags === self.itemsArray.length || self.items().toString().length === self.options.maxInputLength)
        self.$container.addClass('bootstrap-tagsinput-max');

      // If using typeahead, once the tag has been added, clear the typeahead value so it does not stick around in the input.
      if ($('.typeahead, .twitter-typeahead', self.$container).length) {
        self.$input.typeahead('val', '');
      }

      if (this.isInit) {
        self.$element.trigger($.Event('itemAddedOnInit', {
          item: item,
          options: options
        }));
      } else {
        self.$element.trigger($.Event('itemAdded', {
          item: item,
          options: options
        }));
      }
      self.$input.focusin();
    },

    /**
     * Removes the given item. Pass true to dontPushVal to prevent updating the
     * elements val()
     */
    remove: function (item, dontPushVal, options) {
      var self = this;

      if (self.objectItems) {
        if (typeof item === "object")
          item = $.grep(self.itemsArray, function (other) {
            return self.options.itemValue(other) == self.options.itemValue(item);
          });
        else
          item = $.grep(self.itemsArray, function (other) {
            return self.options.itemValue(other) == item;
          });

        item = item[item.length - 1];
      }

      if (item) {
        var beforeItemRemoveEvent = $.Event('beforeItemRemove', {
          item: item,
          cancel: false,
          options: options
        });
        self.$element.trigger(beforeItemRemoveEvent);
        if (beforeItemRemoveEvent.cancel)
          return;

        $('.tag', self.$container).filter(function () {
          return $(this).data('item') === item;
        }).remove();
        $('option', self.$element).filter(function () {
          return $(this).data('item') === item;
        }).remove();
        if ($.inArray(item, self.itemsArray) !== -1)
          self.itemsArray.splice($.inArray(item, self.itemsArray), 1);
      }

      if (!dontPushVal)
        self.pushVal(self.options.triggerChange);

      // Remove class when reached maxTags
      if (self.options.maxTags > self.itemsArray.length)
        self.$container.removeClass('bootstrap-tagsinput-max');

      self.$element.trigger($.Event('itemRemoved', {
        item: item,
        options: options
      }));
    },

    /**
     * Removes all items
     */
    removeAll: function () {
      var self = this;

      $('.tag', self.$container).remove();
      $('option', self.$element).remove();

      while (self.itemsArray.length > 0)
        self.itemsArray.pop();

      self.pushVal(self.options.triggerChange);
    },

    /**
     * Refreshes the tags so they match the text/value of their corresponding
     * item.
     */
    refresh: function () {
      var self = this;
      $('.tag', self.$container).each(function () {
        var $tag = $(this),
          item = $tag.data('item'),
          itemValue = self.options.itemValue(item),
          itemText = self.options.itemText(item),
          tagClass = self.options.tagClass(item);

        // Update tag's class and inner text
        $tag.attr('class', null);
        $tag.addClass('tag ' + htmlEncode(tagClass));
        $tag.contents().filter(function () {
          return this.nodeType == 3;
        })[0].nodeValue = htmlEncode(itemText);

        if (self.isSelect) {
          var option = $('option', self.$element).filter(function () {
            return $(this).data('item') === item;
          });
          option.attr('value', itemValue);
        }
      });
    },

    /**
     * Returns the items added as tags
     */
    items: function () {
      return this.itemsArray;
    },

    /**
     * Assembly value by retrieving the value of each item, and set it on the
     * element.
     */
    pushVal: function () {
      var self = this,
        val = $.map(self.items(), function (item) {
          return self.options.itemValue(item).toString();
        });

      self.$element.val(val, true);

      if (self.options.triggerChange)
        self.$element.trigger('change');
    },

    /**
     * Initializes the tags input behaviour on the element
     */
    build: function () {
      var self = this;

      // When itemValue is set, freeInput should always be false
      if (self.objectItems)
        self.options.freeInput = false;

      makeOptionItemFunction(self.options, 'itemValue');
      makeOptionItemFunction(self.options, 'itemText');
      makeOptionFunction(self.options, 'tagClass');
      if (self.options.width) {
        self.$container.css('width', self.options.width + 'px');
      }
      if (self.$element.attr('disabled')) {
        self.$container.css('background-color', '#F5F5F5');
      }

      //模糊搜索
      if (self.options.fuzzySearch) {

        //更新模糊搜索列表方法
        var updateList = function (data) {
          var $optionsContainer = self.$container.find('.tagsinput-options-container');
          var listHtml = '';
          var itemAttrs = self.options.itemAttrs
          for (let index = 0; index < data.length; index++) {
            const element = data[index];
            var attrsStr = ''
            if (itemAttrs && Array.isArray(itemAttrs) && itemAttrs.length > 0) {
              for (let itemIndex = 0; itemIndex < itemAttrs.length; itemIndex++) {
                var key = itemAttrs[itemIndex]
                if (element[key]) {
                  attrsStr += ' data-' + key + '=' + String(element[key]).replace(/\s/g,'')
                }
              }
            }
            listHtml += '<li class="tagsinput-options-li"><a key=' + element[self.options.keyUnit] + attrsStr + '>' + element[self.options.valUnit] + '</a></li>'
          }
          listHtml = listHtml || '<li class="tagsinput-options-li"><a disabled>无匹配</a></li>';
          $optionsContainer.find('ul').empty().append(listHtml);
          $optionsContainer.show();
        };
        //服务端模糊搜索请求
        //(text, selfDefine) -> (关键字，允许添加自定义选项到列表头)
        var remoteRequest = function (text, selfDefine) {
          var ajaxObj = self.options.ajax;
          if (ajaxObj && ajaxObj.requestPath && ajaxObj.requestData) {
            ajaxObj.requestData[ajaxObj.searchKey] = text;
            $.ajax({
              url: ajaxObj.requestPath,
              method: ajaxObj.requestMethod,
              data: ajaxObj.requestData,
              success: function (data) {
                var filterOptionsData = typeof data === 'string' ? JSON.parse(data).payload : data.payload;
                if (selfDefine) {
                  filterOptionsData.unshift({ id: 'selfDefine', name: text })
                }
                if (self.options.onAfterRemote && typeof self.options.onAfterRemote == 'function') {
                  filterOptionsData = self.options.onAfterRemote(filterOptionsData)
                }
                updateList(filterOptionsData);
              },
              failed: function (err) {
                console.log(err)
              }
            })
          }
        };
        //初始化模糊搜索列表
        self.$input.remove()
        var optionsContainerWidth = parseInt(self.$container.css('width')) + 12 + 'px';
        var optionsContainer = '<div class="tagsinput-options-container" style="width:' + optionsContainerWidth + '">' +
          '<ul class="tagsinput-options-ul">' +
          '</ul>' +
          '</div>';
        var inputwrapperHtml = '<span class="input-wrapper">' +
          optionsContainer +
          '</span>'


        self.$input = $('<input type="text">').prependTo($(inputwrapperHtml));
        // self.$input.closest('.input-wrapper').appendTo(self.$container);
        self.$input.closest('.input-wrapper').appendTo(self.$tagsContainer);


        //监听input事件
        self.$container.on('input', 'input', $.proxy(function (event) {
          var $input = $(event.target);
          var text = $input.val();
          if (text.length == 0) {
            self.$container.find('.tagsinput-options-container ').hide();
          }
          else {
            //模糊搜索列表数组
            if (self.options.data) {
              var filterOptionsData = self.options.data.filter(function (item) {
                return item[self.options.valUnit].toLowerCase().indexOf(text.toLowerCase()) !== -1
              })
              if (filterOptionsData && filterOptionsData.length > 0) {
                updateList(filterOptionsData);
              }
              else {
                //模糊搜索调用远程ajax接口
                if (self.options.ajax) {
                  remoteRequest(text, true)
                }
              }
            }
            else if (self.options.ajax) {
              remoteRequest(text)
            }
          }
        }, self));

        self.$container.on('click', '.tagsinput-options-container', $.proxy(function (event) {
          var $item = $(event.target);
          var key = $(event.target).attr('key');
          if ($item.attr('disabled')) {
            return;
          }
          var itemAttrs = self.options.itemAttrs
          var attrsObj = {}
          if (itemAttrs && Array.isArray(itemAttrs) && itemAttrs.length > 0) {
            for (let itemIndex = 0; itemIndex < itemAttrs.length; itemIndex++) {
              var attrKey = itemAttrs[itemIndex]
              var attrValue = $item.data(attrKey)
              if (attrValue) {
                attrsObj[attrKey] = attrValue
              }
            }
          }
          self.add(Object.assign({ [self.options.keyUnit]: $item.attr('key'), [self.options.valUnit]: $item.text() }, attrsObj), true);
          self.$input.val('');
          $item.closest('.tagsinput-options-container').hide();
        }, self));

        if (self.options.attach && self.options.onAttachHandler && typeof self.options.onAttachHandler == 'function') {
          self.$attach.on('click', $.proxy(function (event) {
            self.options.onAttachHandler(self)
          }, self));

        }
        //点击空白处，隐藏模糊搜索列表，清空input
        $(document).on('click', function (event) {
          if (!$(event.target).is('.tagsinput-options-container a')) {
            self.$container.find('.tagsinput-options-container').hide();
            self.$input.val('');
          }
          var $additionOptions = self.$container.find('.addition-options-container')
          if ($additionOptions.is(':visible') && !$(event.target).is('.addition-options-container a')) {
            $additionOptions.hide();
          }
        })
      }
      //输入为空时，显示 placeholder
      if (self.options.placeholder) {
        self.$input.attr('placeholder', self.options.placeholder);
        self.$input.css('width', (self.options.placeholder.length * self.options.inputFontSize) + 'px');
        self.$container.on('focusin', 'input', function () {
          self.$input.attr('placeholder', '');
          self.$input.css('width', 'auto');
        })
        self.$container.on('focusout', 'input', function () {
          if (self.items().length == 0) {
            self.$input.attr('placeholder', self.options.placeholder);
            self.$input.css('width', (self.options.placeholder.length * self.options.inputFontSize) + 'px');
          }
        })
      }
      self.$container.on('click', $.proxy(function (event) {
        if (!self.$element.attr('disabled')) {
          self.$input.removeAttr('disabled');
        }
        self.$input.focus();
      }, self));

      // Toggle the 'focus' css class on the container when it has focus
      self.$container.on({
        focusin: function () {
          self.$container.addClass(self.options.focusClass);
        },
        focusout: function () {
          self.$container.removeClass(self.options.focusClass);
        },
      });

      self.$container.on('keydown', 'input', $.proxy(function (event) {
        var $input = $(event.target),
          $inputWrapper = self.findInputWrapper();

        if (self.$element.attr('disabled')) {
          self.$input.attr('disabled', 'disabled');
          return;
        }

        switch (event.which) {
          // BACKSPACE
          case 8:
            if (doGetCaretPosition($input[0]) === 0) {
              var prev = $inputWrapper.prev();
              if (prev.length) {
                self.remove(prev.data('item'));
              }
            }
            break;

          // DELETE
          case 46:
            if (doGetCaretPosition($input[0]) === 0) {
              var next = $inputWrapper.next();
              if (next.length) {
                self.remove(next.data('item'));
              }
            }
            break;
          default:
          // ignore
        }

        if (self.options.confirmKeys.indexOf(event.which) !== -1) {
          self.$tagsContainer.find('li:first-child a').trigger('click')
        }

        // Reset internal input's size
        var textLength = $input.val().length,
          wordSpace = Math.ceil(textLength / 5),
          size = textLength + wordSpace + 1;
        $input.attr('size', Math.max(this.inputSize, $input.val().length));
      }, self));

      self.$container.on('keypress', 'input', $.proxy(function (event) {
        var $input = $(event.target);

        if (self.$element.attr('disabled')) {
          self.$input.attr('disabled', 'disabled');
          return;
        }

        // Reset internal input's size
        var textLength = $input.val().length,
          wordSpace = Math.ceil(textLength / 5),
          size = textLength + wordSpace + 1;
        $input.attr('size', Math.max(this.inputSize, $input.val().length));
      }, self));

      // Remove icon clicked
      self.$container.on('click', '[data-role=remove]', $.proxy(function (event) {
        if (self.$element.attr('disabled')) {
          return;
        }
        self.remove($(event.target).closest('.tag').data('item'));
      }, self));

      // additionSelect icon clicked
      self.$container.on('click', '[data-role=additionSelect]', $.proxy(function (event) {
        event.stopPropagation();
        if (self.$element.attr('disabled')) {
          return;
        }
        var $additionOptions = self.$container.find('.addition-options-container');
        if ($additionOptions.is(':visible')) {
          $additionOptions.hide();
        }
        else {
          $additionOptions.show();
        }

      }, self));
      //额外选项列表点击事件
      self.$container.on('click', '.addition-options-container', $.proxy(function (event) {
        event.stopPropagation()
        var $item = $(event.target);
        var key = $(event.target).attr('key');
        if ($item.attr('disabled')) {
          return;
        }
        $(event.target).closest('.tag').find('.content').text(key);
        $item.closest('.addition-options-container').hide();
      }, self));

      // Only add existing value as tags when using strings as tags
      if (self.options.itemValue === defaultOptions.itemValue) {
        if (self.$element[0].tagName === 'INPUT') {
          self.add(self.$element.val());
        } else {
          $('option', self.$element).each(function () {
            self.add($(this).attr('value'), true);
          });
        }
      }
    },

    /**
     * Removes all tagsinput behaviour and unregsiter all event handlers
     */
    destroy: function () {
      var self = this;

      // Unbind events
      self.$container.off('keypress', 'input');
      self.$container.off('click', '[role=remove]');

      self.$container.remove();
      self.$element.removeData('tagsinput');
      self.$element.show();
    },

    /**
     * Sets focus on the tagsinput
     */
    focus: function () {
      this.$input.focus();
    },

    /**
     * Returns the internal input element
     */
    input: function () {
      return this.$input;
    },

    /**
     * Returns the element which is wrapped around the internal input. This
     * is normally the $container, but typeahead.js moves the $input element.
     */
    findInputWrapper: function () {
      var elt = this.$input[0],
        container = this.$tagsContainer[0];
      while (elt && elt.parentNode !== container)
        elt = elt.parentNode;

      return $(elt);
    }
  };

  /**
   * Register JQuery plugin
   */
  $.fn.tagsinput = function (arg1, arg2, arg3) {
    var results = [];

    this.each(function () {
      var tagsinput = $(this).data('tagsinput');
      // Initialize a new tags input
      if (!tagsinput) {
        tagsinput = new TagsInput(this, arg1);
        $(this).data('tagsinput', tagsinput);
        results.push(tagsinput);

        if (this.tagName === 'SELECT') {
          $('option', $(this)).attr('selected', 'selected');
        }

        // Init tags from $(this).val()
        $(this).val($(this).val());
      } else if (!arg1 && !arg2) {
        // tagsinput already exists
        // no function, trying to init
        results.push(tagsinput);
      } else if (tagsinput[arg1] !== undefined) {
        // Invoke function on existing tags input
        if (tagsinput[arg1].length === 3 && arg3 !== undefined) {
          var retVal = tagsinput[arg1](arg2, null, arg3);
        } else {
          var retVal = tagsinput[arg1](arg2);
        }
        if (retVal !== undefined)
          results.push(retVal);
      }
    });

    if (typeof arg1 == 'string') {
      // Return the results from the invoked function calls
      return results.length > 1 ? results : results[0];
    } else {
      return results;
    }
  };

  $.fn.tagsinput.Constructor = TagsInput;

  /**
   * Most options support both a string or number as well as a function as
   * option value. This function makes sure that the option with the given
   * key in the given options is wrapped in a function
   */
  function makeOptionItemFunction(options, key) {
    if (typeof options[key] !== 'function') {
      var propertyName = options[key];
      options[key] = function (item) {
        return item[propertyName];
      };
    }
  }

  function makeOptionFunction(options, key) {
    if (typeof options[key] !== 'function') {
      var value = options[key];
      options[key] = function () {
        return value;
      };
    }
  }
  /**
   * HtmlEncodes the given value
   */
  var htmlEncodeContainer = $('<div />');

  function htmlEncode(value) {
    if (value) {
      return htmlEncodeContainer.text(value).html();
    } else {
      return '';
    }
  }

  /**
   * Returns the position of the caret in the given input field
   * http://flightschool.acylt.com/devnotes/caret-position-woes/
   */
  function doGetCaretPosition(oField) {
    var iCaretPos = 0;
    if (document.selection) {
      oField.focus();
      var oSel = document.selection.createRange();
      oSel.moveStart('character', -oField.value.length);
      iCaretPos = oSel.text.length;
    } else if (oField.selectionStart || oField.selectionStart == '0') {
      iCaretPos = oField.selectionStart;
    }
    return (iCaretPos);
  }

  /**
   * Initialize tagsinput behaviour on inputs and selects which have
   * data-role=tagsinput
   */
  $(function () {
    $("input[data-role=tagsinput], select[multiple][data-role=tagsinput]").tagsinput();
  });
})(window.jQuery);
