## 表格组件

根据配置生产表格和固定列表格组件

### 基本用法

<style>
table{
  margin: 0
}
</style>

<div class="docDemo">
<div class="discript">

<div id="demo" style="width: 600px;"></div>

</div>
<div class="code">

````html

<div id="demo" style="width: 600px;"></div>

````

````js
var cols = [
  {
    label: '姓名',
    key: 'name',
    width: 300
  },
  {
    label: '年龄',
    key: 'age',
    width: 200,
    render: function (row) {
      return '<a href="#">姓名:' + row.name + '--年龄:' + row.age + '</a>'
    }
  },
  {
    label: '性别',
    key: 'sex',
    width: 100,
    click: (row) => {
      console.log(row)
    }
  },
  {
    label: '住址',
    width: 300,
    key: 'address',
  },
  {
    label: '操作',
    key: '',
    width: 100,
    type: 'link',
    text: ['编辑', '删除'],
    fixed: 'right'
  }
]
var tableData = Array(5).fill('').map((item, index) => ({
  name: '张三' + index,
  age: '2'+ index,
  sex: Math.random() > 0.5 ? '男' : '女',
  address: '深圳软件产业基地芒果大厦'
}))

var table = $('#demo').tableRender({
  width: 600,
  height: 300,
  headerFixed: true, // 是否开启头部固定
  cols: cols,
  data: tableData,
})
table.on('hz.table.click', function (e, row) {
  console.log(123)
  console.log(e, row)
})

````

</div>
</div>


### 参数说明

#### 表格参数说明

| 参数 | 说明 | 类型 | 可选值 | 默认值
| ------ | ------ | ------ | ------ | ------ |
| width | 表格宽度 | Number |  可选 | 父级节点宽度
| height | 表格高度 | Number |  可选 | 没有指定就按照行数来计算
| headerFixed | 是否固定头部 | Boolean |  可选 | false
| cols | 列配置 | Array |  必选 | []
| data | 数据 | Array |  可选 | []



<div style="padding: 20px 0"></div>

#### 表格cols列参数说明


| 参数 | 说明 | 类型 | 可选值 | 默认值
| ------ | ------ | ------ | ------ | ------ |
| label | 表头文字 | String |  必选 | 无
| key | 对应数据的key值 | String |  可选 | ''
| width | 列宽度 | Number |  必选 | 无
| render | 自定义表格内容 | Function |  可选 | render(row){}


<script>

var cols = [
  {
    label: '姓名',
    key: 'name',
    width: 300
  },
  {
    label: '年龄',
    key: 'age',
    width: 200,
    render: function (row) {
      return '<a href="#">姓名:' + row.name + '--年龄:' + row.age + '</a>'
    }
  },
  {
    label: '性别',
    key: 'sex',
    width: 100,
    click: (row) => {
      console.log(row)
    }
  },
  {
    label: '住址',
    width: 300,
    key: 'address',
  },
  {
    label: '操作',
    key: '',
    width: 100,
    type: 'link',
    text: ['编辑', '删除'],
    fixed: 'right'
  }
]
var tableData = Array(14).fill('').map((item, index) => ({
  name: '张三' + index,
  age: '2'+ index,
  sex: index % 2 == 0 ? '男' : '女',
  address: '深圳软件产业基地芒果大厦'
}))

var table = null

setTimeout(function () {
  table = $('#demo').tableRender({
    width: 600,
    height: 300,
    headerFixed: true, // 是否开启头部固定
    cols: cols,
    data: tableData,
  })
  table.on('hz.table.click', function (e, row) {
    console.log(123)
    console.log(e, row)
  })
})

$(document).on('click', 'a[data-nav="table"]', function () {
  setTimeout(function () {
    table = $('#demo').tableRender({
      width: 600,
      height: 300,
      headerFixed: true, // 是否开启头部固定
      cols: cols,
      data: tableData,
    })
    table.on('hz.table.click', function (e, row) {
      console.log(123)
      console.log(e, row)
    })
  })
})

</script>


